--The AVIO service
local getset = require "kiloview.getSetHelper"
local avioM = require "avio.avioModule"

local CONFIG_FILE = "/data/configs/avio.conf"

require "configuredb"
require "avio_config"

---------------------------------------------------
_G.avio = CONFIGS.InstanceOf( AVIO_Template, "avio" )
_G.AVIO_CONFIG_LOADING = true	--Lock to avoid trigger events
_G.avio:LoadFile( CONFIG_FILE )
_G.AVIO_CONFIG_LOADING = false
_G.avio:ResetChanges()
---------------------------------------------------

local _M = {
	avioModule = avioM,
	
	AVIO_Groups = nil,		--This is the AVIO_Groups
	AVIO_EachGroup = nil	--AVIO_EachGroup["00"]["Group"], AVIO_EachGroup["00"]["VFE"], AVIO_EachGroup["00"]["AFE"], etc.
}

---------------------------------------------------

local AVIOGroups_SVC = {
	Identifier = "AVIOGroups",
	FriendlyName = "Encoder AVIOGroups service",
	Version = "1.0",
	ConfigPrefix = "avio",
}

function AVIOGroups_SVC:Activate()
	--Nothing
end

function AVIOGroups_SVC:Start()
	--Nothing
end

function AVIOGroups_SVC:Stop()
	--Nothing
end

function AVIOGroups_SVC:Reset()
--TODO: Nothing
end

function AVIOGroups_SVC:Terminate()
--TODO: Nothing
end

function AVIOGroups_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	elseif action == "getAVIOGroups" then
		return self:GetAVIOGroups()
	else
		return -1, {}
	end
end

function AVIOGroups_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function AVIOGroups_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function AVIOGroups_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function AVIOGroups_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function AVIOGroups_SVC:RestoreDefault()
	--TODO:
	return 0
end

function AVIOGroups_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

function AVIOGroups_SVC:GetAVIOGroups()
	return 1, { ["00"] = "Group 00" }
end

----------------------------------------------------

local function ConstructGroup( idr )
	local GROUP_SVC = {
		Identifier = "AVIOGroup-" .. idr,
		FriendlyName = "Encoder AVIOGroup-" .. idr .. " service",
		
		WithVFE = true,
		WithAFE = true,
		WithVBE = false,
		WithABE = false,
		WithSwitch = false,
		WithLED = false,
	}

	function GROUP_SVC:Activate()
	--TODO: Nothing
	end

	function GROUP_SVC:Start()
	--TODO: Nothing
	end

	function GROUP_SVC:Stop()
	--TODO: Nothing
	end

	function GROUP_SVC:Reset()
	--TODO: Nothing
	end

	function GROUP_SVC:Terminate()
	--TODO: Nothing
	end

	function GROUP_SVC:Control(action,params)
		if action == "activate" then
			self:Activate()
			return 0, {}
		elseif action == "start" then
			self:Start()
			return 0, {}
		elseif action == "stop" then
			self:Stop()
			return 0, {}
		elseif action == "Reset" then
			self:Reset()
			return 0, {}
		elseif action == "terminate" then
			self:Terminate()
			return 0, {}
		else
			return -1, {}
		end
	end

	return GROUP_SVC
end

----------------------------------------------------

local function ConstructAVEnd( idr, flag, metaOps )
	local AVEND_SVC = {
		Identifier = flag .. "-" .. idr,
		FriendlyName = "Encoder " .. flag .. "-" .. idr .. " service",
	}

	function AVEND_SVC:Activate()
	--TODO: Nothing
	end

	function AVEND_SVC:Start()
	--TODO: Nothing
	end

	function AVEND_SVC:Stop()
	--TODO: Nothing
	end

	function AVEND_SVC:Reset()
	--TODO: Nothing
	end

	function AVEND_SVC:Terminate()
	--TODO: Nothing
	end

	function AVEND_SVC:Control(action,params)
		if action == "activate" then
			self:Activate()
			return 0, {}
		elseif action == "start" then
			self:Start()
			return 0, {}
		elseif action == "stop" then
			self:Stop()
			return 0, {}
		elseif action == "Reset" then
			self:Reset()
			return 0, {}
		elseif action == "terminate" then
			self:Terminate()
			return 0, {}
		elseif action == "get-sources" then
			return self:GetSources()
		elseif action == "get-targets" then
			return self:GetTargets()
		elseif action == "select-source" then
			self:SelectSource( params and params["source"] or "" )
			return 0, {}
		elseif action == "select-target" then
			self:SelectTarget( params and params["target"] or "" )
			return 0, {}
		elseif action == "detect-source" then
			return self:DetectSourceParameters( params and params["source"] or "", params or {} )
		elseif action == "detect-target" then
			return self:DetectTargetParameters( params and params["target"] or "", params or {} )
		elseif action == "config-source" then
			self:SetSourceParameters( params and params["source"] or "", params or {} )
			return 0, {}
		elseif action == "config-target" then
			self:SetTargetParameters( params and params["source"] or "", params or {} )
			return 0, {}
		else
			return -1, {}
		end
	end

	return setmetatable( AVEND_SVC, metaOps )
end

----------------------------------------------------
--*** VFE operations ***
----------------------------------------------------

local vfe_ops = {}
vfe_ops.__index = vfe_ops

-----------------------------

function vfe_ops:GetSources()
	if type(_M.avioModule.video.supportSources) ~= "table" then
		return 1, {["DEFAULT"] = "Default Video Source"}
	else
		local k,v
		local counter = 0
		for k,v in pairs(_M.avioModule.video.supportSources) do
			counter = counter + 1
		end
		return counter, _M.avioModule.video.supportSources
	end
end

-----------------------------

function vfe_ops:GetTargets()
	if type(_M.avioModule.video.supportTargets) ~= "table" then
		return 1, {["DEFAULT"] = "Default Video Target"}
	else
		local k,v
		local counter = 0
		for k,v in pairs(_M.avioModule.video.supportTargets) do
			counter = counter + 1
		end
		return counter, _M.avioModule.video.supportTargets
	end
end

-----------------------------

function vfe_ops:SelectSource(src)
	src = src:upper()
	getset.SetConfigureValues( "avio", { ["video.source"] = src } )	--Change the configure, and it can auto effect the source selection
end

-----------------------------

function vfe_ops:SelectTarget(target)
	--Nothing to do
end

function vfe_ops:VGAGetOptionFormats(formatCode)
--For VGA applications: Get optional VGA formats while format detection.
	local items = _M.avioModule:VGAGetOptionFormats( formatCode )
	if not items then
		return -1, {}
	else
		local i
		local r = {}
		for i=1, #items do
			r[ items[i].formatName .. " [#" .. items[i]._SEQ .. "]" ] = items[i].modeline
		end
		return #items, r
	end
end

function vfe_ops:VGASetSpecFormat(formatCode, modeline)
--For VGA applications: Set current special format
	_M.avioModule:VGASetSpecFormat( formatCode, modeline )
end

function vfe_ops:VGAPicAdjustment( formatCode, shiftX, shiftY, phase )
--For VGA applications: Adjust the X/Y shift for picture
	_M.avioModule:VGAPicAdjustment( formatCode, shiftX, shiftY, phase )
end

function vfe_ops:VGAGetPicAdjustment( formatCode )
	local r = _M.avioModule:VGAGetPicAdjustment( formatCode )
	if not r then
		return -1, {}
	else
		return 3, { format = formatCode, shiftX = r.X, shiftY = r.Y, phase = r.P or 16 }
	end
end

function vfe_ops:VGASetModelines( modelines )
--Set VGA user modelines
	_M.avioModule:VGASetModelines( modelines )
end

function vfe_ops:VGAGetModelines()
--Get VGA user modelines
	local r = _M.avioModule:VGAGetModelines()
	if not r then
		return -1, {}
	else
		return 1, { modelines = r }
	end
end

-----------------------------

local RATE_MODE_NAMES = {
	[0] = "SD",
	[1] = "HD",
	[2] = "3G",
	[3] = "VGA"
}

-----------------------------

function vfe_ops:DetectSourceParameters(src,options)
	if (src == "VGA" or src == "vga") and type(options) == "table" then	--Check something VGA specialized parameters
		if options["special"] == "OPTIONAL-FORMATS" then
			return self:VGAGetOptionFormats( options["format_code"] )
		elseif options["special"] == "PICTURE-ADJUSTMENT" then
			return self:VGAGetPicAdjustment( options["format_code"] )
		elseif options["special"] == "MODELINES" then
			return self:VGAGetModelines()
		end
	end

	if _M.avioModule then
		local result = {}
		local video = _M.avioModule.video
		result["video.source_detected"] = video.sourceDetected and 1 or 0
		result["video.source"] = video.currentSource or "UNKNOWN"
		if video.format then
			result["video.locked"] = video.format.signal_locked and 1 or 0
			result["video.freerun"] = video.format.free_run and 1 or 0
			result["video.interlaced"] = video.format.interlaced or 0
			result["video.format"] = video.format.format_name or "UNKNOWN"	--TODO:
			result["video.format_name"] = video.format.format_name or "UNKNOWN"
			--TODO: A format code is needed.
			result["video.format_code"] = video.format.format_code or "UNKNOWN"
			result["video.format_seq"] = video.format.format_seq or 0
			-- Source cropped? Spec Output? Output cropped?
			
			if video.format.__formatInfo and video.format._formatInfo.rateMode then
				result["video.standard"] = RATE_MODE_NAMES[video.format._formatInfo.rateMode] or "HD"
			else
				result["video.standard"] = "HD"
			end
			result["video.width"] = video.format.cap_cap_w or 0
			result["video.height"] = video.format.cap_cap_h or 0
			if video.format.frame_rate_value then
				result["video.frame_rate"] = video.format.frame_rate_value
			elseif video.format.frame_rate and video.format.frame_rate > 0 then
				result["video.frame_rate"] = math.floor( 512000000/video.format.frame_rate )
			else
				result["video.frame_rate"] = 0	--Unknown
			end
			
			return 8, result
		else
			result["video.locked"] = 0
			result["video.freerun"] = 0
			result["video.interlaced"] = 0
			result["video.format"] = "UNKNOWN"
			result["video.format_name"] = "UNKNOWN"
			result["video.format_code"] = "UNKNOWN"
			result["video.format_seq"] = 0
			result["video.standard"] = "hd"	--TODO:
			result["video.width"] = 0
			result["video.height"] = 0
			result["video.frame_rate"] = 0
			return 10, result
		end
	else
		return -1, {}
	end
end
-----------------------------
function vfe_ops:DetectTargetParameters(target,options)
	return 0, {}
end
-----------------------------
function vfe_ops:SetSourceParameters(src,options)
	local k,v
	local params = {}
	local counter = 0
	
	if (src == "VGA" or src == "vga") and type(options) == "table" and options["special"] then	--Something VGA speicalized.
		if options["special"] == "SPEC-FORMAT" then
			self:VGASetSpecFormat( options["format_code"] or "", options["modeline"] or "" )
		elseif options["special"] == "PICTURE-ADJUSTMENT" then
			self:VGAPicAdjustment( options["format_code"] or "",
										  options["shiftX"] or 0,
										  options["shiftY"] or 0,
										  options["phase"] )			
		elseif options["special"] == "MODELINES" then
			self:VGASetModelines( options["modelines"] or "" )
		end
		
		return
	end
	
	for k,v in pairs(options) do
		if k == "brightness" then
			params[ "video.brightness" ] = tonumber(v)
			counter = counter + 1
		elseif k == "contrast" then
			params[ "video.contrast" ] = tonumber(v)
			counter = counter + 1
		elseif k == "saturation" then
			params[ "video.saturation" ] = tonumber(v)
			counter = counter + 1
		elseif k == "hue" then
			params[ "video.hue" ] = tonumber(v)
			counter = counter + 1
		end
	end
	if counter > 0 then
		getset.SetConfigureValues( "avio", params )
	end
end
-----------------------------
function vfe_ops:SetTargetParameters(target,options)
	--Nothing
end

----------------------------------------------------
--*** AFE operations ***
----------------------------------------------------

local afe_ops = {}
afe_ops.__index = afe_ops
-----------------------------
function afe_ops:GetSources()
	if type(_M.avioModule.audio.supportSources) ~= "table" then
		return 1, {["DEFAULT"] = "Default Audio Source"}
	else
		local k,v
		local counter = 0
		for k,v in pairs(_M.avioModule.audio.supportSources) do
			counter = counter + 1
		end
		return counter, _M.avioModule.audio.supportSources
	end
end
-----------------------------
function afe_ops:GetTargets()
	if type(_M.avioModule.audio.supportTargets) ~= "table" then
		return 1, {["DEFAULT"] = "Default Audio Target"}
	else
		local k,v
		local counter = 0
		for k,v in pairs(_M.avioModule.audio.supportTargets) do
			counter = counter + 1
		end
		return counter, _M.avioModule.audio.supportTargets
	end
end
-----------------------------
function afe_ops:SelectSource(src)
	src = src:upper()
	getset.SetConfigureValues( "avio", { ["audio.source"] = src } )	--Change the configure, and it can auto effect the source selection

end
-----------------------------
function afe_ops:SelectTarget(target)
	--Nothing
end
-----------------------------
function afe_ops:DetectSourceParameters(src,options)
	if _M.avioModule then
		local result = {}
		local audio = _M.avioModule.audio
		result["audio.source"] = audio.currentSource or "UNKNOWN"
		if audio.format then
			result["audio.locked"] = audio.format.signal_locked and 1 or 0
			result["audio.sampling"] = audio.format.sampling or 0
			result["audio.sample_size"] = audio.format.sample_size or 0
			result["audio.sample_width"] = audio.format.sample_width or 16
			result["audio.channels"] = audio.format.channels or 2
		else
			result["audio.locked"] = 0
			result["audio.sampling"] = 0
			result["audio.sample_size"] = 0
			result["audio.sample_width"] = 16
			result["audio.channels"] = 2
		end
		return 6, result
	else
		return -1, {}
	end
end
-----------------------------
function afe_ops:DetectTargetParameters(target,options)
	return 0, {}
end
-----------------------------
function afe_ops:SetSourceParameters(src,options)
	local k,v
	local params = {}
	local counter = 0
	for k,v in pairs(options) do
		if k == "gain" then
			params[ "audio.gain" ] = tonumber(v)
			counter = counter + 1
		end
	end
	if counter > 0 then
		getset.SetConfigureValues( "avio", params )
	end
end

-----------------------------

function afe_ops:SetTargetParameters(target,options)
end

----------------------------------------------------

function _M.Init( cosvc )
	--Initialize the AVIO module
	_M.avioModule:Init()
	
	_M.avioModule:SelectVideoSource( _G.avio.video.source or "DEFAULT" )
	_M.avioModule:SetVideoParameter( "brightness", _G.avio.video.brightness or 128 )
	_M.avioModule:SetVideoParameter( "contrast", _G.avio.video.contrast or 128 )
	_M.avioModule:SetVideoParameter( "saturation", _G.avio.video.saturation or 128 )
	_M.avioModule:SetVideoParameter( "hue", _G.avio.video.hue or 128 )
	
	_M.SetSourceCrop( _G.avio.video.source_crop.enable ~= 0, 
									_G.avio.video.source_crop.cropX,
									_G.avio.video.source_crop.cropY,
									_G.avio.video.source_crop.cropW,
									_G.avio.video.source_crop.cropH )	--Initialize the cropping.
								
	_M.SetSpecOut( _G.avio.video.spec_out.format, _G.avio.video.spec_out.mode )
	
	_M.SetOutCrop( _G.avio.video.out_crop.enable ~= 0,
								_G.avio.video.out_crop.cropX,
								_G.avio.video.out_crop.cropY,
								_G.avio.video.out_crop.cropW,
								_G.avio.video.out_crop.cropH )	--Initialize the cropping.
	
	_M.SetOutRMF( _G.avio.video.RMF )
	
	_M.avioModule:SelectAudioSource( _G.avio.audio.source or "DEFAULT" )
	
	--NOTE: a bug fixing here: The alsa audio driver is a bit strange that left/right channel gain
	--      is not same at initialization time. So, I must change it and adjust it back to suite this!
	_M.avioModule:SetAudioParameter( "gain", (_G.avio.audio.line_gain or 0) + 3, "LINE" )
	
	_M.avioModule:SetAudioParameter( "gain", _G.avio.audio.line_gain or 0, "LINE" )
	_M.avioModule:SetAudioParameter( "gain", _G.avio.audio.mic_gain or 0, "MIC" )
	_M.avioModule:SetAudioParameter( "line_bypass", _G.avio.audio.line_bypass or 0, "LINE" )

	if cosvc then
		cosvc:ExposeObject( "/AVIO", AVIOGroups_SVC )
	end

	_M.AVIO_Groups = AVIOGroups_SVC
	print( "AVIO groups service online." )

	local grp_svc = ConstructGroup( "00" )
	if not grp_svc then
		error( "Could not create a AVIO group for '00'" )
	end
	_M.AVIO_EachGroup = {}
	_M.AVIO_EachGroup["00"] = { Group = grp_svc }
	
	if cosvc then
		cosvc:ExposeObject( "/AVIO/00", grp_svc )
	end
	print( "AVIO group-00 service online." )

	local vfe_svc = ConstructAVEnd( "00", "VFE", vfe_ops )
	if not vfe_svc then
		error( "Could not create VFE for group '00'" )
	end
	
	_M.AVIO_EachGroup["00"]["VFE"] = vfe_svc
	if cosvc then
		cosvc:ExposeObject( "/AVIO/00/VFE", vfe_svc )
	end
	print( "AVIO group-00 VFE service online." )

	local afe_svc = ConstructAVEnd( "00", "AFE", afe_ops )
	if not afe_svc then
		error( "Could not create AFE for group '00'" )
	end
	
	_M.AVIO_EachGroup["00"]["AFE"] = afe_svc
	if cosvc then
		cosvc:ExposeObject( "/AVIO/00/AFE", afe_svc )
	end
	print( "AVIO group-00 AFE service online." )
	_M.Check()
end

---- Each second checking

function _M.Check()
	_M.avioModule:Check()
end

-----------------------------
function _M.UpdateVideoParameter( pmName, pmValue )
	if pmName == "source" then
		_M.avioModule:SelectVideoSource( pmValue )
	elseif pmName == "brightness" or pmName == "contrast" or pmName == "saturation" or pmName == "hue" then
		_M.avioModule:SetVideoParameter( pmName, pmValue )
	end
end
-----------------------------
function _M.UpdateAudioParameter( pmName, pmValue )
	if pmName == "source" then
		_M.avioModule:SelectAudioSource( pmValue )
	elseif pmName == "line_gain" then
		_M.avioModule:SetAudioParameter( "gain", pmValue, "LINE" )
	elseif pmName == "mic_gain" then
		_M.avioModule:SetAudioParameter( "gain", pmValue, "MIC" )
	elseif pmName == "line_bypass" then
		_M.avioModule:SetAudioParameter( "line_bypass", pmValue, "LINE" )
	end
end
-----------------------------
function _M.SetSourceCrop( en, x, y, w, h )
	-- 向下取偶数，奇数不能用作裁切
	x = x%2==0 and x or x-1
	y = y%2==0 and y or y-1
	w = w%2==0 and w or w-1
	h = h%2==0 and h or h-1
	_M.avioModule:ConvSetSourceCrop( en, x, y, w, h )
end
-----------------------------
function _M.SetSpecOut( format, mode )
	_M.avioModule:ConvSetSpecOut( format, mode )
end
-----------------------------
function _M.SetOutCrop( en,x,y,w,h )
	x = x%2==0 and x or x-1
	y = y%2==0 and y or y-1
	w = w%2==0 and w or w-1
	h = h%2==0 and h or h-1
	_M.avioModule:SetOutputCrop( en, x, y, w, h )
end
-----------------------------
function _M.SetOutRMF( rmf )
	_M.avioModule:SetOutputRotateMirrorFlip( rmf )
end
-----------------------------

return _M

