package.path = package.path .. ";./avio/?.lua;./chips/?.lua;./avio/chips/?.lua"

local CONFIG_LOC = "/data/configs"
local STEP_SWITCH_SOURCE_TIMES = 5	--How many times 'Step' called to switch next source for source detection
--That means total '<Step interval> * STEP_SWITCH_SOURCE_TIMES' ms to switch next source
require "md3xx"
require "features"	--What device current configured?
require "avinBridge"

if not _G.DEVICE_TYPE then	--Assume a default type G2
	_G.DEVICE_TYPE = "G2"
end

local avfeType = "avfe_" .. _G.DEVICE_TYPE
local avfe = require( avfeType )
if not avfe then
	print( "ERROR: Could not initialize the A/V front end of product type '" .. _G.DEVICE_TYPE .. "'!" )
	return nil
end

local converter = nil
local convensionRules = nil
--if _G.ENABLE_CONVERTER then
	--require "md3xx"
	converter = MD3XX.createNew( "/dev/i2c-0", 0xdc )
	if not converter then
		print( "WARNING: The converter of this device could not be initialized, the convension functions are disabled!" )
	else
		convensionRules = require "md3xxConv"
		convensionRules.Init()	--Initialize( that open source crop and convension output settings )
		
		--For OEM blue showing.
		if _G.OEM_BLUE_PATTERN then
			converter:SetBluePattern( _G.OEM_BLUE_PATTERN )
			converter:SetBluePatternColor( _G.OEM_BLUE_COLOR_R or 0, _G.OEM_BLUE_COLOR_G or 0, _G.OEM_BLUE_COLOR_B or 230, 255 )
		end

		converter:ShowBlue()
		converter:SetDestFormat( VIDOUT_1920x1080p25 )
		converter:Care()	--Care it first, do some initializations.
	end
--end

avfe:Init()	--Initialize the A/V front end

-------------------------------------------------------------------------

local _M = {
	converter = converter,
	convensionRules = convensionRules,
	lowerDriverReady = false,	--Mark the vin driver is initialized
	vinFormatBridge = nil,
	ainFormatBridge = nil,
	
	video = {
		supportSources = { ["DEFAULT"] = "Default video input" },
		supportTargets = { ["DEFAULT"] = "Default digital target" },
		currentSource = "UNKNOWN",
		sourceDetected = false,
		sourceChanged = false,
		
		version = nil,
		format = nil,
		convFormat = nil,
		formatChanged = false,
		
		specOutCrop = nil,
		rmf = "0",
	},
	
	audio = {
		supportSources = { ["DEFAULT"] = "Default audio input" },
		supportTargets = { ["DEFAULT"] = "Default digital audio target" },
		currentSource = "UNKNOWN",
		sourceDetected = false,
		sourceChanged = false,
		
		version = nil,
		format = nil,
		formatChanged = false,
	},
	
}

-------------------------------------------------------------------------
local counter = 1
-------------------------------------------------------------------------
local detectSequence = { "video", "audio" }
-------------------------------------------------------------------------
function _M:UpdateSpecCropAndRMF(videoFormat)
	if self.video.specOutCrop then
		videoFormat.out_crop = true
		
		if self.video.specOutCrop.cropX < 0 or self.video.specOutCrop.cropX > videoFormat.cap_cap_w then
			videoFormat.out_crop_x = 0
		else
			videoFormat.out_crop_x = self.video.specOutCrop.cropX & 0xfffffffe
		end
		
		if self.video.specOutCrop.cropY < 0 or self.video.specOutCrop.cropY > videoFormat.cap_cap_h then
			videoFormat.out_crop_y = 0
		else
			videoFormat.out_crop_y = self.video.specOutCrop.cropY & 0xfffffffe
		end
		
		if self.video.specOutCrop.cropW < 240 then	--NOTE: Smallest size is 240x240
			videoFormat.out_crop_width = 240
		elseif self.video.specOutCrop.cropW  > videoFormat.cap_cap_w then
			videoFormat.out_crop_width = videoFormat.cap_cap_w
		else
			videoFormat.out_crop_width = self.video.specOutCrop.cropW & 0xfffffffe
		end
		
		if self.video.specOutCrop.cropH < 240 then	--NOTE: Smallest size is 240x240
			videoFormat.out_crop_height = 240
		elseif self.video.specOutCrop.cropH  > videoFormat.cap_cap_h then
			videoFormat.out_crop_height = videoFormat.cap_cap_h
		else
			videoFormat.out_crop_height = self.video.specOutCrop.cropH & 0xfffffffe
		end
	end
	
	videoFormat.rmf = self.video.rmf
end
-------------------------------------------------------------------------
local function UpdateSignalOSD(nosignal)
	if _G.osd then
		local i, j
		CONFIGS.BeginUpdate()
		for i=0,2 do
			for j=0,15 do
				--if _G.osd.stream[i].items[j].osd_type ~= "none" and _G.osd.stream[i].items[j].show_when ~= "always" then
					-- --if _G.osd.stream[i].items[j].show_when == "nosignal" then
					-- --	_G.osd.stream[i].items[j].showing = nosignal and 1 or 0
					-- --else
				_G.osd.stream[i].items[j].showing = nosignal and 0 or 1
					-- --end
				--end
			end -- for items
		end -- for stream
		CONFIGS.Submit()
	end --if
end

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local socket = require "cosocket.core"
client.InitSocket(socket, adapter)
local function GetProxy(objPath, service, loc, timeout)
	local cli = client.New( service, loc, nil, timeout or 30 )
	if not cli then
		print("** ERROR, " .. "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return
	end

	local proxy = cli:CreateProxy(objPath, true)
	if not proxy then
		print("** ERROR, " .. "Fail to create object proxy for '" .. (objPath or "unknown"))
		return
	end
	return proxy
end

local function set_led(status)
	local proxy = GetProxy("/", "ioled")
	if proxy then
		local is_ok, skip = proxy:Control("led_uctrl", {id = "signal", status = status})
		if not is_ok or not skip then
				print("** ERROR, Failed to set led status.")
		end
		proxy:Destroy()
	end
end

function _M:Check()
	avfe:StepCheck( counter%STEP_SWITCH_SOURCE_TIMES == 0 )
	
	--Check detection
	local x
	for x=1, #detectSequence do
		local detected, currentSource = avfe:ReportDetectingStatus( detectSequence[x] )
		local detectChanged = false
		--Source detection check
		if detected ~= self[detectSequence[x]].sourceDetected then
			self[detectSequence[x]].sourceDetected= detected
			detectChanged = true
		end

		if currentSource ~= self[detectSequence[x]].currentSource then
			self[detectSequence[x]].currentSource = currentSource
			detectChanged = true
		end

		if detectChanged then
			print( detected and "SOURCE DETECTED:" or "DETECTING:", detectSequence[x], currentSource or self.video.currentSource )
			self[detectSequence[x]].sourceChanged = true
		end
	end --For each media detecting
	
	if self.video.sourceDetected then	--Video source ensured.
		local versionCode, mformat = avfe:ReportVideoFormat( self.video.version )
		if versionCode ~=  self.video.version then	--Format changed!
			self.video.version = versionCode
			self.video.format = mformat
			self.video.formatChanged = true
			
			if converter then
				self.video.convFormat = self:UpdateConvension( self.video.format )
				if self.video.convFormat then
					self:UpdateSpecCropAndRMF( self.video.convFormat )	--If user specified output crop, then update it.
					if self.vinFormatBridge then
						self.vinFormatBridge:UpdateArgs( self.video.convFormat )
						self.lowerDriverReady = true
						--print( "DEBUG: Updated video format for lower driver, lower driver is ready." )
						--os.execute( "/bin/cat /proc/avin/0/vformat.cfg" )
					end

					if not self.video.formatChanged then
						self.video.formatChanged = true   
						--Note:这里获取各种信息需要一点时间，有可能这个变量已经被encoder中的Check修改了，但是裁切之类的信息没有更新，没有进行重新编码。
					end
					
					print( "Convension DONE." )
				else
					print( "ERROR: Could not convert source format by converter, this may work wrong!" )
				end
			else
				self:UpdateSpecCropAndRMF( self.video.format )	--If user specified output crop, then update it.
				if self.vinFormatBridge then
					self.vinFormatBridge:UpdateArgs( self.video.format )
					self.lowerDriverReady = true
					--print( "DEBUG: Updated video format for lower driver, lower driver is ready." )
					--os.execute( "/bin/cat /proc/avin/0/vformat.cfg" )
				end
			end
		end
	else
		if self.video.format then
			self.video.format.signal_locked = false
		end
	end
	
	if self.video.convFormat == nil and self.converter and self.convensionRules then	--Assume a default output by converter
		self.video.convFormat = self:UpdateConvension(nil)
		if self.video.convFormat then
			self:UpdateSpecCropAndRMF( self.video.convFormat )	--If user specified output crop, then update it.
			if self.vinFormatBridge then
				self.vinFormatBridge:UpdateArgs( self.video.convFormat )
				self.lowerDriverReady = true
				--print( "DEBUG: Updated video format for lower driver, lower driver is ready." )
				--os.execute( "/bin/cat /proc/avin/0/vformat.cfg" )
			end
			print( "Converter output a default blue screen." )
		else
			print( "ERROR: Converter can't output a default blue screen!" )
		end
	end

	if self.audio.sourceDetected then	--Current source ensured.
		local versionCode, mformat = avfe:ReportAudioFormat( self.audio.version )
		if versionCode ~= self.audio.version then	--Format changed
			self.audio.version = versionCode
			self.audio.format = mformat
			self.audio.formatChanged = true
			
			print( "Audio format changed:", versionCode )	--TODO:
			
			if self.ainFormatBridge then
				self.ainFormatBridge:UpdateArgs( self.audio.format )
			end
		end
	else
		if self.audio.format then
			self.audio.format.signal_locked = false
		end
	end
	
	counter = counter + 1
	if counter > STEP_SWITCH_SOURCE_TIMES then
		counter = 1
		collectgarbage()		--collect garbage
	end
end	--End of Check(...)

------------------------------------------------------------------------

function _M:CalcConvensionFormatInfo( convRslt, srcFormatName, signal_locked )
	local videoFormat = {}
	videoFormat.video_src = "CONVERTER"
	--TODO:
	videoFormat.cap_start_x = 0
	videoFormat.cap_start_y = 0
	videoFormat.signal_locked = signal_locked
	videoFormat.cap_cap_w = convRslt.vout_format.width
	videoFormat.line_width = convRslt.vout_format.width
	
	videoFormat.total_width = convRslt.vout_format.width	--It's wrong, but don't care this
	videoFormat.cap_cap_h = convRslt.vout_format.height
	videoFormat.total_height = convRslt.vout_format.height --It's wrong, but don't care this
	
	if convRslt.vout_format.interlaced then
		videoFormat.f0_height = convRslt.vout_format.height/2
		videoFormat.f1_height = convRslt.vout_format.height/2
	else
		videoFormat.f0_height = convRslt.vout_format.height
		videoFormat.f1_height = convRslt.vout_format.height
	end

	--TODO:
	videoFormat.sync_start = 0
	
	videoFormat.video_system = _G.AMBA_VIDEO_SYSTEM_AUTO or 0x0
	
	if AMBA_VIDEO_FPS then
		videoFormat.frame_rate = AMBA_VIDEO_FPS(convRslt.vout_format.frameRate)
	else
		videoFormat.frame_rate = AMBA_VIDEO_FPS_60 or convRslt.vout_format.frameRate
	end
	
	videoFormat.frame_rate_value = convRslt.vout_format.frameRate
	
	if convRslt.vout_format.aspect_ratio then
		videoFormat.aspect_ratio = convRslt.vout_format.aspect_ratio
	else
		videoFormat.aspect_ratio = _G.AMBA_VIDEO_RATIO_AUTO or 0
	end
	
	if convRslt.vout_format.yuv8bit then
		videoFormat.input_type = _G.AMBA_VIDEO_TYPE_YUV_656 or 2
		videoFormat.bit_resolution = 8
	else
		videoFormat.input_type = _G.AMBA_VIDEO_TYPE_YUV_656 or 2
		videoFormat.bit_resolution = 16
	end
	
	if convRslt.vout_format.interlaced then
		videoFormat.video_format = _G.AMBA_VIDEO_FORMAT_INTERLACE or 1
		videoFormat.input_format = _G.AMBA_VIN_INPUT_FORMAT_YUV_422_INTLC or 1
		videoFormat.interlaced = 1
	else
		videoFormat.video_format = _G.AMBA_VIDEO_FORMAT_PROGRESSIVE or 2
		videoFormat.input_format = _G.AMBA_VIN_INPUT_FORMAT_YUV_422_PROG or 2
		videoFormat.interlaced = 0
	end
	
	--TODO:
	videoFormat.hs_front_porch = 0
	videoFormat.hs_pulse_width = 0
	videoFormat.hs_back_porch = 0
	--TODO:
	videoFormat.f0_vs_front_porch = 0
	videoFormat.f0_vs_pulse_width = 0
	videoFormat.f0_vs_back_porch = 0
	--TODO:
	videoFormat.f1_vs_front_porch = 0
	videoFormat.f1_vs_pulse_width = 0
	videoFormat.f1_vs_back_porch = 0

	videoFormat.pixel_freq = 148500000	--Not correct, but don't care this.
	
	videoFormat.format_name = srcFormatName or (videoFormat.cap_cap_w .. "x" .. videoFormat.cap_cap_h)
	videoFormat.video_signal_type = _G.VIDEO_SIGNAL_DIGITAL or 3
	
	if convRslt.out_crop then
		videoFormat.out_crop = true
		videoFormat.out_crop_width = convRslt.out_crop_width
		videoFormat.out_crop_height = convRslt.out_crop_height
		videoFormat.out_crop_x = convRslt.out_crop_x	--Always should be 0
		videoFormat.out_crop_y = convRslt.out_crop_y	--Always should be 0
	else
		videoFormat.out_crop = false
	end
	
	return videoFormat
end

------------------------------------------------------------------------

function _M:UpdateConvension( formatInfo )
	if converter and convensionRules then
		local convRslt
		if formatInfo and formatInfo.cap_cap_w and formatInfo.cap_cap_h then
			convRslt = convensionRules.Convert( formatInfo.cap_cap_w, formatInfo.cap_cap_h, formatInfo.frame_rate_value or 60, formatInfo.interlaced and formatInfo.interlaced > 0 or false )
		else
			convRslt = convensionRules.Convert( 1920, 1080, 25, false )	--Assume 1920x1080p25 if no source detected
		end
		
		if not convRslt then
			print( "ERROR: Match convension rules fail for current format '" .. formatInfo.format_name .. "'" )
			return nil
		end
		
		converter:SetSourceFormat( convRslt.vin )
		if avfe.sdFormat_10bit and (convRslt.vin == VIDSRC_720x480i60 or convRslt.vin == VIDSRC_720x576i50) then
			print( "*** Control source input mode is MUX656" )
			converter:SetSourceMode( MDIN_SRC_MUX656_10 )
		else
			print( "*** Control source input mode is EMB422" )
			converter:SetSourceMode( MDIN_SRC_EMB422_10 )
		end
		
		converter:SetDestFormat( convRslt.vout )
		converter:SetOutMode( convRslt.outctrl )
		if convRslt.src_crop then
			converter:SetSourceCrop( convRslt.src_crop_x, convRslt.src_crop_y, convRslt.src_crop_width, convRslt.src_crop_height )
		else
			converter:ResetSourceCrop()
		end
	
		--Notice the source pixel repeat
		converter:SetSourcePixelRepeat( convRslt.pixelRepeat or 1 )
	
		if convRslt.out_crop then
			converter:SetOutAtLeftTopCorner( 1 )	--The crop picture put at left-top corner
		else
			converter:SetOutAtLeftTopCorner( 0 )	-- Center (default)
		end
		
		--NOTE: If with converter, though it with freerun signal, I also show blue scrren.
		if formatInfo and formatInfo.signal_locked and not formatInfo.free_run then
			if formatInfo.hdcp_content and formatInfo.hdcp_content ~= 0 then
				set_led("BLINK")
				if _G.DISABLE_HDCP then
					converter:ShowBlue()
					UpdateSignalOSD(true)
				else
					converter:HideBlue()
					UpdateSignalOSD(false)
				end
			else
				set_led("ON")
				converter:HideBlue()
				UpdateSignalOSD(false)
			end
		else
			set_led("OFF")
			converter:ShowBlue()
			UpdateSignalOSD(true)
		end
		
		converter:Care()
		
		return self:CalcConvensionFormatInfo( convRslt, formatInfo and formatInfo.format_name or "UNKNOWN", formatInfo and formatInfo.signal_locked )
	else
		return nil
	end
end
------------------------------------------------------------------------

function _M:SelectVideoSource( src )
	local srcName = avfe:SelectVideoSource(src)
	if  srcName then
		if srcName ~= "AUTO" and srcName ~="ONESHOT" and srcName ~= self.video.currentSource then
			self.video.currentSource = srcName
			self.video.sourceChanged = true
		end
		return true
	else
		return false
	end
end

------------------------------------------------------------------------
function _M:SelectAudioSource( src )
	print( "Selecting audio source of:", src )
	local srcName = avfe:SelectAudioSource(src) 
	if srcName then
		if srcName ~= "AUTO" and srcName ~="ONESHOT" and srcName ~= self.audio.currentSource then
			self.audio.currentSource = srcName
			self.audio.sourceChanged = true
		end
		print( "Audio source selection DONE")
		return true
	else
		print( "Audio source selection FAIL")
		return false
	end	
end
------------------------------------------------------------------------
function _M:SetVideoParameter( pmName, pmValue, src )
	if self.converter then	--With converter, adjust these parameters by converter
		if pmName == "brightness" then
			converter:SetBrightness( tonumber(pmValue) )
		elseif pmName == "contrast" then
			converter:SetContrast( tonumber(pmValue) )
		elseif pmName == "saturation" then
			converter:SetSaturation( tonumber(pmValue) )
		elseif pmName == "hue" then
			converter:SetHue( tonumber(pmValue) )
		else
			avfe:SetVideoParameter( pmName, pmValue, src )
		end
	else --Adjust by the AVFE if it support
		avfe:SetVideoParameter( pmName, pmValue, src )
	end
end
------------------------------------------------------------------------
function _M:SetAudioParameter( pmName, pmValue, src )
	avfe:SetAudioParameter( pmName, pmValue, src )
end
------------------------------------------------------------------------
--Set the output crop: It's cropped by ambarella actually.
function _M:SetOutputCrop( enable, cropX, cropY, cropW, cropH )
	if not enable then
		print( "*** Disabled output cropping." )
		_M.video.specOutCrop = nil
		--[[
		local file = io.open( CONFIG_LOC .. "/out_crop.conf", "w" )
		if file then
			file:write( "return nil\n" )
			file:close()
		end
		]]
	else
		_M.video.specOutCrop = {
			cropX = cropX and math.floor(tonumber(cropX)) or 0,
			cropY = cropY and math.floor(tonumber(cropY)) or 0,
			cropW = cropW and math.floor(tonumber(cropW)) or 0,
			cropH = cropH and math.floor(tonumber(cropH)) or 0
		}
		print( "*** Enabled output cropping: (" .. _M.video.specOutCrop.cropX .. ", " .. _M.video.specOutCrop.cropY .. ", " .. _M.video.specOutCrop.cropW .. ", " .. _M.video.specOutCrop.cropH .. ")" )
		--[[
		local file = io.open( CONFIG_LOC .. "/out_crop.conf", "w" )
		if file then
			file:write( "return { cropX = " .. (cropX or "0") .. ", cropY = " .. (cropY or "0") .. ", cropW = " .. (cropW or "0") .. ", cropH = " .. (cropH or "0") .. " }\n" )
			file:close()
		end
		]]
	end
	
	self.video.version = nil		--Force to update while next Check(...)
end

------------------------------------------------------------------------

function _M:SetOutputRotateMirrorFlip( rmf )
	print( "*** Set output Rotate/Mirror/Flip: ", rmf )
	if not rmf then
		self.video.rmf = "0"
	else
		self.video.rmf = rmf
	end

	self.video.version = nil		--Force to update while next Check(...)
end

------------------------------------------------------------------------
--All those special VGA operations, may not fit for all AVIO types.
------------------------------------------------------------------------
function _M:VGAGetOptionFormats( formatCode )
	if type(avfe["VGAGetOptionFormats"]) == "function" then	--It support this
		return avfe:VGAGetOptionFormats( formatCode )
	else
		return nil
	end
end
------------------------------------------------------------------------
--For VGA applications: Set current special format
function _M:VGASetSpecFormat(formatCode, modeline)
	if type(avfe["VGASetSpecFormat"]) == "function" then	--It support this
		return avfe:VGASetSpecFormat( formatCode, modeline )
	else
		return false
	end
end
------------------------------------------------------------------------
--For VGA applications: Adjust the X/Y shift for picture
function _M:VGAPicAdjustment( formatCode, shiftX, shiftY, phase )
	if type(avfe["VGAPicAdjustment"]) == "function" then	--It support this
		return avfe:VGAPicAdjustment( formatCode, shiftX, shiftY, phase )
	else
		return false
	end
end
------------------------------------------------------------------------
function _M:VGAGetPicAdjustment( formatCode )
	if type(avfe["VGAGetPicAdjustment"]) == "function" then	--It support this
		return avfe:VGAGetPicAdjustment( formatCode )
	else
		return nil
	end
end
------------------------------------------------------------------------
--Set VGA user modelines
function _M:VGASetModelines( modelines )
	if type(avfe["VGASetModelines"]) == "function" then	--It support this
		return avfe:VGASetModelines( modelines )
	else
		return false
	end
end
------------------------------------------------------------------------
--Get VGA user modelines
function _M:VGAGetModelines()
	if type(avfe["VGAGetModelines"]) == "function" then	--It support this
		return avfe:VGAGetModelines( )
	else
		return nil
	end
end
------------------------------------------------------------------------
function _M:ConvSetSourceCrop( en, x, y, w, h )
	if convensionRules then
		convensionRules.SetSpecCrop( en, x, y, w, h )
		self.video.version = nil		--Force to update while next Check(...)
	end
end
------------------------------------------------------------------------
function _M:ConvSetSpecOut( format, mode )
	if convensionRules then
		convensionRules.SetSpecOut( format, mode )
		self.video.version = nil		--Force to update while next Check(...)
	end
end
------------------------------------------------------------------------

function _M:Init()
	print( "Creating VIN arguments bridge..." )
	self.vinFormatBridge = AVIN.createVinArgs( "/proc/avin/0/vformat.data" )
	if not self.vinFormatBridge then
		print( "WARNING: Could not create video format bridge to the kernel driver! (/proc/avin/0/vformat.data)" )
	else
		print( "VIN arguments bridge created." )
		self.vinFormatBridge:ResetDefaults()
	end
	
	print( "Creating AIN arguments bridge..." )
	self.ainFormatBridge = AVIN.createAinArgs( "/proc/avin/0/aformat.data" )
	if not self.ainFormatBridge then
		print( "WARNING: Could not create audio format bridge to the kernel driver! (/proc/avin/0/vformat.data)" )
	else
		print( "AIN arguments bridge created." )
		self.ainFormatBridge:ResetDefaults()
	end
	
	local vSrcs = avfe:GetSupportVideoSources()
	if type(vSrcs) == "table" then
		self.video.supportSources = vSrcs
	end
	--Now, don't care the targets
	
	local aSrcs = avfe:GetSupportAudioSources()
	if type(aSrcs) == "table" then
		self.audio.supportSources = aSrcs
	end
	--Now, don't care the targets
--[[	
	local r
	r, self.video.specOutCrop = pcall( dofile, CONFIG_LOC .. "/out_crop.conf" )
	if not r then
		_M.video.specOutCrop = nil
	end
]]
end

------------------------------------------------------------------------

return _M

--End

