local allLines = require "orig_builtin_modelines"

local MAIN_FREQ = _G.MAIN_FREQ or 28636360

local function ParseModeline( k, v )
	local hPulse = 1
	local vPulse = 1
	local interlaced = false
	local doublescan = false
	local uniKey = nil
	
	local name, pixelFreq, actH, hSyncStart, hSyncEnd, totalH, actV, vSyncStart, vSyncEnd, totalV =
			k:match('"(.*)"%s+([%d%.]+)%s+(%d-)%s+(%d-)%s+(%d-)%s+(%d-)%s+(%d-)%s+(%d-)%s+(%d-)%s+(%d+).*$')
	if nil ~= name then	--Parse OK
		if not v or type(v) ~= "table" then
			v = {}
		end
		
		k = string.lower(k)
		if k:match("-hsync") then
			hPulse = 0
		end
		
		if k:match("-vsync") then
			vPulse = 0
		end
		
		if k:match("interlace") then
			interlaced = true
		end
		
		if k:match("doublescan") then
			doublescan = true
		end
		
		v.pixelFreq = tonumber( pixelFreq ) * 1000000
		v.hTotal = tonumber(totalH)
		v.hAct = tonumber(actH)
		v.hSyncStart = tonumber(hSyncStart)
		v.hSyncEnd = tonumber(hSyncEnd)
		v.vTotal = tonumber(totalV)
		v.vAct = tonumber(actV)
		v.vSyncStart = tonumber(vSyncStart)
		v.vSyncEnd = tonumber(vSyncEnd)
		
		v.hPulse = hPulse
		v.vPulse = vPulse
		v.doubleScan = doublescan
		
		v.interlaced = interlaced
		
		if v.pixelFreq > 0 and v.hTotal > 0 and v.vTotal > 0 then
			--Restrict it in our video standards
			v.frameRate = v.pixelFreq / (v.hTotal * v.vTotal)
			if v.doubleScan then
				v.frameRate = v.frameRate/2
			end
			
			local base = math.floor(v.frameRate)
			if v.frameRate - base < 0.5 then
				v.frameRate = base
			else
				v.frameRate = base + 1
			end
			
			v.BL = MAIN_FREQ / ((v.pixelFreq/v.hTotal)/8)
			v.LCF = v.vTotal
			v.FCL = MAIN_FREQ / (v.pixelFreq / (v.hTotal * v.vTotal)) / 256
			v.LCVS = v.vSyncEnd - v.vSyncStart
		else
			v.frameRate = 60	--Assume default
			v.BL = MAIN_FREQ/ ((v.frameRate*v.vTotal)/8)
			v.LCF = v.vTotal
			v.FCL = MAIN_FREQ / v.frameRate / 256
			v.LCVS = v.vSyncEnd - v.vSyncStart
		end
		
		if name:match("@") then
			v.formatName = name
		else
			v.formatName = name .. "@" .. v.frameRate .. (interlaced and "i" or "Hz") 
		end
	end
	return v
end
---------------------------------------------

local parsed = {}
local sortIndex = {}

---------------------------------------------
-- To find duplicated mode lines.
---------------------------------------------

local function FindDupl( k,v )
	local kk,vv
	for kk,vv in pairs(parsed) do
		if 
			vv.BL == v.BL and
			vv.LCF == v.LCF and
			vv.FCL == v.FCL and
			vv.LCVS == v.LCVS and
			vv.interlaced == v.interlaced
		then
			return kk
		end
	end
	return nil
end

---------------------------------------------

print( "-- This is an auto generated mode-lines file which parsed from orig_builtin_modelines.lua, restricted duplications." )
print( "" )

---------------------------------------------

local k,v, pk, pv
for k,v in pairs(allLines) do
	pv = ParseModeline(k)
	
	if v._VESA then
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 60) + (v._MFLAG or 0)
	elseif v._HDTV then
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 50) + (v._MFLAG or 0)
	elseif v._NTSC or v._PAL then
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 40) + (v._MFLAG or 0)
	elseif v._CVT or v._GTF then
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 30) + (v._MFLAG or 0)
	elseif v._NVIDIA then
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 20) + (v._MFLAG or 0)
	else
		pv._MATCH_LEVEL = (v._MATCH_LEVEL or 10) + (v._MFLAG or 0)
	end
		
	pk = FindDupl( k, pv )
	if pk ~= nil then
		local xv = parsed[pk]
		
		if pv._MATCH_LEVEL > xv._MATCH_LEVEL then
			parsed[pk] = nil --Drop old and replace the new because it more match
			pv._OO = v --Original Options
			parsed[k] = pv
			print( "-- REPLACED: [ " .. pk .. " ], level=", xv._MATCH_LEVEL )
			print( "-- TO................[ " .. k .. " ], level=", pv._MATCH_LEVEL )
		else	
			print( "-- Duplicated: [ " .. k .. " ], level=", pv._MATCH_LEVEL )
			print( "-- ....................[ " .. pk .. " ], level=", xv._MATCH_LEVEL )
		end
	else
		pv._OO = v --Original Options
		parsed[ k ] = pv
	end
end

for pk,pv in pairs(parsed) do
	local _groupStr = string.format( "%04d%04d%04d%05d", math.floor(pv.LCF), math.floor(pv.LCVS), math.floor(pv.FCL), math.floor(pv.BL) )
	table.insert( sortIndex, _groupStr .. pk )		--Insert the modeline with '_groupStr' as prefix, sort to keep group sequence.
end

table.sort( sortIndex )

local function FormatItem(counter, line, opts)
	print( "\t[ [[" .. line .. "]] ] = {" )
	print( "\t\t_SEQ=" .. counter .. "," )

	local argline = string.format( [[formatName="%s", frameRate=%d, LCF=%.2f, LCVS=%.2f, FCL=%.2f, BL=%.2f, pixelFreq=%d, hTotal=%d, hAct=%d, hSyncStart=%d, hSyncEnd=%d, vTotal=%d, vAct=%d, vSyncStart=%d, vSyncEnd=%d, hPulse=%d, vPulse=%d, doubleScan=%s, interlaced=%s, _MATCH_LEVEL=%d]], 
	opts.formatName, 
	opts.frameRate or 0, 
	opts.LCF or 0, 
	opts.LCVS or 0, 
	opts.FCL or 0,
	opts.BL or 0, 
	opts.pixelFreq or 0,
	opts.hTotal or 0, opts.hAct or 0, opts.hSyncStart or 0, opts.hSyncEnd or 0,
	opts.vTotal or 0, opts.vAct or 0, opts.vSyncStart or 0, opts.vSyncEnd or 0, 
	opts.hPulse or 0, opts.vPulse or 0, opts.doubleScan and "true" or "false", opts.interlaced and "true" or "false",
	opts._MATCH_LEVEL or 0 )

	if type(v._OO) == "table" then
		for pk, pv in pairs(v._OO) do
			argline = argline .. ", "
			argline = argline .. pk .. " = "
			if type(pv) == "boolean" then
				argline = argline .. ( pv and "true" or "false" )
			elseif type(pv) == "number" then
				argline = argline .. string.format( "0x%x", pv )
			elseif type(pv) == "string" then
				argline = argline .. '"' .. pv .. '"'
			end
		end
	end
	

	print( "\t", argline )
	print( "\t},\n" )
end

print( "\nlocal fineModelines = {" )

local __, counter
counter = 1

local token
local indexModelines = {}

for __, k in pairs(sortIndex) do
	token = k:sub(1,8)
	k = k:sub(18)
	v = parsed[k]
	FormatItem(counter, k,v)
	--table.insert( indexModelines, "[[" .. k .. "]] = " .. counter .. "," )
	counter = counter + 1
end

print("}")

--print( "\nlocal indexModelines = {" )
--for __, k in pairs(indexModelines) do
--	print( k )
--end
--print( "}" )

--print( "\nreturn fineModelines, indexModelines" )
print( "\nreturn fineModelines" )

