local _M = {}

---------------------------
_M.VIRATE_SD = 0
_M.VIRATE_HD = 1
_M.VIRATE_3G = 2
_M.VIRATE_VGA = 3
---------------------------
local VIRATE_SD = _M.VIRATE_SD
local VIRATE_HD = _M.VIRATE_HD
local VIRATE_3G = _M.VIRATE_3G
local VIRATE_VGA = _M.VIRATE_VGA

---------------------------
local RATIO_AUTO = _G.AMBA_VIDEO_RATIO_AUTO or 0
local RATIO_4_3 = _G.AMBA_VIDEO_RATIO_4_3 or 1
local RATIO_16_9 = _G.AMBA_VIDEO_RATIO_16_9 or 2
local RATIO_1_1 = _G.AMBA_VIDEO_RATIO_1_1 or 4

_M.RATIO_AUTO = RATIO_AUTO
_M.RATIO_4_3 = RATIO_4_3
_M.RATIO_16_9 = RATIO_16_9
_M.RATIO_1_1 = RATIO_1_1
---------------------------

local VIDEO_SYSTEM_AUTO = _G.AMBA_VIDEO_SYSTEM_AUTO or 0x0
local VIDEO_SYSTEM_NTSC = _G.AMBA_VIDEO_SYSTEM_NTSC or 0x1
local VIDEO_SYSTEM_PAL = _G.AMBA_VIDEO_SYSTEM_PAL or 0x2
local VIDEO_SYSTEM_SECAM = _G.AMBA_VIDEO_SYSTEM_SECAM or 0x4
local VIDEO_SYSTEM_ALL = _G.AMBA_VIDEO_SYSTEM_ALL or 0xf

_M.VIDEO_SYSTEM_AUTO = VIDEO_SYSTEM_AUTO
_M.VIDEO_SYSTEM_NTSC = VIDEO_SYSTEM_NTSC
_M.VIDEO_SYSTEM_PAL = VIDEO_SYSTEM_PAL
_M.VIDEO_SYSTEM_SECAM = VIDEO_SYSTEM_SECAM
_M.VIDEO_SYSTEM_ALL = VIDEO_SYSTEM_ALL

--------------------------

_M.format_map = {
	[0x2b] = {
		{
			formatId = "1080p60",
			formatName = "1080p 60Hz",
			frameRate = 60,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p59",
			formatName = "1080p 59.94Hz",
			frameRate = 59.94,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
		
		{
			formatId = "1080p30",
			formatName = "1080p 30Hz",
			frameRate = 30,
			hTotal = 4400,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p29",
			formatName = "1080p 29.97Hz",
			frameRate = 29.97,
			hTotal = 4400,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},		
	},
	
	[0x2d] = {
		{
			formatId = "1080p50",
			formatName = "1080p 50Hz",
			frameRate = 50,
			hTotal = 2640,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p25",
			formatName = "1080p 25Hz",
			frameRate = 25,
			hTotal = 5280,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		}
	},
	
	[0x2a] = {
		{
			formatId = "1080i60",
			formatName = "1080i 60Hz",
			frameRate = 60,
			hTotal = 4400,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 540,
			vTotal1 = 563,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080i59",
			formatName = "1080i 59.94Hz",
			frameRate = 59.94,
			hTotal = 4400,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 540,
			vTotal1 = 563,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		}
	},
	
	[0x2c] = {
		{
			formatId = "1080i50",
			formatName = "1080i 50Hz",
			frameRate = 50,
			hTotal = 5280,
			hAct = 1920,
			vTotal0 = 625,
			vAct0 = 540,
			vTotal1 = 625,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		}
	},

	[0x20] = {
		{
			formatId = "720p60",
			formatName = "720p 60Hz",
			frameRate = 60,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p59",
			formatName = "720p 59.94Hz",
			frameRate = 59.94,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},
	
	[0x24] = {
		{
			formatId = "720p50",
			formatName = "720p 50Hz",
			frameRate = 50,
			hTotal = 3960,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},
	
	[0x26] = {
		{
			formatId = "720p25",
			formatName = "720p 25Hz",
			frameRate = 25,
			hTotal = 7920,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},
	
	[0x30] = {
		{
			formatId = "1080p24",
			formatName = "1080p 24Hz",
			frameRate = 24,
			hTotal = 5500,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p23",
			formatName = "1080p 23.976Hz",
			frameRate = 23.976,
			hTotal = 5500,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		}
	},
	
	[0x28] = {
		{
			formatId = "720p24",
			formatName = "720p 24Hz",
			frameRate = 24,
			hTotal = 8250,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p23",
			formatName = "720p 23.976Hz",
			frameRate = 23.976,
			hTotal = 8250,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_3G,
			tmdsFreq = 148500000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},
	
	[0x15] = {
		{
			formatId = "1035i60",
			formatName = "1920x1035i 60Hz",
			frameRate = 60,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 517,
			vTotal1 = 563,
			vAct1 = 518,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},	
		
		{
			formatId = "1035i59",
			formatName = "1920x1035i 59.94Hz",
			frameRate = 59.94,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 517,
			vTotal1 = 563,
			vAct1 = 518,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},	
	},
	
	[0x14] = {
		{
			formatId = "1080i50",
			formatName = "1080i 50Hz",
			frameRate = 50,
			hTotal = 2376,
			hAct = 1920,
			vTotal0 = 625,
			vAct0 = 540,
			vTotal1 = 625,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},	
	},

	[0x0a] = {
		{
			formatId = "1080i60",
			formatName = "1080i 60Hz",
			frameRate = 60,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 540,
			vTotal1 = 563,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},	
		
		{
			formatId = "1080i59",
			formatName = "1080i 59.94Hz",
			frameRate = 59.94,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 562,
			vAct0 = 540,
			vTotal1 = 563,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},	
	},

	[0x0c] = {
		{
			formatId = "1080i50",
			formatName = "1080i 50Hz",
			frameRate = 50,
			hTotal = 2640,
			hAct = 1920,
			vTotal0 = 625,
			vAct0 = 540,
			vTotal1 = 625,
			vAct1 = 540,
			interlaced = true,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080I,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},	
	},

	[0x0b] = {
		{
			formatId = "1080p30",
			formatName = "1080p 30Hz",
			frameRate = 30,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p29",
			formatName = "1080p 29.97Hz",
			frameRate = 29.97,
			hTotal = 2200,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},			
	},

	[0x0d] = {
		{
			formatId = "1080p25",
			formatName = "1080p 25Hz",
			frameRate = 25,
			hTotal = 2640,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},
	
	[0x10] = {
		{
			formatId = "1080p24",
			formatName = "1080p 24Hz",
			frameRate = 24,
			hTotal = 2750,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "1080p23",
			formatName = "1080p 23.976Hz",
			frameRate = 23.976,
			hTotal = 2750,
			hAct = 1920,
			vTotal0 = 1125,
			vAct0 = 1080,
			vTotal1 = 1125,
			vAct1 = 1080,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_1080P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},
	
	--Ignored 0x0e, and 0x12
	
	[0x02] = {
		{
			formatId = "720p30",
			formatName = "720p 30Hz",
			frameRate = 30,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p29",
			formatName = "720p 29.97Hz",
			frameRate = 29.97,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},
	
	[0x03] = {
		{
			formatId = "720p30",
			formatName = "720p 30Hz(EM)",
			frameRate = 30,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p29",
			formatName = "720p 29.97Hz(EM)",
			frameRate = 29.97,
			hTotal = 3300,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},	

	[0x04] = {
		{
			formatId = "720p50",
			formatName = "720p 50Hz",
			frameRate = 50,
			hTotal = 1980,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},
	
	[0x05] = {
		{
			formatId = "720p50",
			formatName = "720p 50Hz(EM)",
			frameRate = 50,
			hTotal = 1980,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},
	
	[0x06] = {
		{
			formatId = "720p25",
			formatName = "720p 25Hz",
			frameRate = 25,
			hTotal = 3960,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},	
	
	[0x07] = {
		{
			formatId = "720p25",
			formatName = "720p 25Hz(EM)",
			frameRate = 25,
			hTotal = 3960,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
	},	

	[0x08] = {
		{
			formatId = "720p24",
			formatName = "720p 24Hz",
			frameRate = 24,
			hTotal = 4125,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p23",
			formatName = "720p 23.976Hz",
			frameRate = 23.976,
			hTotal = 4125,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},		
	},	
	
	[0x09] = {
		{
			formatId = "720p24",
			formatName = "720p 24Hz(EM)",
			frameRate = 24,
			hTotal = 4125,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p23",
			formatName = "720p 23.976Hz(EM)",
			frameRate = 23.976,
			hTotal = 4125,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},		
	},	

	[0x00] = {
		{
			formatId = "720p60",
			formatName = "720p 60Hz",
			frameRate = 60,
			hTotal = 1650,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p59",
			formatName = "720p 59.94Hz",
			frameRate = 59.94,
			hTotal = 1650,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = false,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},	

	[0x01] = {
		{
			formatId = "720p60",
			formatName = "720p 60Hz(EM)",
			frameRate = 60,
			hTotal = 1650,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "720p59",
			formatName = "720p 59.94Hz(EM)",
			frameRate = 59.94,
			hTotal = 1650,
			hAct = 1280,
			vTotal0 = 750,
			vAct0 = 720,
			vTotal1 = 750,
			vAct1 = 720,
			interlaced = false,
			rateMode = VIRATE_HD,
			tmdsFreq = 74250000,
			spec_amba_mode = AMBA_VIDEO_MODE_720P,
			pixelRepeat = true,
			aspect_ratio = RATIO_16_9,
			video_system = VIDEO_SYSTEM_NTSC
		},
	},	

	[0x16] = {
		{
			formatId = "480i",
			formatName = "525i 60Hz",
			frameRate = 60,
			hTotal = 1716,
			hAct = 720,
			vTotal0 = 263,
			vAct0 = 244,
			vTotal1 = 262,
			vAct1 = 243,
			interlaced = true,
			rateMode = VIRATE_SD,
			tmdsFreq = 27000000,
			spec_amba_mode = AMBA_VIDEO_MODE_480I,
			pixelRepeat = true,
			aspect_ratio = RATIO_4_3,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "480i",
			formatName = "525i 59.94Hz",
			frameRate = 59.94,
			hTotal = 1716,
			hAct = 720,
			vTotal0 = 263,
			vAct0 = 244,
			vTotal1 = 262,
			vAct1 = 243,
			interlaced = true,
			rateMode = VIRATE_SD,
			tmdsFreq = 27000000,
			spec_amba_mode = AMBA_VIDEO_MODE_480I,
			pixelRepeat = true,
			aspect_ratio = RATIO_4_3,
			video_system = VIDEO_SYSTEM_NTSC
		},		
	},	

	[0x17] = {
		{
			formatId = "507i",
			formatName = "525i(507) 60Hz",
			frameRate = 60,
			hTotal = 1716,
			hAct = 720,
			vTotal0 = 263,
			vAct0 = 254,
			vTotal1 = 262,
			vAct1 = 253,
			interlaced = true,
			rateMode = VIRATE_SD,
			tmdsFreq = 27000000,
			spec_amba_mode = AMBA_VIDEO_MODE_480I,
			pixelRepeat = true,
			aspect_ratio = RATIO_4_3,
			video_system = VIDEO_SYSTEM_AUTO
		},
		
		{
			formatId = "507i",
			formatName = "525i(507) 59.94Hz",
			frameRate = 59.94,
			hTotal = 1716,
			hAct = 720,
			vTotal0 = 263,
			vAct0 = 254,
			vTotal1 = 262,
			vAct1 = 253,
			interlaced = true,
			rateMode = VIRATE_SD,
			tmdsFreq = 27000000,
			spec_amba_mode = AMBA_VIDEO_MODE_480I,
			pixelRepeat = true,
			aspect_ratio = RATIO_4_3,
			video_system = VIDEO_SYSTEM_NTSC
		},	
	},

	[0x18] = {
		{
			formatId = "576i",
			formatName = "625i 50Hz",
			frameRate = 50,
			hTotal = 1728,
			hAct = 720,
			vTotal0 = 312,
			vAct0 = 288,
			vTotal1 = 313,
			vAct1 = 288,
			interlaced = true,
			rateMode = VIRATE_SD,
			tmdsFreq = 27000000,
			spec_amba_mode = AMBA_VIDEO_MODE_576I,
			pixelRepeat = true,
			aspect_ratio = RATIO_4_3,
			video_system = VIDEO_SYSTEM_AUTO
		}
	},
}

function _M.Init(  )
end

function _M.Standard_Match( standardId, hTotal, hAct, vTotal, vAct, interlaced, ntsc )
	if standardId then
		local args = _M.format_map[ standardId ]
		if args then
			if #args > 2 then
				local x
				local diff_hTotal, diff_vTotal
				for x=1, #args do
					diff_hTotal = math.abs( args[x].hTotal - hTotal )
					if args[x].interlaced then
						diff_vTotal = math.abs( (args[x].vTotal0 + args[x].vTotal1) - vTotal )
					else
						diff_vTotal = math.abs( args[x].vTotal0 - vTotal )
					end
					
					if diff_hTotal <= 4 and diff_vTotal <= 4 and ( not ntsc or args[x].video_system == VIDEO_SYSTEM_NTSC ) then
						return args[x]
					end
				end --For
				return nil, "No match"
			else
				if ntsc then
					return args[2] or args[1]
				else
					return args[1]
				end
			end
		else
			return nil, "No match"
		end
	else
		return nil, "Invalid arguments"
	end
end

return _M
