--------------------------------------------------
-- Convension rules for MD3xx
--------------------------------------------------
require "md3xx"
local CONFIG_LOC = "/data/configs"

local _M = {
	lastMatchItem = nil,
	convOptions = nil,
	specCrop = nil,
	specOut = nil
}

local RATIO_AUTO = _G.AMBA_VIDEO_RATIO_AUTO or 0
local RATIO_4_3 = _G.AMBA_VIDEO_RATIO_4_3 or 1
local RATIO_16_9 = _G.AMBA_VIDEO_RATIO_16_9 or 2
local RATIO_1_1 = _G.AMBA_VIDEO_RATIO_1_1 or 4

_M.RATIO_AUTO = RATIO_AUTO
_M.RATIO_4_3 = RATIO_4_3
_M.RATIO_16_9 = RATIO_16_9
_M.RATIO_1_1 = RATIO_1_1

--All the MD3XX supported output formats
--To add more supports...
local OUT_FORMAT_TABLE = {
	[VIDOUT_720x480i60] = {
		aspect_ratio = RATIO_4_3,
		width=720,
		height=480,
		interlaced=true,
		yuv8bit=true,
		frameRate=60,
	},
	
	[VIDOUT_720x576i50] = {
		aspect_ratio = RATIO_4_3,
		width=720,
		height=576,
		interlaced=true,
		yuv8bit=true,
		frameRate=50,
	},
	
	[VIDOUT_720x480p60] = {
		aspect_ratio = RATIO_4_3,
		width=720,
		height=480,
		--yuv8bit=true,
		frameRate=60,
	},
	
	[VIDOUT_720x576p50] = {
		aspect_ratio = RATIO_4_3,
		width=720,
		height=576,
		--yuv8bit=true,
		frameRate=60,
	},
	
	[VIDOUT_1280x720p60] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=60,
	},
	
	[VIDOUT_1280x720p59] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=59.94,
	},
	
	[VIDOUT_1280x720p50] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=50
	},
	
	[VIDOUT_1280x720p30] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=30
	},
	
	[VIDOUT_1280x720p25] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=25
	},
	
	[VIDOUT_1280x720p24] = {
		aspect_ratio = RATIO_16_9,
		width=1280,
		height=720,
		frameRate=24
	},
	
	[VIDOUT_1920x1080i60] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=60,
		interlaced=true
	},
	
	[VIDOUT_1920x1080i59] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=59.94,
		interlaced=true
	},
	
	[VIDOUT_1920x1080i50] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=50,
		interlaced=true
	},
	
	[VIDOUT_1920x1080p60] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=60,
	},
	
	[VIDOUT_1920x1080p59] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=59.94,
	},
	
	[VIDOUT_1920x1080p50] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=50,
	},

	[VIDOUT_1920x1080p30] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=30,
	},

	[VIDOUT_1920x1080p25] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=25,
	},
	
	[VIDOUT_1920x1080p24] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=24,
	},

	[VIDOUT_640x480p60] = {
		aspect_ratio = RATIO_4_3,
		width=640,
		height=480,
		frameRate=60,
	},
	
	[VIDOUT_640x480p75] = {
		aspect_ratio = RATIO_4_3,
		width=640,
		height=480,
		frameRate=75,
	},
	
	[VIDOUT_800x600p60] = {
		aspect_ratio = RATIO_4_3,
		width=800,
		height=600,
		frameRate=60,
	},

	[VIDOUT_800x600p75] = {
		aspect_ratio = RATIO_4_3,
		width=800,
		height=600,
		frameRate=75,
	},

	[VIDOUT_1024x768p60] = {
		aspect_ratio = RATIO_4_3,
		width=1024,
		height=768,
		frameRate=60,
	},
	
	[VIDOUT_1024x768p75] = {
		aspect_ratio = RATIO_4_3,
		width=1024,
		height=768,
		frameRate=75,
	},
	
	[VIDOUT_1280x1024p60] = {
		aspect_ratio = RATIO_AUTO,
		width=1280,
		height=1024,
		frameRate=60,
	},

	[VIDOUT_1280x1024p75] = {
		aspect_ratio = RATIO_AUTO,
		width=1280,
		height=1024,
		frameRate=75,
	},

	[VIDOUT_1360x768p60] = {
		aspect_ratio = RATIO_AUTO,
		width=1360,
		height=768,
		frameRate=60,
	},

	[VIDOUT_1600x1200p60] = {
		aspect_ratio = RATIO_4_3,
		width=1600,
		height=1200,
		frameRate=60,
	},

	[VIDOUT_1440x900p60] = {
		aspect_ratio = RATIO_AUTO,
		width=1440,
		height=900,
		frameRate=60,
	},
	
	[VIDOUT_1440x900p75] = {
		aspect_ratio = RATIO_AUTO,
		width=1440,
		height=900,
		frameRate=75,
	},
	
	[VIDOUT_1680x1050p60] = {
		aspect_ratio = RATIO_AUTO,
		width=1680,
		height=1050,
		frameRate=60,
	},

	[VIDOUT_1680x1050pRB] = {
		aspect_ratio = RATIO_AUTO,
		width=1680,
		height=1050,
		frameRate=60,
	},
	
	[VIDOUT_1920x1080pRB] = {
		aspect_ratio = RATIO_16_9,
		width=1920,
		height=1080,
		frameRate=60,
	},
	
	[VIDOUT_1920x1200pRB] = {
		aspect_ratio = RATIO_AUTO,
		width=1920,
		height=1200,
		frameRate=60,
	},
}

local CONVENSION_TABLE = {
	{ width = 1440, height = 480, frameRate = 60, interlaced = true, vin = VIDSRC_720x480i60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true, cropWidth = 720, cropHeight = 480 },
	{ width = 1440, height = 480, frameRate = 59.94, interlaced = true, vin = VIDSRC_720x480i60, vout = VIDOUT_1280x720p59, outctrl = OUTCTRL_P2P, crop=true, cropWidth = 720, cropHeight = 480 },
	{ width = 1440, height = 576, frameRate = 50, interlaced = true, vin = VIDSRC_720x576i50, vout = VIDOUT_1280x720p25, outctrl = OUTCTRL_P2P, crop=true, cropWidth = 720, cropHeight = 576 },
	
	{ width = 720, height = 480, frameRate = 60, interlaced = true, vin = VIDSRC_720x480i60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 720, height = 480, frameRate = 59.94, interlaced = true, vin = VIDSRC_720x480i60, vout = VIDOUT_1280x720p59, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 720, height = 576, frameRate = 50, interlaced = true, vin = VIDSRC_720x576i50, vout = VIDOUT_1280x720p25, outctrl = OUTCTRL_P2P, crop=true },
	
	{ width = 720, height = 480, frameRate = 60, vin = VIDSRC_720x480p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 720, height = 480, frameRate = 59.94, vin = VIDSRC_720x480p60, vout = VIDOUT_1280x720p59, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 720, height = 576, frameRate = 50, vin = VIDSRC_720x576p50, vout = VIDOUT_1280x720p50, outctrl = OUTCTRL_P2P, crop=true },
	
	{ width = 1280, height = 720, frameRate = 60, vin = VIDSRC_1280x720p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_ASPECT },
	--{ width = 1280, height = 720, frameRate = 60, vin = VIDSRC_1280x720p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1280, height = 720, frameRate = 59.94, vin = VIDSRC_1280x720p60, vout = VIDOUT_1280x720p59, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 50, vin = VIDSRC_1280x720p50, vout = VIDOUT_1280x720p50, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 30, vin = VIDSRC_1280x720p60, vout = VIDOUT_1280x720p30, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 29.97, vin = VIDSRC_1280x720p60, vout = VIDOUT_1280x720p30, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 25, vin = VIDSRC_1280x720p50, vout = VIDOUT_1280x720p25, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 24, vin = VIDSRC_1280x720p50, vout = VIDOUT_1280x720p24, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 23.976, vin = VIDSRC_1280x720p50, vout = VIDOUT_1280x720p24, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 720, frameRate = 0, vin = VIDSRC_1280x720p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_ASPECT },
	
	{ width = 1920, height = 1080, frameRate = 60, interlaced = true, vin = VIDSRC_1920x1080i60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 59.94, interlaced = true, vin = VIDSRC_1920x1080i60, vout = VIDOUT_1920x1080p59, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 50, interlaced = true, vin = VIDSRC_1920x1080i50, vout = VIDOUT_1920x1080p50, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 0, interlaced = true, vin = VIDSRC_1920x1080i60, vout = VIDOUT_1920x1080p50, outctrl = OUTCTRL_ASPECT },
	
	{ width = 1920, height = 1080, frameRate = 60, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 59.94, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p59, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 50, vin = VIDSRC_1920x1080p50, vout = VIDOUT_1920x1080p50, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 30, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p30, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 29.97, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p30, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 25, vin = VIDSRC_1920x1080p50, vout = VIDOUT_1920x1080p25, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 24, vin = VIDSRC_1920x1080p50, vout = VIDOUT_1920x1080p24, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 23.976, vin = VIDSRC_1920x1080p50, vout = VIDOUT_1920x1080p24, outctrl = OUTCTRL_ASPECT },
	{ width = 1920, height = 1080, frameRate = 0, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT },
	
	{ width = 640, height = 480, frameRate = 60, vin = VIDSRC_640x480p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 640, height = 480, frameRate = 72, vin = VIDSRC_640x480p72, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 640, height = 480, frameRate = 75, vin = VIDSRC_640x480p75, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 640, height = 480, frameRate = 0, vin = VIDSRC_640x480p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	
	{ width = 800, height = 600, frameRate = 56, vin = VIDSRC_800x600p56, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 800, height = 600, frameRate = 60, vin = VIDSRC_800x600p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 800, height = 600, frameRate = 72, vin = VIDSRC_800x600p72, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 800, height = 600, frameRate = 75, vin = VIDSRC_800x600p75, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 800, height = 600, frameRate = 0, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop=true },
	
	{ width = 1024, height = 768, frameRate = 60, vin = VIDSRC_1024x768p60, vout = VIDOUT_1024x768p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1024, height = 768, frameRate = 70, vin = VIDSRC_1024x768p70, vout = VIDOUT_1024x768p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1024, height = 768, frameRate = 75, vin = VIDSRC_1024x768p75, vout = VIDOUT_1024x768p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1024, height = 768, frameRate = 0, vin = VIDSRC_1024x768p60, vout = VIDOUT_1024x768p60, outctrl = OUTCTRL_ASPECT },
	
	{ width = 1280, height = 1024, frameRate = 60, vin = VIDSRC_1280x1024p60, vout = VIDOUT_1280x1024p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 1024, frameRate = 75, vin = VIDSRC_1280x1024p75, vout = VIDOUT_1280x1024p60, outctrl = OUTCTRL_ASPECT },
	{ width = 1280, height = 1024, frameRate = 0, vin = VIDSRC_1280x1024p60, vout = VIDOUT_1280x1024p60, outctrl = OUTCTRL_ASPECT },
	
	{ width = 1360, height = 768, frameRate = 0, vin = VIDSRC_1360x768p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1600, height = 1200, frameRate = 0, vin = VIDSRC_1600x1200p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop=true, cropWidth=1440, cropHeight=1080 },
	
	{ width = 1440, height = 900, frameRate = 60, vin = VIDSRC_1440x900p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 1440, height = 900, frameRate = 75, vin = VIDSRC_1440x900p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 1440, height = 900, frameRate = 0, vin = VIDSRC_1440x900p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop=true },
	
	{ width = 1680, height = 1050, frameRate = 0, vin = VIDSRC_1680x1050p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop=true },
	{ width = 1920, height = 1200, frameRate = 0, vin = VIDSRC_1920x1200p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop=true, cropWidth=1728, cropHeight = 1080 },

	{ width = 1152, height = 864, frameRate = 0, vin = VIDSRC_1152x864p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1280, height = 800, frameRate = 0, vin = VIDSRC_1280x800p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1280, height = 960, frameRate = 0, vin = VIDSRC_1280x960p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	
	{ width = 640, height = 350, frameRate = 0, vin = VIDSRC_640x350p70, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 640, height = 400, frameRate = 0, vin = VIDSRC_640x400p70, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop = true },
	
	{ width = 720, height = 400, frameRate = 0, vin = VIDSRC_720x400p85, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 768, height = 576, frameRate = 0, vin = VIDSRC_768x576p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1280, height = 600, frameRate = 0, vin = VIDSRC_1280x600p60, vout = VIDOUT_1280x720p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1280, height = 768, frameRate = 0, vin = VIDSRC_1280x768p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1368, height = 768, frameRate = 0, vin = VIDSRC_1368x768p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1400, height = 1050, frameRate = 0, vin = VIDSRC_1400x1050p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1600, height = 900, frameRate = 0, vin = VIDSRC_1600x900p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true },
	{ width = 1792, height = 1344, frameRate = 0, vin = VIDSRC_1792x1344p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop = true, cropWidth=1440, cropHeight=1080 },
	{ width = 1856, height = 1392, frameRate = 0, vin = VIDSRC_1856x1392p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop = true, cropWidth=1440, cropHeight=1080 },
	{ width = 1800, height = 1440, frameRate = 0, vin = VIDSRC_1800x1440p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop = true, cropWidth=1350, cropHeight=1080 },
	{ width = 1920, height = 1440, frameRate = 0, vin = VIDSRC_1920x1440p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_ASPECT, crop = true, cropWidth=1440, cropHeight=1080 },
	--Any other unknown resolutions, fit this:
	-- Crop source (0,0, actual_width, actual_height), and output as 1:1 center, crop the center picture to encoder
	{ width = 0, height = 0, frameRate = 0, vin = VIDSRC_1920x1080p60, vout = VIDOUT_1920x1080p60, outctrl = OUTCTRL_P2P, crop = true }
}

-------------------------
function _M.Init()
--[[
	local r
	r, _M.specCrop = pcall( dofile, CONFIG_LOC .. "/conv_crop.conf" )
	if not r then
		_M.specCrop = nil
	end
	
	r, _M.specOut = pcall( dofile, CONFIG_LOC .. "/conv_out.conf" )
	if not r then
		_M.specOut = nil
	end
]]
end

-------------------------

function _M.SetSpecCrop( enable, cropX, cropY, cropW, cropH )
	if not enable then
		print( "Disabled convension source cropping." )
		_M.specCrop = nil
		--[[
		local file = io.open( CONFIG_LOC .. "/conv_crop.conf", "w" )
		if file then
			file:write( "return nil\n" )
			file:close()
		end
		]]
	else
		_M.specCrop = {
			cropX = cropX and math.floor(tonumber(cropX)) or 0,
			cropY = cropY and math.floor(tonumber(cropY)) or 0,
			cropW = cropW and math.floor(tonumber(cropW)) or 0,
			cropH = cropH and math.floor(tonumber(cropH)) or 0
		}
		
		print( "Enable convension source cropping: (" .. _M.specCrop.cropX .. ", " .. _M.specCrop.cropY .. ", " .. _M.specCrop.cropW .. ", " .. _M.specCrop.cropH .. ")" )
		
		--[[
		local file = io.open( CONFIG_LOC .. "/conv_crop.conf", "w" )
		if file then
			file:write( "return { cropX = " .. (cropX or "0") .. ", cropY = " .. (cropY or "0") .. ", cropW = " .. (cropW or "0") .. ", cropH = " .. (cropH or "0") .. " }\n" )
			file:close()
		end
		]]
	end
end

-------------------------

function _M.SetSpecOut( outFormat, outMode )
	if not outFormat then
		print( "Disabled convension special output format. (That will be default output)" )
		_M.specOut = nil
		--[[
		local file = io.open( CONFIG_LOC .. "/conv_out.conf", "w" )
		if file then
			file:write( "return nil\n" )
			file:close()
		end
		]]
	else
		_M.specOut = {}
		if outFormat == "default" or outFormat == "DEFAULT" or outFormat == "" then
			print( "Set convension output format is DEFAULT." )
			_M.specOut.format = nil
		else
			print( "Set convension output format to: ", outFormat )
			_M.specOut.format = _G[ outFormat ]	--Index the global variant name / or nil.
		end
		
		if not outMode or outMode == "default" or outMode == "DEFAULT" or outMode == "" then
			print( "Set convension output mode is DEFAULT ( That is keep aspect ratio )" )
			_M.specOut.mode = nil
		else
			print( "Set convension output mode is:", outMode )
			_M.specOut.mode = _G[ outMode ]
		end
		--[[
		local file = io.open( CONFIG_LOC .. "/conv_out.conf", "w" )
		if file then
			file:write( "return { \n" )
			if _M.specOut.format then
				file:write( "format = _G['" .. outFormat .. "'], \n" )
			end
			
			if _M.specOut.mode then
				file:write( "mode = _G['" .. outMode .. "'], \n" )
			end
			
			file:write( "}\n" )
			file:close()
		end
		]]
	end
end

-------------------------

local function ConvertSpecials( convOpts )
	if not _M.specCrop and  not _M.specOut then
		--Nothing special, so return directly
		return convOpts
	else
		local r = {}
		local k,v
		for k,v in pairs(convOpts) do	--Dump it.
			r[k] = v
		end
		
		if _M.specCrop then -- Crop
			r.src_crop = true
			--No need value range checking because the MD3xx driver should do this.
			r.src_crop_x = _M.specCrop.cropX
			r.src_crop_y = _M.specCrop.cropY
			r.src_crop_width = _M.specCrop.cropW 
			r.src_crop_height = _M.specCrop.cropH
			
			--Default, also crop the output, because of the 1:1 output required. (But if user specified output format, it should changed)
			r.out_crop = true
			r.out_crop_x = 0
			r.out_crop_y = 0
			r.out_crop_width = _M.specCrop.cropW 
			r.out_crop_height = _M.specCrop.cropH
		end
	
		if _M.specOut then --Special output format
			if _M.specOut.format then --Specified format, otherwise using default
				r.vout = _M.specOut.format
				r.vout_format = OUT_FORMAT_TABLE[r.vout]	--Maybe nil?
				r.outctrl = OUTCTRL_ASPECT	--Default output is keep aspect. (this should be changed follow)
				
				if r.out_crop then	--If specified output format (not by default), then output should not be cropped.
					r.out_crop = false
				end
			end
		
			if _M.specOut.mode then	--Specified mode, otherwise using default
				r.outctrl = _M.specOut.mode
			end
		end
		
		return r
	end
end

-------------------------

function _M.Convert( srcWidth, srcHeight, srcFrate, interlaced )
	local aspect_ratio = srcHeight / srcWidth
	local pixelRepeat = 1
	if aspect_ratio < 0.5 and (srcWidth ~= 1440 or (srcHeight ~= 576 and srcHeight ~= 480)) then	--TODO: May not suite for all applications
		pixelRepeat = 2
	end
	
	local i
	for i=1, #CONVENSION_TABLE do
		if CONVENSION_TABLE[i].width == srcWidth and
			CONVENSION_TABLE[i].height == srcHeight and
			(CONVENSION_TABLE[i].interlaced and true or false) == interlaced and
			( CONVENSION_TABLE[i].frameRate == 0 or CONVENSION_TABLE[i].frameRate == srcFrate )
		then
			--This is the last matched item, so don't calculate it again.
			if _M.lastMatchItem == CONVENSION_TABLE[i] then
				return ConvertSpecials( _M.convOptions )
			end
			
			local convOptions = {
				vin = CONVENSION_TABLE[i].vin,
				vout = CONVENSION_TABLE[i].vout,
				vout_format = OUT_FORMAT_TABLE[CONVENSION_TABLE[i].vout],	--Maybe nil?
				outctrl = CONVENSION_TABLE[i].outctrl,
				pixelRepeat = pixelRepeat
			}
			
			if CONVENSION_TABLE[i].crop or pixelRepeat > 1 then	--Crop output needed
				convOptions.out_crop = true
				convOptions.out_crop_width = math.floor(CONVENSION_TABLE[i].cropWidth or (srcWidth/pixelRepeat))
				convOptions.out_crop_height = CONVENSION_TABLE[i].cropHeight or srcHeight
				convOptions.out_crop_x = 0	--It should always cropped at left-top corner.
				convOptions.out_crop_y = 0
				--[[
				convOptions.out_crop_x = math.floor( (convOptions.vout_format.width - convOptions.out_crop_width)/2 )
				convOptions.out_crop_y = math.floor( (convOptions.vout_format.height - convOptions.out_crop_height)/2 )
				if convOptions.out_crop_x < 0 then
					convOptions.out_crop_x = 0
				end
				
				if convOptions.out_crop_y < 0 then
					convOptions.out_crop_y = 0
				end
				]]
			end
		
			_M.lastMatchItem = CONVENSION_TABLE[i]
			_M.convOptions = convOptions
			return ConvertSpecials( _M.convOptions )
			
		elseif CONVENSION_TABLE[i].width == 0 and CONVENSION_TABLE[i].height == 0 then	--Any match
			local convOptions = {
				vin = CONVENSION_TABLE[i].vin,
				vout = CONVENSION_TABLE[i].vout,
				vout_format = OUT_FORMAT_TABLE[CONVENSION_TABLE[i].vout],	--Maybe nil?
				outctrl = CONVENSION_TABLE[i].outctrl,
				src_crop = true,
				src_crop_width = srcWidth,
				src_crop_height = srcHeight,
				src_crop_x = 0,
				src_crop_y = 0,
				pixelRepeat = pixelRepeat
			}
			
			if CONVENSION_TABLE[i].crop or pixelRepeat > 1 then	--Crop output needed
				convOptions.out_crop = true
				convOptions.out_crop_width = math.floor(CONVENSION_TABLE[i].cropWidth or (srcWidth/pixelRepeat))
				convOptions.out_crop_height = CONVENSION_TABLE[i].cropHeight or srcHeight
				convOptions.out_crop_x = 0 --math.floor( (convOptions.vout_format.width - convOptions.out_crop_width)/2 )
				convOptions.out_crop_y = 0 --math.floor( (convOptions.vout_format.height - convOptions.out_crop_height)/2 )
				--[[
				if convOptions.out_crop_x < 0 then
					convOptions.out_crop_x = 0
				end
				
				if convOptions.out_crop_y < 0 then
					convOptions.out_crop_y = 0
				end
				]]
			end
			
			_M.lastMatchItem = nil
			_M.convOptions = convOptions
			return ConvertSpecials( _M.convOptions )
		end
	end
	
	return nil
end

return _M
