require "kiloview.configure_helper"
----------------------------
_G.AVIO_CONFIG_LOADING = false

local function OnVideoParamChanged( owner, pObj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local avio = require "avio"
	if pObj:Name() == "source" then
		avio.UpdateVideoParameter( pObj:Name(), pObj:InternalGetString())
	else
		avio.UpdateVideoParameter( pObj:Name(), pObj:InternalGetInt())
	end
end
----------------------------
local function OnAudioParamChanged( owner, pObj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local avio = require "avio"
	if pObj:Name() == "source" then
		avio.UpdateAudioParameter( pObj:Name(), pObj:InternalGetString())
	elseif pObj:Name() == "line_bypass" then
		avio.UpdateAudioParameter( pObj:Name(), pObj:InternalGetInt())
	else
		avio.UpdateAudioParameter( pObj:Name(), pObj:InternalGetDouble())
	end
end
----------------------------
local function OnPreviewParamChanged( obj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local mo_encoder = require "encoder"
	mo_encoder.NotifyReset()
end
----------------------------
local function OnSourceCropChanged( obj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local avio = require "avio"
	avio.SetSourceCrop( obj.enable ~= 0, obj.cropX, obj.cropY, obj.cropW, obj.cropH )
end

----------------------------

local function OnSpecOutChanged( obj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local avio = require "avio"
	avio.SetSpecOut( obj.format, obj.mode )
end

----------------------------

local function OnOutCropChanged( obj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	
	local avio = require "avio"
	avio.SetOutCrop( obj.enable ~= 0, obj.cropX, obj.cropY, obj.cropW, obj.cropH )
end

----------------------------

local function SetRMF( owner, pObj, value )
	local curVal = pObj:InternalGetString()
	if value ~= "0" and value ~= "90" and value ~= "180" and value ~= "270" and value ~= "mirror" and value ~="flip" then
		pObj:InternalSetString( "0" )
	else
		pObj:InternalSetString( value )
	end
	if curVal ~= value then
		pObj:InternalSetChanged(true)
	end
end

----------------------------

local function OnRMFChanged( owner, pObj )
	if _G.AVIO_CONFIG_LOADING then
		return
	end
	local avio = require "avio"
	avio.SetOutRMF( pObj:InternalGetString() )
end


----------------------------

AVIO_Template = OBJECT({
	preview = OBJECT({
		enable = { type="integer", value=1 },
		format = { type="string", value="auto" }
	}, OnPreviewParamChanged),

	video = OBJECT({
		source = { type="string", value="AUTO", onChanged = OnVideoParamChanged },
		
		brightness = { type="integer", value=128, onChanged = OnVideoParamChanged },
		contrast = { type="integer", value=128, onChanged = OnVideoParamChanged },
		saturation = { type="integer", value=128, onChanged = OnVideoParamChanged },
		hue = { type="integer", value=128, onChanged = OnVideoParamChanged },
		
		source_crop = OBJECT({
			enable = { type="integer", value=0 },
			cropX = { type="number", value=0 },
			cropY = { type="number", value=0 },
			cropW = { type="number", value=240 },
			cropH = { type="number", value=240 },
		}, OnSourceCropChanged ),

		spec_out = OBJECT({
			format = { type="string", value="default" },
			mode = { type="string", value="default" }
		}, OnSpecOutChanged ),

		out_crop = OBJECT({
			enable = { type="integer", value=0 },
			cropX = { type="number", value=0 },
			cropY = { type="number", value=0 },
			cropW = { type="number", value=0 },
			cropH = { type="number", value=0 },
		}, OnOutCropChanged ),

		RMF = { type="string", value="0", onWrite=SetRMF, onChanged = OnRMFChanged },
	}),

	audio = OBJECT({
		source = { type="string", value="AUTO", onChanged = OnAudioParamChanged },
		line_gain = { type="double", value=0, onChanged = OnAudioParamChanged },	--Gain as DB
		mic_gain = { type="double", value=0, onChanged = OnAudioParamChanged },
		line_bypass = { type="int", value=1, onChanged = OnAudioParamChanged }
	})
})

-------------The End--------------
