local checker = require "statusChecker"

local _M = {}

--The M2 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = true,	--With FPGA
	FPGA_DEV_ADDR = 0xf4,	--FPGA I2C device address
	WITH_FPGA_LEDS = false,	--No FPGA LEDs
	------
	RESET_PIN = 9,	--GPIO9 as RESET button
	
	------
	GPIO_LEDS = {
		["signal"] = 82,
		["link"] = 81
--		["signal"] = 58,
--		["link"] = 57
	},
	
	------
	FPGA_LEDS = {
	},
	
	------
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "signal", "link" },
	ROLLING_LANG_STATUS = {
		[0x01] = "BLINK",	--Default language is CN
		[0x04] = "BLINK-FAST",	--Default language is EN
		[0x00] = "BLINK-FAST"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 2,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	ACTION_STATUS = {
		["default"] = {
			["signal"] = "OFF",
			["link"] = "OFF",
		},
	
		["restore"] = {
			["signal"] = "OFF",
			["link"] = "OFF",
		},
	
		["reboot"] = {
			["signal"] = "OFF",
			["link"] = "OFF",
		},
		
		["reset"] = {
			["signal"] = "BLINK",
			["link"] = "BLINK",
		}
	},
	-------
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

local ctrl_module = nil

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module then
		return
	end
	
	if not checker then
		return
	end
	
	--Signal checking
	local signal, src = checker.CheckSignal()
	if signal then
		ctrl_module.LedControl( "signal", "ON" )
	elseif src == "DETECTING" then
		ctrl_module.LedControl( "signal", "BLINK-FAST" )
	else
		ctrl_module.LedControl( "signal", "OFF" )
	end
	
	local devName, status = checker.CheckEthernetStatus()
	if not devName then
		ctrl_module.LedControl( "link", "OFF" )
	else
		if status == "DOWN" or status == "DISABLED" or status == "NOCARRIE" then
			ctrl_module.LedControl( "link", "OFF" )
		elseif status == "NOIP" then
			ctrl_module.LedControl( "link", "BLINK" )
		elseif status == "UP" then
			ctrl_module.LedControl( "link", "ON" )
		else
			ctrl_module.LedControl( "link", "OFF" )
		end
	end
end

-----------

return _M
