local checker = require "statusChecker"

local _M = {
	count = 0,
	running = false,
}

--The G2 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = true,	--With FPGA
	FPGA_DEV_ADDR = 0xf4,	--FPGA I2C device address
	WITH_FPGA_LEDS	= false,	--With FPGA LEDs
	------
	RESET_PIN = 9,	--GPIO9 as RESET button
	------
	PUSH_STOP_PIN = 13,	--GPIO9 as PUSH_STOP button
	------
	GPIO_LEDS = {
		["stream"] = 58,	--GPIO streaming(cloud) LED
		["signal"] = 57,	--GPIO hdmi(signal) LED
		["wifi"] = 35
	},
	------
	FPGA_LEDS = {
	},
	------
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "signal", "stream" },
	
	ROLLING_LANG_STATUS = {
		[0x01] = "RED:ON",		--Default language is CN
		[0x04] = "GREEN:ON",	--Default language is EN
		[0x00] = "YELLOW:ON"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 1,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	
	ACTION_STATUS = {
		["default"] = {
			["signal"] = "OFF",
			["wifi"] = "OFF",
			["stream"] = "OFF",
		},
	
		["restore"] = {
			["signal"] = "BLINK",
			["wifi"] = "BLINK",
			["stream"] = "BLINK",
		},
	
		["reboot"] = {
			["signal"] = "BLINK",
			["wifi"] = "BLINK",
			["stream"] = "BLINK",
		},
		
		["reset"] = {
			["signal"] = "BLINK",
			["wifi"] = "BLINK",
			["stream"] = "BLINK",
		}
	},
	
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

local ctrl_module = nil

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module then
		return
	end
	
	if not checker then
		return
	end

	-- if ctrl_module.userControl then return end

	_M.count = _M.count + 1
	if _M.count < 40 then
		return
	else
		_M.count = 0
	end	

	if _M.running then
		return
	end
	_M.running = true
	
	--Signal checking
	local signal = checker.CheckSignal()
	if signal then
		ctrl_module.LedControl( "signal", "ON" )
	else
		ctrl_module.LedControl( "signal", "OFF" )
	end

	--Streaming checking
	local status = checker.CheckStreaming()
	local ledStatus
	if status == "UNKNOWN" then
		ledStatus = "OFF"
	elseif status == "STOPED" then
		ledStatus = "OFF"
	elseif status == "STARTED" then
		ledStatus = "ON"
	elseif status == "FAILED" then
		ledStatus = "BLINK"
	else
		ledStatus = "OFF"
	end
	ctrl_module.LedControl( "stream", ledStatus )
	
	--Wifi checking
	local wifistatus = checker.CheckWifiStatus()
	if not wifistatus then
		ctrl_module.LedControl( "wifi", "OFF" )
	else
		if not wifistatus.enable then
			ctrl_module.LedControl( "wifi", "OFF" )
		elseif not wifistatus.started then
			ctrl_module.LedControl( "wifi", "BLINK-FAST" )
		elseif not wifistatus.station then
			ctrl_module.LedControl( "wifi", "BLINK" )
		else
			ctrl_module.LedControl( "wifi", "ON" )
		end
	end
	_M.running = false	
end

-----------

return _M
