#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /data/log/" )
	local logF = roll.Create("/data/log/ioled.log")
	if not logF or not logF:init() then
		_dbg.Error("IOLED", "Fail to open log file: /data/log/ioled.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "ioled" )

_G.print = function(...)
	_dbg.Info1( "IOLED", ... )
end

local module_service
local control

--------------------------------
service.InitSocket()
client.InitSocket( socket, adapter )
--------------------------------

local cosvc_IOLED
local period_timer

--------------------------------
local function periodCheck()
	control.Check( 250 )	-- 250ms
end
--------------------------------
local function Main()
	module_service = require "moduleService"
	control = require "controlModule"

	print( "*** LED/IO status service running ***" )

	cosvc_IOLED = service.New( "ioled", "unix:/var/run/ioled.sock" )
	if cosvc_IOLED then
		_dbg.Info( "IOLED", "CoService of 'ioled' is created." )
		cosvc_IOLED:CreateCoServerProxy()
	else
		_dbg.Error( "IOLED", "Fail created CoService!" )
	end

	module_service.Init( cosvc_IOLED )
	control.Init()
	period_timer = coaf.NewTimer( periodCheck, 0.01, 0.25 ) --After 0.01 sec, and period 250ms
	if period_timer then
		period_timer:start()
	end
end
--------

coaf.Start(Main)

--------
_dbg.Info( "IOLED", "IOLED service is terminated." )


