require "features"	--Get OEM information, for special checks.
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local client = require "coaf.client"

local _M = {}

local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "IOLED", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "IOLED", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

function _M.CheckEthernetStatus()
	--TODO: Check the ethernet status, returns:
	--		devName [ eth0/... ],
	--		status	[DOWN/DISABLED/NOCARRIE/NOIP/UP]
	--return nil, nil
	return "eth0", "NOIP"
end	

function _M.CheckWifiStatus()	
	-- 不亮：WIFI模块故障/获取WIFI状态异常
	-- 灯快闪：AP启动但未正常工作
	-- 灯常亮：AP正常并且STAION已连接
	-- 灯闪烁：AP正常，STAION未连接
	local ap_proxy = GetCoAFProxy( "/raw/HOSTAP", "nm_daemon" )
	if not ap_proxy then
		return nil, "ap_proxy Not initialized"
	end

	local r,enable = ap_proxy:get_Enable()
	if not r then
		ap_proxy:Destroy()
		return nil, "get_Enable error"
	end

	local started
	r,started = ap_proxy:get_Started()
	if not r then
		ap_proxy:Destroy()
		return nil, "get_Started error"
	end
	ap_proxy:Destroy()

	local nm_proxy = GetCoAFProxy( "/raw/NETWORK_MANAGER", "nm_daemon" )
	if not nm_proxy then
		return nil, "nm_proxy Not initialized"
	end

	local wlan0
	r,wlan0 = nm_proxy.GetDevicesBy("wlan0")
	if not r then
		nm_proxy:Destroy()
		return nil, "Get wlan0 status error"
	end
	nm_proxy:Destroy()

	local station = false
	if wlan0 and wlan0[1] and wlan0[1]["State"] and wlan0[1]["State"] == 100 then
		station = true
	end

	local wifistatus ={
		enable = enable,
		started = started,
		station = station,
	}
	return wifistatus
end

function	_M.Check4GStatus()
	--TODO: Check 4G Status, returns:
	--	deviceName[ /dev/ttyUSB0 ],
	--	mode( GSM/CDMA ),
	--	status( IDLE/CONNECTING/CONFIGED/GETIP/CONNECTED/INTERNET_OK/ERROR)
	return nil, nil, nil
end

function	_M.CheckSignal()
	-- returns: signalLocked, SourceName( or "DETECTING" means detecting)
	return true, "default"
end

function	_M.CheckStreaming()
	local svcProxy = GetCoAFProxy( "/", "streamer")
	if not svcProxy then
		_dbg.Error( "IOLED", "Could not get streamer proxy" )
		return "UNKNOWN"
	else
		local r,status = svcProxy:GetPushStatus()
		svcProxy:Destroy()
		if not r then
			_dbg.Error( "IOLED", "Get Push Status failed!" )
			return "UNKNOWN"
		else
			return status
		end
	end
	return "UNKNOWN"
end

function	_M.CheckAudioSource()
	-- returns:	DIGITAL/LINE/MIC/NONE
	return	"NONE"
end

function _M.CheckRunningModules()
	-- returns: an array of running modules
	-- [ AVIO, CODEC, STREAMER, NETWORKMANAGER, HOSTAPM, WEB, ... ]
	return { 
		["AVIO"] = "running", 
		["CODEC"] = "running", 
		["STREAMER"] = "running", 
		["NETWORKMANAGER"] = "running", 
		["HOSTAPM"] = "running",
		["WEB"] = "running"
	}
end

return _M


