local busOprs = require "busOprs"

local devices = busOprs.GetIfDevices("*")	--Get all devices.

--NOTE: IANA-IfTypes:
-- ethernetCsmacd(6), -- for all ethernet-like interfaces,
                      -- regardless of speed, as per RFC3635
-- fast Ethernet(62)
-- gigabitEthernet (117), -- Obsoleted via RFC3635
                          -- ethernetCsmacd (6) should be used instead

-- ppp(23),
-- capwapDot11Profile (252), -- WLAN Profile Interface
-- capwapDot11Bss (253), -- WLAN BSS Interface
-- wwanPP (243), -- 3GPP WWAN
-- wwanPP2 (244), -- 3GPP2 WWAN

local response = {
	NetworkInterfaces = {}
}

local k, v
--First step, check-out all wired interfaces.
for k,v in pairs(devices) do
	if v.TypeName == "ETHERNET" and v.State > 10 then	--State: 10 is unmanaged.
		local item = {
			token = v.Name,
			Enabled = 0,
			Info = {
				Name = v.Name,
				MTU = 1472	--TODO: Assume it always be this.
			},
			
			Link = {
				InterfaceType = 6, --ethernet/AllTypes.
				AdminSettings = {
					AutoNegotiation = 1,
					Speed = 100,
					Duplex = 0, --Full
				},
				
				OperSettings = {
					AutoNegotiation = 1,
					Speed = 100,
					Duplex = 0
				},
			},
			
			IPv4 = {
				Enabled = 0,
				Config = {
					DHCP = 1
				}
			}
		}
		
		if v.State > 30 and v.State ~= 120 then
			item.Enabled = 1
		end
		
		local devInfo = busOprs.GetIfDeviceInfo( v.Object, { WIRE = true, IP4 = true } )
		if devInfo then
			item.Info.HwAddress = devInfo.WIRE.HwAddress
			local ifconfig = busOprs.CheckIfConfig( v.Object )
			if ifconfig then
				item.IPv4.Enabled = 1
				if ifconfig.dhcp then
					item.IPv4.Config.DHCP = 1
					if devInfo.IP4 and devInfo.IP4.Addresses and #devInfo.IP4.Addresses > 0 then
						item.IPv4.Config.FromDHCP = {
							Address = devInfo.IP4.Addresses[1].address,
							PrefixLength = devInfo.IP4.Addresses[1].prefix
						}
					end
				else
					item.IPv4.Config.DHCP = 0
					if ifconfig.ip4addr and ifconfig.ip4addr ~= "" then
						item.IPv4.Config.Manual = {
							Address = ifconfig.ip4addr,
							PrefixLength = ifconfig.prefix
						}
					end
				end
			end
		end
		table.insert( response.NetworkInterfaces, item )
	end
end

--Second step, check-out all wlan interfaces.
for k,v in pairs(devices) do
	if v.TypeName == "WIFI" and v.State > 10 then	--State: 10 is unmanaged.
		local item = {
			token = v.Name,
			Enabled = 0,
			Info = {
				Name = v.Name,
				MTU = 1472	--TODO: Assume it always be this.
			},
			
			Link = {
				InterfaceType = 252, --wifi
				AdminSettings = {
					AutoNegotiation = 1,
					Speed = 54,
					Duplex = 0, --Full
				},
				
				OperSettings = {
					AutoNegotiation = 1,
					Speed = 54,
					Duplex = 0
				},
			},
			
			IPv4 = {
				Enabled = 0,
				Config = {
					DHCP = 1
				}
			}
		}
		
		if v.State > 30 and v.State ~= 120 then
			item.Enabled = 1
		end

		local devInfo = busOprs.GetIfDeviceInfo( v.Object, { WIFI = true, IP4 = true } )
		if devInfo then
			item.Info.HwAddress = devInfo.WIFI.HwAddress
			local ifconfig = busOprs.CheckIfConfig( v.Object, "WIFI" )
			if ifconfig then
				item.IPv4.Enabled = 1
				if ifconfig.dhcp then
					item.IPv4.Config.DHCP = 1
					if devInfo.IP4 and devInfo.IP4.Addresses and #devInfo.IP4.Addresses > 0 then
						item.IPv4.Config.FromDHCP = {
							Address = devInfo.IP4.Addresses[1].address,
							PrefixLength = devInfo.IP4.Addresses[1].prefix
						}
					end
				else
					item.IPv4.Config.DHCP = 0
					if ifconfig.ip4addr and ifconfig.ip4addr ~= "" then
						item.IPv4.Config.Manual = {
							Address = ifconfig.ip4addr,
							PrefixLength = ifconfig.prefix
						}
					end
				end
			end
		end
		table.insert( response.NetworkInterfaces, item )
	end
end

return response

--[==[ *ABSTRACT* The 'response' arguments:
{
	__sizeNetworkInterfaces: <int> (!)
	NetworkInterfaces[...]: (!)
	{
		token: <char*> (!)
		Enabled: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		Info: 
		{
			Name: <char*>
			HwAddress: <char*> (!)
			MTU: <int*>
		}
		Link: 
		{
			AdminSettings: (!)
			{
				AutoNegotiation: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				Speed: <int> (!)
				Duplex: (!)
				(
					tt__Duplex__Full: 0,
					tt__Duplex__Half: 1,
				)
			}
			OperSettings: (!)
			{
				AutoNegotiation: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				Speed: <int> (!)
				Duplex: (!)
				(
					tt__Duplex__Full: 0,
					tt__Duplex__Half: 1,
				)
			}
			InterfaceType: <int> (!)
		}
		IPv4: 
		{
			Enabled: (!)
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			Config: (!)
			{
				__sizeManual: <int>
				Manual[...]: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				LinkLocal: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				FromDHCP: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				DHCP: (!)
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
		}
		IPv6: 
		{
			Enabled: (!)
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			Config: 
			{
				AcceptRouterAdvert: 
				(
					xsd__boolean__false_: 0,
					xsd__boolean__true_: 1,
				)
				DHCP: (!)
				(
					tt__IPv6DHCPConfiguration__Auto: 0,
					tt__IPv6DHCPConfiguration__Stateful: 1,
					tt__IPv6DHCPConfiguration__Stateless: 2,
					tt__IPv6DHCPConfiguration__Off: 3,
				)
				__sizeManual: <int>
				Manual[...]: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				__sizeLinkLocal: <int>
				LinkLocal[...]: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				__sizeFromDHCP: <int>
				FromDHCP[...]: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				__sizeFromRA: <int>
				FromRA[...]: 
				{
					Address: <char*> (!)
					PrefixLength: <int> (!)
				}
				Extension: 
				{
					__size: <int>
					__any: <_XML>
				}
				__anyAttribute: <_XML>
			}
		}
		Extension: 
		{
			__size: <int>
			__any: <_XML>
			InterfaceType: <int> (!)
			__sizeDot3: <int>
			Dot3[...]: 
			{
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			__sizeDot11: <int>
			Dot11[...]: 
			{
				SSID: (!)
				{
					__ptr: <unsigned char*>
					__size: <int>
				}
				Mode: (!)
				(
					tt__Dot11StationMode__Ad_hoc: 0,
					tt__Dot11StationMode__Infrastructure: 1,
					tt__Dot11StationMode__Extended: 2,
				)
				Alias: <char*> (!)
				Priority: <char*> (!)
				Security: (!)
				{
					Mode: (!)
					(
						tt__Dot11SecurityMode__None: 0,
						tt__Dot11SecurityMode__WEP: 1,
						tt__Dot11SecurityMode__PSK: 2,
						tt__Dot11SecurityMode__Dot1X: 3,
						tt__Dot11SecurityMode__Extended: 4,
					)
					Algorithm: 
					(
						tt__Dot11Cipher__CCMP: 0,
						tt__Dot11Cipher__TKIP: 1,
						tt__Dot11Cipher__Any: 2,
						tt__Dot11Cipher__Extended: 3,
					)
					PSK: 
					{
						Key: 
						{
							__ptr: <unsigned char*>
							__size: <int>
						}
						Passphrase: <char*>
						Extension: 
						{
							__size: <int>
							__any: <_XML>
						}
						__anyAttribute: <_XML>
					}
					Dot1X: <char*>
					Extension: 
					{
						__size: <int>
						__any: <_XML>
						__anyAttribute: <_XML>
					}
					__anyAttribute: <_XML>
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
			}
		}
		__anyAttribute: <_XML>
	}
}
]==]

