local timedatectl = require "kiloview.systemd-timedate"
local timeinfo = timedatectl.GetTimeDateInfo()

local function ParseTime(str)
	local y,m,d,h,i,s = str:match( "(%d+)%-(%d+)%-(%d+)%s+(%d+):(%d+):(%d+)" )
	return {
		year = tonumber(y),
		month = tonumber(m),
		day = tonumber(d),
		hour = tonumber(h),
		min = tonumber(i),
		sec = tonumber(s)
	}
end

local local_time = ParseTime( timeinfo["local"] )
local utc_time = ParseTime( timeinfo["utc"] )
--local tz = timeinfo["zone_name"] .. timeinfo["zone_offset"]
local sign,hour,min = timeinfo["zone_offset"]:match( "([%+%-]*)(%d%d)(%d%d)" )
if not sign then
	sign = "+"
elseif sign == "+" then
	sign = "-"
else
	sign = "+"
end

local tz = "GMT" .. sign .. hour .. ":" .. min

return {
	SystemDateAndTime = {
		DateTimeType = tt__SetDateTimeType__Manual,
		DaylightSavings = 0,
		TimeZone = { TZ = tz },
		UTCDateTime = {
			Date = {
				Year = utc_time.year,
				Month = utc_time.month,
				Day = utc_time.day,
			},
			
			Time = {
				Hour = utc_time.hour,
				Minute = utc_time.min,
				Second = utc_time.sec
			}
		},
		
		LocalDateTime = {
			Date = {
				Year = local_time.year,
				Month = local_time.month,
				Day = local_time.day,
			},
			
			Time = {
				Hour = local_time.hour,
				Minute = local_time.min,
				Second = local_time.sec
			}
		}
	}
}

--NOTE: SVC_OK = 0, SVC_NOIMPL = 1

