local args = {...}
local request = args[1]
if not request then
	error( "Invalid request argument passed!" )
end

local profiles = require "profiles"

g__supportResolutions = g__supportResolutions or {
	{ Width=1920, Height=1080, __mainOnly = true },
	{ Width=1280, Height=720, __mainOnly = true },
	{ Width=1024, Height=576 },
	{ Width=960, Height=540 },
	{ Width=800, Height=600 },
	{ Width=854, Height=480 },
	{ Width=720, Height=576 },
	{ Width=720, Height=480 },
	{ Width=720, Height=405, __subOnly = true },
	{ Width=640, Height=480 },
	{ Width=640, Height=360, __subOnly = true },
	{ Width=480, Height=270, __subOnly = true },
	{ Width=352, Height=288, __subOnly = true },
	{ Width=320, Height=240, __subOnly = true },
	{ Width=0, Height=0 }, --That should reset default.
}

g__h264Options = g__h264Options or {
	ResolutionsAvailable = nil,
	GovLengthRange = { Min=1, Max=600 },
	FrameRateRange = { Min=1, Max=60 },
	EncodingIntervalRange = { Min=0, Max=2 },	--TODO: Note the max value.
	H264ProfilesSupported = { tt__H264Profile__Baseline, tt__H264Profile__Main, tt__H264Profile__High },
	BitrateRange = { Min=128, Max=25000 }
}

g__jpegOptions = g__jpegOptions or {
	ResolutionsAvailable = nil,
	
	FrameRateRange = { Min=1, Max=30 },
	EncodingIntervalRange = { Min=5, Max=600 },
	BitrateRange = { Min=512, Max=30000 }
}

local streamId = 0
if request.ProfileToken then
	if request.ProfileToken:match("Main") then
		streamId = 0
	elseif request.ProfileToken:match("Sub") then
		streamId = 1
	else --TODO: JPEG
		streamId = 1
	end
elseif request.ConfigurationToken then
	streamId = tonumber(request.ConfigurationToken) or 0
end

if streamId == 0 then
	local res = {}
	local vss = profiles.GetAllVideoSources()
	local w = vss[1].Resolution.Width or 0
	local h = vss[1].Resolution.Height or 0
	local i
	for i=1, #g__supportResolutions do
		if not g__supportResolutions[i].__subOnly and g__supportResolutions[i].Width <= w and g__supportResolutions[i].Height <= h then
			table.insert( res, g__supportResolutions[i] )
		end
	end
	
	g__h264Options.ResolutionsAvailable = res
	
	return {
		Options = {
			QualityRange = { Min=0, Max=100 },
			H264 = g__h264Options,
			Extension = {
				H264 = g__h264Options
			}
		}
	}
else --TODO: JPEG
	local res = {}
	local vss = profiles.GetAllVideoSources()
	local w = vss[1].Resolution.Width or 0
	local h = vss[1].Resolution.Height or 0
	local i
	for i=1, #g__supportResolutions do
		if not g__supportResolutions[i].__mainOnly and g__supportResolutions[i].Width <= w and g__supportResolutions[i].Height <= h then
			table.insert( res, g__supportResolutions[i] )
		end
	end
	
	g__h264Options.ResolutionsAvailable = res
	
	return {
		Options = {
			QualityRange = { Min=0, Max=100 },
			H264 = g__h264Options,
			Extension = {
				H264 = g__h264Options
			}
		}
	}
end

--[==[ *ABSTRACT* The 'request' arguments:
{
	ConfigurationToken: <char*>
	ProfileToken: <char*>
}
]==]

--[==[ *ABSTRACT* The 'response' arguments:
{
	Options: (!)
	{
		QualityRange: (!)
		{
			Min: <int> (!)
			Max: <int> (!)
		}
		JPEG: 
		{
			__sizeResolutionsAvailable: <int> (!)
			ResolutionsAvailable[...]: (!)
			{
				Width: <int> (!)
				Height: <int> (!)
			}
			FrameRateRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			EncodingIntervalRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
		}
		MPEG4: 
		{
			__sizeResolutionsAvailable: <int> (!)
			ResolutionsAvailable[...]: (!)
			{
				Width: <int> (!)
				Height: <int> (!)
			}
			GovLengthRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			FrameRateRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			EncodingIntervalRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			__sizeMpeg4ProfilesSupported: <int> (!)
			Mpeg4ProfilesSupported[...]: (!)
			(
				tt__Mpeg4Profile__SP: 0,
				tt__Mpeg4Profile__ASP: 1,
			)
		}
		H264: 
		{
			__sizeResolutionsAvailable: <int> (!)
			ResolutionsAvailable[...]: (!)
			{
				Width: <int> (!)
				Height: <int> (!)
			}
			GovLengthRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			FrameRateRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			EncodingIntervalRange: (!)
			{
				Min: <int> (!)
				Max: <int> (!)
			}
			__sizeH264ProfilesSupported: <int> (!)
			H264ProfilesSupported[...]: (!)
			(
				tt__H264Profile__Baseline: 0,
				tt__H264Profile__Main: 1,
				tt__H264Profile__Extended: 2,
				tt__H264Profile__High: 3,
			)
		}
		Extension: 
		{
			__size: <int>
			__any: <_XML>
			JPEG: 
			{
				__sizeResolutionsAvailable: <int> (!)
				ResolutionsAvailable[...]: (!)
				{
					Width: <int> (!)
					Height: <int> (!)
				}
				FrameRateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				EncodingIntervalRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				BitrateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			MPEG4: 
			{
				__sizeResolutionsAvailable: <int> (!)
				ResolutionsAvailable[...]: (!)
				{
					Width: <int> (!)
					Height: <int> (!)
				}
				GovLengthRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				FrameRateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				EncodingIntervalRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				__sizeMpeg4ProfilesSupported: <int> (!)
				Mpeg4ProfilesSupported[...]: (!)
				(
					tt__Mpeg4Profile__SP: 0,
					tt__Mpeg4Profile__ASP: 1,
				)
				BitrateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			H264: 
			{
				__sizeResolutionsAvailable: <int> (!)
				ResolutionsAvailable[...]: (!)
				{
					Width: <int> (!)
					Height: <int> (!)
				}
				GovLengthRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				FrameRateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				EncodingIntervalRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				__sizeH264ProfilesSupported: <int> (!)
				H264ProfilesSupported[...]: (!)
				(
					tt__H264Profile__Baseline: 0,
					tt__H264Profile__Main: 1,
					tt__H264Profile__Extended: 2,
					tt__H264Profile__High: 3,
				)
				BitrateRange: (!)
				{
					Min: <int> (!)
					Max: <int> (!)
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
			}
		}
		__anyAttribute: <_XML>
	}
}
]==]


