local args = {...}
local request = args[1]
if not request then
	error( "Invalid request argument passed!" )
end

local busOprs = require "busOprs"

if request.Configuration.token then
	local streamId = tonumber(request.Configuration.token) or 0
	local sets = {}
	if request.Configuration.Resolution then
		if not request.Configuration.Resolution.Width or request.Configuration.Resolution.Width <= 0 or not request.Configuration.Resolution.Height or request.Configuration.Resolution.Height <= 0 then
			sets["encoder.stream[" .. streamId .. "].video.scale"] = "default"
		else
			sets["encoder.stream[" .. streamId .. "].video.scale"] = request.Configuration.Resolution.Width .. "x" .. request.Configuration.Resolution.Height
		end
	end
	
	if request.Configuration.H264 then
		if request.Configuration.H264.H264Profile == tt__H264Profile__Baseline then
			sets["encoder.stream[" .. streamId .. "].video.profile"] = "baseline"
		elseif request.Configuration.H264.H264Profile == tt__H264Profile__Main then
			sets["encoder.stream[" .. streamId .. "].video.profile"] = "main"
		elseif request.Configuration.H264.H264Profile == tt__H264Profile__High then
			sets["encoder.stream[" .. streamId .. "].video.profile"] = "high"
		end
		
		if request.Configuration.H264.GovLength and request.Configuration.H264.GovLength > 0 then
			sets["encoder.stream[" .. streamId .. "].video.gop_size"] = math.floor(request.Configuration.H264.GovLength)
		end
	end
	
	if request.Configuration.RateControl then
		local interval = 0
		if request.Configuration.RateControl.EncodingInterval then
			if request.Configuration.RateControl.EncodingInterval == 2 then
				sets["encoder.stream[" .. streamId .. "].video.framerate_mode"] = "half"
				interval = 2
			elseif request.Configuration.RateControl.EncodingInterval == 1 then
				sets["encoder.stream[" .. streamId .. "].video.framerate_mode"] = "default"
				interval = 1
			else
				interval = 0
			end
		end
		
		if interval == 0 then
			if request.Configuration.RateControl.FrameRateLimit and request.Configuration.RateControl.FrameRateLimit >= 1 then
				sets["encoder.stream[" .. streamId .. "].video.framerate"] = request.Configuration.RateControl.FrameRateLimit
			end
		end
		
		if request.Configuration.RateControl.BitrateLimit and request.Configuration.RateControl.BitrateLimit > 0 then
			sets["encoder.stream[" .. streamId .. "].video.bitrate"] = request.Configuration.RateControl.BitrateLimit * 1000
		end
	end
	
	if request.Configuration.Quality then
		sets["encoder.stream[" .. streamId .. "].video.quality"] = math.floor(request.Configuration.Quality)
	end
	
	busOprs.Set( sets )
end

return SVC_OK


--TODO: Check the 'request' arguments, and fill the 'response' arguments, see <same-name>.txt file for reference.

--[==[ *ABSTRACT* The 'request' arguments:
{
	Configuration: (!)
	{
		Name: <char*> (!)
		UseCount: <int> (!)
		token: <char*> (!)
		Encoding: (!)
		(
			tt__VideoEncoding__JPEG: 0,
			tt__VideoEncoding__MPEG4: 1,
			tt__VideoEncoding__H264: 2,
		)
		Resolution: (!)
		{
			Width: <int> (!)
			Height: <int> (!)
		}
		Quality: <float> (!)
		RateControl: 
		{
			FrameRateLimit: <int> (!)
			EncodingInterval: <int> (!)
			BitrateLimit: <int> (!)
		}
		MPEG4: 
		{
			GovLength: <int> (!)
			Mpeg4Profile: (!)
			(
				tt__Mpeg4Profile__SP: 0,
				tt__Mpeg4Profile__ASP: 1,
			)
		}
		H264: 
		{
			GovLength: <int> (!)
			H264Profile: (!)
			(
				tt__H264Profile__Baseline: 0,
				tt__H264Profile__Main: 1,
				tt__H264Profile__Extended: 2,
				tt__H264Profile__High: 3,
			)
		}
		Multicast: (!)
		{
			Address: (!)
			{
				Type: (!)
				(
					tt__IPType__IPv4: 0,
					tt__IPType__IPv6: 1,
				)
				IPv4Address: <char*>
				IPv6Address: <char*>
			}
			Port: <int> (!)
			TTL: <int> (!)
			AutoStart: (!)
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			__size: <int>
			__any: <_XML>
			__anyAttribute: <_XML>
		}
		SessionTimeout: <LONG64> (!)
		__size: <int>
		__any: <_XML>
		__anyAttribute: <_XML>
	}
	ForcePersistence: (!)
	(
		xsd__boolean__false_: 0,
		xsd__boolean__true_: 1,
	)
}
]==]

