--The Streaming Applications service
local getset = require "kiloview.getSetHelper"

local CONFIG_FILE = "/data/configs/streamer.conf"
local USER_FILE = "/data/configs/media_users.conf"

require "configuredb"
require "streams_config"
require "medusers_config"

---------------------------------------------------

_G.streamer = CONFIGS.InstanceOf( Streamer_Template, "streamer" )
_G.STREAMS_CONFIG_LOADING = true --Lock to avoid trigger changed events
_G.streamer:LoadFile( CONFIG_FILE )
_G.STREAMS_CONFIG_LOADING = false
_G.streamer:ResetChanges()
--_G.streamer:SaveFile()

_G.media_users = CONFIGS.InstanceOf( MediaUsers_Template, "media_users" )
_G.media_users:LoadFile( USER_FILE )
_G.media_users:ResetChanges()

---------------------------------------------------

local DefaultApp_SVC = {
	Identifier = "DefaultApp",
	FriendlyName = "Encoder default streaming application service",
	Version = "1.0",
	ConfigPrefix = "streamer",
}

function DefaultApp_SVC:Activate()
end

function DefaultApp_SVC:Start()
end

function DefaultApp_SVC:Stop()
end

function DefaultApp_SVC:Reset()
end

function DefaultApp_SVC:Terminate()
	os.exit(0)
end

function DefaultApp_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	else
		return -1, {}
	end
end

function DefaultApp_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function DefaultApp_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function DefaultApp_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function DefaultApp_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function DefaultApp_SVC:RestoreDefault()
	return 0
end

function DefaultApp_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

function DefaultApp_SVC:GetStreams()
	return 3, { 
		["00"] = "Main stream",
		["01"] = "Sub stream",
		["02"] = "Snap stream"
	}
end

function DefaultApp_SVC:GetStreamByID(streamId)
	if streamId == "00" or streamId == "01" or streamId == "02" then
		return "/DefaultApp/" .. streamId
	else
		return "/"	--Means not exists.
	end
end

---------------------------------------------------

local MediaUsers_SVC = {
	Identifier = "MediaUsers",
	FriendlyName = "Media users management",
	Version = "1.0",
	ConfigPrefix = "media_users",
}

function MediaUsers_SVC:Activate()
end

function MediaUsers_SVC:Start()
end

function MediaUsers_SVC:Stop()
end

function MediaUsers_SVC:Reset()
end

function MediaUsers_SVC:Terminate()
end

function MediaUsers_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	else
		return -1, {}
	end
end

function MediaUsers_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function MediaUsers_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function MediaUsers_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function MediaUsers_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function MediaUsers_SVC:RestoreDefault()
	return 0
end

function MediaUsers_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

----------------------------------------------------

local _M = {}

function _M.Init( cosvc )
	if cosvc then
		cosvc:ExposeObject( "/DefaultApp", DefaultApp_SVC )
	end
	print( "DefaultApp service online." )

	if cosvc then
		cosvc:ExposeObject( "/MediaUsers", MediaUsers_SVC )
	end
	print( "MediaUsers service online." )
end

---- Each second checking

function _M.Check()
end

return _M




