require "configuredb"
require "audio_engine_config"
local getset = require "kiloview.getSetHelper"
require "features"
local _ignore, hwChk = pcall( require, "kiloview." .. (_G.DEVICE_TYPE or "G2") .. ".hardwareCheck" )
if not _ignore or not hwChk then
	print( "ERROR: Invalid device hardware checker loaded." )
	hwChk = { hardwareVersionSwCode = 0, Check = function() end }
end

require "kmCore"	--Media core. [ got KM_STREAMING global ]

---------------------

local CONFIG_FILE = "/data/configs/audioEngine.conf"

---------------------

_G.audioEngine = CONFIGS.InstanceOf( AudioEngine_Template, "audioEngine" )
_G.audioEngine:LoadFile( CONFIG_FILE )
_G.audioEngine:ResetChanges()
--_G.streamer:SaveFile()

---------------------

local AudioEngine_SVC = {
	Identifier = "AudioEngine",
	FriendlyName = "Audio Codec Engine",
	Version = "1.0",
	ConfigPrefix = "audioEngine",
}

function AudioEngine_SVC:Activate()
end

function AudioEngine_SVC:Start()
	--TODO
end

function AudioEngine_SVC:Stop()
	--TODO
end

function AudioEngine_SVC:Reset()
end

function AudioEngine_SVC:Terminate()
	os.exit(0)
end

function AudioEngine_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	else
		return -1, {}
	end
end

function AudioEngine_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function AudioEngine_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function AudioEngine_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function AudioEngine_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function AudioEngine_SVC:RestoreDefault()
	return 0
end

function AudioEngine_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

----------------------------------------------------
local _M = {
	audioEncodingEngine = nil,
	encodingChannels = {},	--All audio encoding channels
}
----------------------------------------------------

function _M.Init( cosvc )
	if cosvc then
		cosvc:ExposeObject( "/AudioEngine", AudioEngine_SVC )
	end

	print( "AudioEngine service online." )
	
	_M.audioEncodingEngine = KM_STREAMING.AudioEngine.Encoder.createNew()
	if nil == _M.audioEncodingEngine then
		print( "*** ERROR: Could not create audio encoding engine!" )
		return
	end
	
	hwChk.Check()

	local i
	for i=1, _G.audioEngine.maxEncoderChannels do
		_M.UpdateEncoderChannel( i )
	end
end

----------------------------------------------------
function _M.CheckAudioSource( i )
	if  _M.encodingChannels[i] and  _M.encodingChannels[i].channel then
		--TODO: Check separated audio device
		if hwChk.hardwareVersionSwCode >= 0x20 then --If old version, only support 48KHz sampling.
			local _ignore, info = pcall( dofile, "/proc/avin/0/aformat.lua" )
			if _ignore and info then
				if _M.encodingChannels[i].currentFixedSampling ~= info.sampling then
					print( "***INFO: Audio sampling changed to:", info.sampling )
					_M.encodingChannels[i].currentFixedSampling = info.sampling or 48000
					_M.encodingChannels[i].channel:SetParameters({
						source_fixed_sampling = _M.encodingChannels[i].currentFixedSampling,
						reset = 1 --To reset the capture since sampling is changed.
					})
				end
			end
			info = nil
		end
	end
end

function _M.UpdateEncoderChannel( i )
	if _G.audioEngine.encoders[i].enable and _G.audioEngine.encoders[i].enable ~= 0 then
		local capture = (_G.audioEngine.encoders[i].source.type or "ALSA") .. ":" .. (_G.audioEngine.encoders[i].source.device or "default")
		local codec = _G.audioEngine.encoders[i].encoder.codec or "AAC"
		local resample = _G.audioEngine.encoders[i].source.resample or "fastest"

		print( "Audio encoding engine channel [" .. i .. "] is enabled, try create it. (Capture=" .. capture .. ", Codec=" .. codec .. ")" )
		if not _M.encodingChannels[i] or not _M.encodingChannels[i].channel then
			_M.encodingChannels[i] = _M.encodingChannels[i] or {}
			_M.encodingChannels[i].currentCapture = capture
			_M.encodingChannels[i].currentCodec = codec
			_M.encodingChannels[i].currentResampleType = resample
		else
			if _M.encodingChannels[i].currentCapture ~= capture or _M.encodingChannels[i].currentResampleType ~= resample or _M.encodingChannels[i].currentCodec ~= codec then
				print( "The old channel is exist but new Capture/Codec is changed, so destroy it and re-create it!" )
				--TODO: Notify streams to remove this channel
				_M.encodingChannels[i].channel:Destroy()
				_M.encodingChannels[i].channel = nil
				_M.encodingChannels[i].currentCapture = capture
				_M.encodingChannels[i].currentCodec = codec
				_M.encodingChannels[i].currentResampleType = resample
			end
		end
		
		if nil == _M.encodingChannels[i].channel then
			_M.encodingChannels[i].channel = _M.audioEncodingEngine:CreateEncodingChannel(
				{ type = _G.audioEngine.encoders[i].source.type or "ALSA",
				  device = _G.audioEngine.encoders[i].source.device or "default",
				  resample = resample
			    }, --Capture, ALSA and device

				{ type = codec, bufferSize = 512000, channels = 2 } --Encoding, max channels = 2
				--NOTE: The audio encoding buffer size should be large, if the RTMP network is not stable.
			)
			
			if nil == _M.encodingChannels[i].channel then
				print( "***ERROR: Audio Encoding Engine: Fail to create audio encoding channel, capture = " .. capture .. ", codec = " .. codec )
				return false
			else
				if hwChk.hardwareVersionSwCode < 0x20 then --Old version, only support 48KHz sampling.
					_M.encodingChannels[i].currentFixedSampling = 48000
					_M.encodingChannels[i].channel:SetParameters( {
						source_support_samplings = { 48000 },
						source_fixed_sampling = 48000
					})
				else
					local _ignore, info = pcall( dofile, "/proc/avin/0/aformat.lua" )
					if _ignore and info then
						_M.encodingChannels[i].currentFixedSampling = info.sampling or 48000
					else
						_M.encodingChannels[i].currentFixedSampling = 48000
					end

					_M.encodingChannels[i].channel:SetParameters( {
						source_support_samplings = { 8000,16000,32000,44100,48000,64000,88200,96000,176000,192000 },
						source_fixed_sampling = _M.encodingChannels[i].currentFixedSampling
					})
				end

				print( "Audio encoding channel [" .. i .. "] is created." )
			end
			
			--TODO: Notify streams that this channel is ready
		else
			print( "Audio encoding channel [" .. i .. "] is already exist, and it with same Capture/Codec configure, using the exist one." )
		end
	
		_M.encodingChannels[i].channel:SetParameters( {
			sampling = _G.audioEngine.encoders[i].encoder.sampling or 8000,
			channels = _G.audioEngine.encoders[i].encoder.channels or 1,
			sampleSize = ( codec == "AAC" or codec == "aac" or codec == "FDK-AAC" or codec == "fdk-aac" ) and 1024 or _G.audioEngine.encoders[i].encoder.sampleSize or 256,
			bitrate = _G.audioEngine.encoders[i].encoder.bitrate or 64000,
			aacType = _G.audioEngine.encoders[i].encoder.aac.type or "AAC-LC",
			aacFormat = _G.audioEngine.encoders[i].encoder.aac.format or "RAW",
			g711Law = _G.audioEngine.encoders[i].encoder.g711.law or "ULAW"
		} )

		_M.encodingChannels[i].channel:Start()		--NOTE: Automatically started?
		
	else --Disabled, so destroy() this channel
		print( "Audio encoding channel [" .. i .. "] is disabled." )
		if  _M.encodingChannels[i] and _M.encodingChannels[i].channel then
			print( "Destroy the exist audio encoding channel [" .. i .. "]" )
			_M.encodingChannels[i].channel:Destroy()
			_M.encodingChannels[i].channel = nil
			--TODO: Notify streams that this channel is destroied.
			_M.encodingChannels[i].currentCapture = nil
			_M.encodingChannels[i].currentCodec = nil
		end
	end
	return true
end

----------------------------------------------------

return _M
