local _M = {}
---------------------------------------------------
-- HLS CoAF bus service operations
---------------------------------------------------

local hls_svc_methods = {}
hls_svc_methods.__index = hls_svc_methods

function hls_svc_methods:get_ServiceType()
	return "HTTP/HLS"
end

function hls_svc_methods:ServiceName()
	return "HLS service of stream " .. self.appId .. "-" .. self.streamId
end

function hls_svc_methods:get_Enabled()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].HLS
	if not thisConfig then
		return false
	else
		return _G.streamer.stream[self.cfgId].enabled ~= 0 and thisConfig.enabled ~= 0
	end
end

function hls_svc_methods:get_ServiceUri()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].HLS
	--or _G.streamer.stream[self.cfgId].enabled == 0 or thisConfig.enabled == 0 
	if not thisConfig then
		return "offline"
	elseif self.streamer then
		return "http://%HOST%/hls/" .. self.streamer.session_name .. "/playlist.m3u8"
	else
		return "offline"
	end
end

function hls_svc_methods:Enable(en)
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].HLS
	if not thisConfig then
		return false
	else
		CONFIGS.BeginUpdate()
		thisConfig.enabled = en and 1 or 0
		CONFIGS.Submit()
		return thisConfig.enabled ~= 0 and _G.streamer.stream[self.cfgId].enabled ~= 0
	end
end

function hls_svc_methods:Update()
	local streams = require "streams"
	streams.UpdateStreamService( self.streamId, "Dynamic_" .. string.format( "%02d", self.dynamicIndex ), self.dynamicIndex )
end

function hls_svc_methods:GetStatus()
	if not self.streamer then
		return 1, { ["stream.status"] = "offline" }
	else
		--TODO: More details.
		return 1, { ["stream.status"] = "online" }
	end
end

--------------------------------------------

function _M.ServiceMethods()
	return hls_svc_methods
end

return _M
