local BUSOPR = require "busOprs"
require "interactions"
local cfgM = require "kiloview.configManager"

require "features"
local x, reset = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".reset" )
if not x or not reset then
	x, reset = pcall( require, "kiloview.reset" )
end

local function Reset()
	os.exit(0)
end

local function Reboot()
	if reset then
		print( "*** Reboot the device" )
		reset.Reboot()
	end
end
---------------------------------------------------

local _M = {
	sessions = {},
}

---------------------------------------------------
-- Junshi module
---------------------------------------------------

local JunshiCallbacks = {
	--Login
	["C2S_LOGIN_RES"] = function(sessObj, msg)
		sessObj.loginPending = false
		if msg.iRetCode ~= 0 then
			sessObj.status = "error:AUTH"
			sessObj.hasLogin = false
		else
			sessObj.status = "online"
			sessObj.hasLogin = true
			sessObj.junsSessId = sessObj.junsObj:GetCurrentSession() --Record the newest session Id.
		end
	end,
	
	--Heart beat
	["C2S_HEARTBEAT_RES"] = function(sessObj, msg)
	end,
	
	--Alarm
	["C2S_ALARM_RES"] = function(sessObj, msg)
		--Ignore all.
	end,
	
	--Client-Server stop pushing (Ack)
	["C2S_STOP_PUSH_RES"] = function(sessObj, msg)
		--print( "C2S_STOP_PUSH_RES:", msg.iRetCode == 0 and "DONE." or "FAIL!" )
	end,
	
	--Server-Client start pushing
	["S2C_START_PUSH_REQ"] = function(sessObj, msg)
		--print("S2C_START_PUSH_REQ: Media URL --", msg.cPushUrl )
		if msg.cPushUrl ~= "" then
			local signInfo = BUSOPR.GetSignalStatus()
			if not signInfo then
				sessObj.rtmpPushPendingUrl = nil
				sessObj:_ProcessingPushAck(false)	--Just ack directly
			else
				sessObj.rtmpPushPendingUrl = msg.cPushUrl
				--OK, now get server settings.
				local fr = tonumber(signInfo.video["video.frame_rate"])
				if fr <= 0 then
					fr = 30	--Assume default
				end
				
				sessObj.junsObj:SendMessage( "C2S_GET_PUSH_PARAMETERS_REQ", {
					version = 2,
					cInputVideoFPS = "" .. fr
				})
			end
		else
			sessObj.rtmpPushPendingUrl = nil
			sessObj:_ProcessingPushAck(false)	--Just ack directly
		end
	end,
	
	--Server-Client stop pushing.
	["S2C_STOP_PUSH_REQ"] = function(sessObj, msg)
		--print("Got S2C_STOP_PUSH_REQ.")
		sessObj:_BreakRtmpConnect()
		sessObj.junsObj:SendMessage( "S2C_STOP_PUSH_RES", {
			version = msg.version or 2,
			iRetCode = 0
		})
		--print("S2C_STOP_PUSH_RES acked.")
	end,
	
	--Server-Client add overlay
	["S2C_OVERLAY_ADD_REQ"] = function(sessObj, msg)
		--print("Got S2C_OVERLAY_ADD_REQ.")
		sessObj.junsObj:SendMessage( "S2C_OVERLAY_ADD_RES", {
			version = msg.version or 2,
			iResourcesID = msg.iResourcesID or 0,
			iRetCode = 0
		})
		--print("S2C_OVERLAY_ADD_RES acked.")
	end,
	
	--Server-Client del overlay
	["S2C_OVERLAY_DEL_REQ"] = function(sessObj, msg)
		--print("Got S2C_OVERLAY_DEL_REQ.")
		sessObj.junsObj:SendMessage( "S2C_OVERLAY_DEL_RES", {
			version = msg.version or 2,
			iResourcesID = msg.iResourcesID or 0
		})
		--print("S2C_OVERLAY_DEL_RES acked.")
	end,

	["S2C_CMD_REQ"] = function(sessObj, msg)
		--print("Got S2C_CMD_REQ.")
		if msg.iCMD == 0 then
			sessObj.nofee = true
			sessObj.status = "online:NORES"
			print("***WARNING: No fee!!!")
		elseif msg.iCMD == 1 then
			print("*** NOTIFY: Start recording" )
			sessObj:_StartRecord()
			if sessObj.rtmpSession then
				sessObj.status = "online:STREAM+REC"
			else
				sessObj.status = "online:REC"
			end
		elseif msg.iCMD == 2 then
			print("*** NOTIFY: Stop recording" )
			sessObj:_StopRecord()
			if sessObj.rtmpSession then
				sessObj.status = "online:STREAM"
			else
				sessObj.status = sessObj.nofee and "online:NORES" or "online"
			end
		elseif msg.iCMD == 3 then
			print( "*** NOTIFY: Quick reset" )

			Reset()
		elseif msg.iCMD == 4 then
			print( "*** NOTIFY: Reboot" )
			Reboot()
		end
		
		sessObj.junsObj:SendMessage( "S2C_CMD_RES", {
			version = msg.version or 2,
			iRetCode = 0
		})
		--print("S2C_CMD_RES acked.")
	end,
	
	-- While get pushing parameters setting from server
	["C2S_GET_PUSH_PARAMETERS_RES"] = function(sessObj, msg)
		if msg.iRetCode == 0 then
			BUSOPR.SetJUNSArgs( sessObj.streamId, msg.cParameters )
		end
		sessObj:_ProcessingPushAck(true)
		sessObj.rtmpPushPendingUrl = nil
	end
}

---------------------------------------------------------------------------------------------------

function _M.Init()
	BUSOPR.Init({ local_get_set = {"streamer_global", "streamer", "media_users", "audioEngine"} })
	return true
end

---------------------------------------------------------------------------------------------------
function _M.CreateSession( mediaSource, rtmpPusher, streamId, host, port, secKey, vendorKey )
	if not mediaSource then
		print( "ERROR: Invalid MediaSource specified, so could not create JUNS interaction session!" )
		return nil
	end
	
	if not rtmpPusher then
		print( "ERROR: Invalid RTMP pusher, so could not create JUNS interaction session!" )
		return nil
	end
	
	if not streamId then
		print( "ERROR: Must specify a valid stream-id for JUNS interaction session!" )
		return nil
	end
	
	local streamNum = tonumber(streamId)
	
	local junsObj = INTERACTIONS.JUNSHI.createNew( "JUNS-" .. streamId, host or "device.vgemv.com", port or 9200, cfgM.GetSerialNo() .. ((streamNum==0) and "H" or "L") )
	if not junsObj then
		print( "ERROR: Could not create JUNS interaction object for stream '" .. streamId .. '"' )
		return nil
	end
	
	--To return a session object.
	local sessObj = {
		mediaSource = mediaSource, 
		rtmpPusher = rtmpPusher,
		rtmpPushPendingUrl = nil, --Current pending URL.
		rtmpSession = nil,
		name = "JUNS-" .. streamId, 
		host = host,
		port = port,
		secretKey = secKey or "888888",
		vendorKey = vendorKey or "A0000002",
		streamId = streamId,
		loginPending = false,
		hasLogin = false,
		junsObj = junsObj,
		junsSessId = -1,
		status = "offline",
		nofee = false,
		lastHBAckTime = os.time()
	}
	--------------------------------
	function sessObj:_BreakRtmpConnect()
		if self.rtmpSession then
			self.rtmpPusher:RemoveSession( self.rtmpSession )
			self.rtmpSession = nil
			self.status = self.nofee and "online:NORES" or "online"
		end
	end
	--------------------------------
	function sessObj:_CreateRtmpConnect(uri)
		if self.rtmpSession then
			self:_BreakRtmpConnect()	--Break current connection
		end
		
		self.rtmpSession = self.rtmpPusher:AddSession( "juns", self.mediaSource, uri, { user="", pass="", connTimeout = 10, connIntv = 2 } )
		if self.rtmpSession then
			print( self.name .. ": Created RTMP pushing stream of " .. uri )
			self.status = "online:STREAM"
		else
			print( self.name .. ": Warning! Could not create RTMP pushing stream of " .. uri )
		end
	end
	--------------------------------
	function sessObj:_StopRecord()
		--TODO: Stop recording.
	end
	--------------------------------
	function sessObj:_StartRecord()
		--TODO: Start recording.
	end
	--------------------------------
	function sessObj:_ProcessingPushAck( enable )
		local streamInfo 
		if enable and self.rtmpPushPendingUrl then
			streamInfo = BUSOPR.GetEncoderStreamStatus( self.streamId )
		end
		
		if not enable or not self.rtmpPushPendingUrl or not streamInfo[ "stream.enabled" ] or streamInfo["stream.enabled"] == 0 then
			self.junsObj:SendMessage( "S2C_START_PUSH_RES", {
				version = 2, --NOTE here.
				cVideoFormat = "unknown",
				cVideoDimension = "0x0",
				cVideoFPS = "0.0",
				cVideoBitrate = "0",
				cAudioFormat = "none",
				cAudioSampleRate = "0",
				cAudioChannelCount = "0",
				cAudioBitrate = "0",
				iRetCode = 1
			})
		else
			self:_CreateRtmpConnect( self.rtmpPushPendingUrl )
			--local signal_info = BUSOPR.GetSignalStatus()
			self.junsObj:SendMessage( "S2C_START_PUSH_RES", {
				version = 2, --NOTE here.
				cVideoFormat = "H.264",
				cVideoDimension = streamInfo["video.width"] .. "x" .. streamInfo["video.height"],
				cVideoFPS = streamInfo["video.framerate"],
				cVideoBitrate = streamInfo["video.bitrate"],
				cAudioFormat = streamInfo["audio.codec"],
				cAudioSampleRate = streamInfo["audio.sampling"],
				cAudioChannelCount = streamInfo["audio.channels"],
				cAudioBitrate = streamInfo["audio.bitrate"],
				iRetCode = self.rtmpSession and 0 or 1
			})
		end		
	end
	
	--------------------------------
	function sessObj:Step()	--Step checking current session.
		local stat, msg = self.junsObj:CheckStatus()
		
		if stat == "NORMAL" and not self.loginPending and self.junsSessId ~= self.junsObj:GetCurrentSession() then --That reconnected, so need login.
			self.status = "authorizing"
			self:_BreakRtmpConnect()
			self.hasLogin = false
			self.loginPending = true
			self.junsObj:SendMessage( "C2S_LOGIN_REQ", {
				version = 2,
				cSecretKey = self.secretKey,
				cVendorKey = self.vendorKey
			})
		elseif stat == "MSG_PENDING" then
			if type(msg) ~= "table" or not msg.code then
				print( "***ERROR: Got MSG_PENDING state but no message body/invalid message body! ( ID=" .. self.name .. " )" )
			else
				local cb = JunshiCallbacks[msg.code]
				if type(cb) == "function" then
					cb(self, msg) --Call backs.
					self.lastHBAckTime = os.time()
				else
					print("***WARNING: Unsupported message of '" .. msg.code .. "' ( ID=" .. self.name .. " )" )
				end
			end
			--NOTE: Always clear the pending flag!
			self.junsObj:ClearPending()
		elseif stat == "NORMAL" and self.hasLogin then
			local tack = os.time() - self.lastHBAckTime
			if tack > 6 then --6 seconds (3*2 seconds)
				print( "***ERROR: Long time no received server acking message, close and try re-open it!" )
				self:_BreakRtmpConnect()
				self.hasLogin = false
				self.junsObj:Reset()
			else
				local tta = self.junsObj:GetTimeToLastHeartbeat()
				if tta >= 2000 then --Heartbeat
					self.junsObj:SendMessage( "C2S_HEARTBEAT_REQ", {
						version = 2,
						iStatus = self.rtmpSession and 3 or 5,
						--TODO: Others.
					})
				end
			end
		else
			if stat == "CONNECTING" then
				self.status = "connecting"
			elseif stat == "ERROR" then
				self.status = "error:" .. msg
			elseif stat == "RETRYING" then
				self.status = "retrying"
			end
		end
	end
	
	--------------------------------
	function sessObj:Start()
		self.junsObj:Start()
		if self.status == "offline" then
			self.status = "connecting"
		end
	end
	--------------------------------
	function sessObj:Stop()
		self:_BreakRtmpConnect()
		self.junsObj:Stop()
		self.loginPending = false
		self.hasLogin = false
		self.nofee = false
		self.status = "offline"
	end
	--------------------------------
	function sessObj:GetStatus()
		return self.status
	end
	--------------------------------
	
	table.insert( _M.sessions, sessObj )
	return sessObj
end

function _M.RemoveSession( sess )
	local i
	for i=1, #_M.sessions do
		if _M.sessions[i] == sess then
			sess:Stop()
			sess.mediaSource = nil
			sess.rtmpPusher = nil
			sess.junsObj = nil
			table.remove( _M.sessions, i )
			return
		end
	end	
end

function _M.Step()
	local i
	for i=1, #_M.sessions do
		_M.sessions[i]:Step()
	end
end
---------------------------------------------------
-- Junshi interaction service operations
---------------------------------------------------
local junshi_svc_methods = {}
junshi_svc_methods.__index = junshi_svc_methods

function junshi_svc_methods:get_ServiceType()
	return "INTERACT/JUNSHI"
end

function junshi_svc_methods:ServiceName()
	return "JUNSHI interaction of stream " .. self.appId .. "-" .. self.streamId
end

function junshi_svc_methods:get_Enabled()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].JUNSHI
	if not thisConfig then
		return false
	else
		return _G.streamer.stream[self.cfgId].enabled ~= 0 and thisConfig.enabled ~= 0
	end
end

function junshi_svc_methods:get_ServiceUri()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].JUNSHI
	if not thisConfig then
		return "offline"
	elseif self.session then
		return "juns://" .. thisConfig.host .. ":" .. thisConfig.port
	else
		--return "offline"
		return "juns://" .. thisConfig.host .. ":" .. thisConfig.port
	end
end

function junshi_svc_methods:Enable(en)
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].JUNSHI
	if not thisConfig then
		return false
	else
		CONFIGS.BeginUpdate()
		thisConfig.enabled = en and 1 or 0
		CONFIGS.Submit()
		return thisConfig.enabled ~= 0 and _G.streamer.stream[self.cfgId].enabled ~= 0
	end
end

function junshi_svc_methods:Update()
	local streams = require "streams"
	streams.UpdateStreamService( self.streamId, "Dynamic_" .. string.format( "%02d", self.dynamicIndex ), self.dynamicIndex )
end

function junshi_svc_methods:GetStatus()
	if not self.streamer then
		return 1, { ["stream.status"] = "offline" }
	else
		return 1, { ["stream.status"] = self.session:GetStatus() }
	end
end

--------------------------------------------

function _M.ServiceMethods()
	return junshi_svc_methods
end

return _M

