local _M = {}
---------------------------------------------------
-- RTMP CoAF bus pushing service operations
---------------------------------------------------

local rtmp_push_svc_methods = {}
rtmp_push_svc_methods.__index = rtmp_push_svc_methods

function rtmp_push_svc_methods:get_ServiceType()
	return "RTMP/PUSH"
end

function rtmp_push_svc_methods:ServiceName()
	return "RTMP pushing of stream " .. self.appId .. "-" .. self.streamId
end

function rtmp_push_svc_methods:get_Enabled()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtmp_push
	if not thisConfig then
		return false
	else
		return _G.streamer.stream[self.cfgId].enabled ~= 0 and thisConfig.enabled ~= 0
	end
end

function rtmp_push_svc_methods:get_ServiceUri()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtmp_push
	--or _G.streamer.stream[self.cfgId].enabled == 0 or thisConfig.enabled == 0 
	if not thisConfig then
		return "offline"
	elseif self.streamer then
		return thisConfig.pubPoint
	else
		--return "offline"
		return thisConfig.pubPoint
	end
end

function rtmp_push_svc_methods:Enable(en)
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtmp_push
	if not thisConfig then
		return false
	else
		CONFIGS.BeginUpdate()
		thisConfig.enabled = en and 1 or 0
		CONFIGS.Submit()
		return thisConfig.enabled ~= 0 and _G.streamer.stream[self.cfgId].enabled ~= 0
	end
end

function rtmp_push_svc_methods:Update()
	local streams = require "streams"
	streams.UpdateStreamService( self.streamId, "Dynamic_" .. string.format( "%02d", self.dynamicIndex ), self.dynamicIndex )
end

function rtmp_push_svc_methods:GetStatus()
	if not self.streamer then
		return 1, { ["stream.status"] = "offline" }
	else
		--TODO: More details.
		return 1, { ["stream.status"] = "online" }
	end
end

--------------------------------------------

function _M.ServiceMethods()
	return rtmp_push_svc_methods
end

return _M
