local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

local net_pusher = require "stream.net_pusher"
local juns_m = require "busobj.junshi"

local _M = {
}

----------------------------------------------

local juns_meta = {}

function juns_meta:AddSession( idr, mediaSource, targetAddr, options )
	if not mediaSource or not options then
		_dbg.Error( "JUNS", "Invalid arguments (MediaSource/options ...). Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	if self.session then
		_dbg.Warn( "JUNS", "The JUNS session is already created. Stream:", self.stream_id, ", IDR:", idr )
		return self.session
	end

	self.rtmp_pusher = net_pusher.GetStreamer( self.stream_id )
	if not self.rtmp_pusher then
		_dbg.Error( "JUNS", "Fail to create RTMP pusher for JUNS interaction! Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	self.session = juns_m.CreateSession( mediaSource, self.rtmp_pusher,
										 idr, options.host, options.port, options.secure_key, options.vender_key )
	if not self.session then
		self.rtmp_pusher:Destroy()
		self.rtmp_pusher = nil
		_dbg.Error( "JUNS", "Fail to create JUNS session for Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	_dbg.Info( "JUNS", "Created JUNS session for Stream:", self.stream_id, ", IDR:", idr, ", Address:", options.host, ":", options.port )
	self.session_idr = idr
	return self.session
end

function juns_meta:RemoveSession()
	if self.session then
		juns_m.RemoveSession( self.session )
		_dbg.Info( "JUNS", "Destroied JUNS session for Stream:", self.stream_id, ", IDR:", self.session_idr )
		self.session = nil
		self.session_idr = nil
		self.rtmp_pusher:Destroy()
		self.rtmp_pusher = nil
		collectgarbage()
	end
	return true
end

function juns_meta:StartSession()
	if self.session then
		self.session:Start()
		_dbg.Info( "JUNS", "Started JUNS session for Stream:", self.stream_id, ", IDR:", self.session_idr )
		return true
	else
		_dbg.Error( "JUNS", "No session created while start session, Stream:", self.stream_id )
		return false
	end
end

function juns_meta:StopSession()
	if self.session then
		self.session:Stop()
		_dbg.Info( "JUNS", "Stopped JUNS session for Stream:", self.stream_id, ", IDR:", self.session_idr )
		return true
	else
		_dbg.Error( "JUNS", "No session created while stop session, Stream:", self.stream_id )
		return false
	end
end

function juns_meta:Destroy()
	self:RemoveSession()
end

juns_meta.__index = juns_meta
----------------------------------------------

function _M.GetStreamer( stream_id )
	stream_id = tonumber(stream_id)
	stream_id = string.format( "%02d", stream_id )

	local pusher = {
		stream_id = stream_id
	}

	return setmetatable( pusher, juns_meta )
end

----------------------------------------------
return _M


