local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

----------------------------------------------
local _M = {
	sip_ep = nil,
	sip_ep_details = {
		port = nil,
		account = nil,
		callId = nil,
		autoAnswer = nil,
		media_bind = nil
	},
	current_streamer = nil --NOTE:TODO:Now only support one stream.
}

------------------------------------------------------------------------------------

local function on_incomingCall( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId .. ", AccountID=" .. acc_id .. ", CallID=" .. call_id )
	if acc_id == _M.sip_ep_details.account and not _M.sip_ep_details.callId then
		_M.sip_ep_details.callId = call_id
	else
		print( "*** ERROR: Invalid incoming call account ID or current call in progress!" )
		_M.sip_ep:hangupCall( call_id, 0 )
		return
	end

	if _M.current_streamer and _M.current_streamer.autoAnswer then
		_M.sip_ep:answerCall( call_id, 200 )
		print( "***SIP Auto answered." )
	else
		_M.sip_ep:hangupCall( call_id, 0 )
		print( "***SIP call refused." )
	end
end

------------------------------------------------------------------------------------
local function on_callState( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	--state == DISCONNCTD
	--state == CALLING
	--state == EARLY
	--state == CONNECTING
	if args.state == "DISCONNCTD" then
		if call_id == _M.sip_ep_details.callId then
			print( "*** Call is disconnected." )
			_M.sip_ep_details.callId = nil
			_M.sip_ep_details.media_bind = nil
		end
	elseif args.state == "CONFIRMED" then
		--TODO: Hook media source here? or ...on_updateVideoStream?
	end

	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_mediaState( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	--media_status = "ACTIVECONNECTING"
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_initVideoSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_initAudioSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_startVideoSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_startAudioSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_destroyVideoSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_destroyAudioSubsystem( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function processTx( args )
	--TODO: Here.
	if _M.current_streamer and _M.current_streamer.mediaSource then
		if _M.sip_ep_details.media_bind ~= _M.current_streamer.mediaSource then
			_M.sip_ep:BindMediaSource( _M.current_streamer.mediaSource )
			_M.sip_ep_details.media_bind = _M.current_streamer.mediaSource
		end
	end
end
------------------------------------------------------------------------------------
local function on_updateVideoStream( evId, acc_id, call_id, args, acker )
	print( "***SIP On: updateVideoStream, with accountID=" .. acc_id )
	processTx(args)
	if acker then
		acker:PutMessage( "replace_remote_addr", _G.streamer.sipEp.server )
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_updateAudioStream( evId, acc_id, call_id, args, acker )
	print( "***SIP On: updateAudioStream, with accountID=" .. acc_id )
	processTx(args)

	if acker then
		acker:PutMessage( "replace_remote_addr", _G.streamer.sipEp.server )
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_getVideoCodecAttrs( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	--format_name == "H264" / "PCMU" / ...
	--format_id == "..." (video only)
	--org_dir == "encoding_decoding" (video only)
	if acker then
		acker:Post()
	end
end

------------------------------------------------------------------------------------
local function on_getAudioCodecAttrs( evId, acc_id, call_id, args, acker )
	print( "***SIP ON:" .. evId, acc_id, call_id )
	if acker then
		acker:Post()
	end
end
----------------------------------------------

local sip_meta = {}

function sip_meta:AddSession( idr, mediaSource, targetAddr, options )
	if not mediaSource then
		_dbg.Error( "SIP", "Invalid arguments (MediaSource/options ...). Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	if self.mediaSource then
		_dbg.Warn( "SIP", "The SIP session is already created. Stream:", self.stream_id, ", IDR:", idr )
		return true
	end

	self.mediaSource = mediaSource
	self.callee = targetAddr
	self.autoAnswer = options and options.autoAnswer == 1
	_dbg.Info( "SIP", "Created SIP session for Stream:", self.stream_id, ", IDR:", idr, ", Callee:", self.callee, ", AutoAnswer:", self.autoAnswer )
	self.session_idr = idr
	return true
end

function sip_meta:RemoveSession()
	if self.mediaSource then
		--TODO: Hangup call, ... etc.
		_M.HangupSipCall()
		_M.sip_ep_details.media_bind = nil
		_dbg.Info( "SIP", "Destroied SIP session for Stream:", self.stream_id, ", IDR:", self.session_idr )
		self.mediaSource = nil
		self.callee = nil
		self.autoAnswer = nil
		self.session_idr = nil
		collectgarbage()
	end
	return true
end

function sip_meta:StartSession()
	_dbg.Info( "SIP", "Started SIP session for Stream:", self.stream_id, ", IDR:", self.session_idr )
	return true
end

function sip_meta:StopSession()
	_dbg.Info( "SIP", "Stopped SIP session for Stream:", self.stream_id, ", IDR:", self.session_idr )
	return true
end

function sip_meta:Destroy()
	self:RemoveSession()
	_M.current_streamer = nil
end

sip_meta.__index = sip_meta

----------------------------------------------

function _M.GetStreamer( stream_id )
	if _M.current_streamer then
		_dbg.Error( "SIP", "Current SIP streamer is created, now it only support 1 SIP streamer. For:", stream_id )
		return nil
	end

	stream_id = tonumber(stream_id)
	stream_id = string.format( "%02d", stream_id )

	local streamer = {
		stream_id = stream_id
	}

	_M.current_streamer = setmetatable( streamer, sip_meta )
	return _M.current_streamer
end

----------------------------------------------

function _M.HangupSipCall()
	if _M.sip_ep and _M.sip_ep_details.account then
		if _M.sip_ep_details.callId then
			print( "*** Hanging up current call, ID with:", _M.sip_ep_details.callId )
			_M.sip_ep:hangupCall( _M.sip_ep_details.callId, 0 )
			--_M.sip_ep:hangupAll()
		end
		_M.sip_ep_details.media_bind = nil
		--_M.sip_ep_details.callId = nil
	end
end

----------------------------------------------

function _M.UpdateSipEndpoint()
	_M.HangupSipCall()
	if _M.sip_ep then
		if _G.streamer.sipEp.enabled ~= 1 or  _G.streamer.sipEp.localPort ~= _M.sip_ep_details.port then
			if _M.sip_ep_details.account then
				_M.sip_ep:removeAccount( _M.sip_ep_details.account )
				_M.sip_ep_details.account = nil
			end
			_M.sip_ep = nil
		end
		collectgarbage()
	end

	if _G.streamer.sipEp.enabled ~= 1 then
		return 
	end

	if not _M.sip_ep then
		_M.sip_ep_details.account = nil
		_M.sip_ep = KM_STREAMING.SIPEndpoint.instance(_G.streamer.sipEp.localPort, "REMUX", 3)
		if not _M.sip_ep then
			print( "*** ERROR: Fail to create SIP endpoint!" )
			return
		end

		_M.sip_ep:on("incomingCall", on_incomingCall)
		_M.sip_ep:on("callState", on_callState)
		_M.sip_ep:on("mediaState", on_mediaState)
		_M.sip_ep:onMedia("initVideoSubsystem", on_initVideoSubsystem)
		_M.sip_ep:onMedia("initAudioSubsystem", on_initAudioSubsystem)
		_M.sip_ep:onMedia("startVideoSubsystem", on_startVideoSubsystem)
		_M.sip_ep:onMedia("startAudioSubsystem", on_startAudioSubsystem)
		_M.sip_ep:onMedia("destroyVideoSubsystem", on_destroyVideoSubsystem)
		_M.sip_ep:onMedia("destroyAudioSubsystem", on_destroyAudioSubsystem)
		_M.sip_ep:onMedia("updateVideoStream", on_updateVideoStream)
		_M.sip_ep:onMedia("updateAudioStream", on_updateAudioStream)
		_M.sip_ep:onMedia("getVideoCodecAttrs", on_getVideoCodecAttrs)
		_M.sip_ep:onMedia("getAudioCodecAttrs", on_getAudioCodecAttrs)
	end

	if not _M.sip_ep_details.account then
		_M.sip_ep_details.account = _M.sip_ep:addAccount( _G.streamer.sipEp.localUri,
		{
			register = "sip:" .. _G.streamer.sipEp.server .. ":" .. _G.streamer.sipEp.serverPort,
			realm = _G.streamer.sipEp.realm,
			username = _G.streamer.sipEp.username,
			password = _G.streamer.sipEp.password
		})
		if not _M.sip_ep_details.account then
			print( "***ERROR: Fail to add sip account of '" .. _G.streamer.sipEp.localUri .. "'!" )
		else
			print( "SIP account '" .. _G.streamer.sipEp.localUri .. "' registered." )
		end
	else
		if not _M.sip_ep:modifyAccount( _M.sip_ep_details.account, {
			id = _G.streamer.sipEp.localUri,
			register = "sip:" .. _G.streamer.sipEp.server .. ":" .. _G.streamer.sipEp.serverPort,
			realm = _G.streamer.sipEp.realm,
			username = _G.streamer.sipEp.username,
			password = _G.streamer.sipEp.password
		}) then
			print( "*** WARNING: Fail to modify the SIP account!" )
		else
			print( "!!! SIP account is updated." )
		end
	end
end
----------------------------------------------
function _M.CheckSip()
	if _M.sip_ep then
		if not _M.sip_ep_details.callId then
			if _M.sip_ep_details.account and _M.current_streamer and _M.current_streamer.callee and _M.current_streamer.callee ~= "" then
				_M.sip_ep_details.callId = _M.sip_ep:makeCall( _M.sip_ep_details.account, _M.current_streamer.callee )
				if not _M.sip_ep_details.callId or _M.sip_ep_details.callId < 0 then
					print( "ERROR: Fail to call target:",  _M.current_streamer.callee )
					_M.sip_ep_details.callId = nil
				end
			end
		end
		_M.sip_ep:Polling(2)
	end
end
----------------------------------------------
return _M

