--The Stream service
local thread = require "coaf.thread"
local periphery = require "periphery"
local SERIAL = periphery.Serial
require "features"
------------------------------------------------------------
require "kmCore"	--Media core. [ got KM_STREAMING global ]
------------------------------------------------------------
local audioEngine		= require "audio_engine"
------------------------------------------------------------

local rtsp_server_m		= require "busobj.rtsp_server"
local kpull_server_m	= require "busobj.kmpPull_server"
local hls_m				= require "busobj.hls"
local rtp_push_m		= require "busobj.rtp_push"
local rtmp_push_m		= require "busobj.rtmp_push"
local net_push_m		= require "busobj.net_push"
local kmp_push_m		= require "busobj.kmp_push"
local sproxy_push_m		= require "busobj.sproxy_push"
local ts_push_m			= require "busobj.ts_push"
local juns_m			= require "busobj.junshi"
local sip_m				= require "busobj.sip"
local nil_m				= require "busobj.nil_service"
local srt_push_m		= require "busobj.srt_push"
local omni_push_m		= require "busobj.omni_stream"

------------------------------------------------------------
local recorder			= require "recorder"
------------------------------------------------------------

_G._ENV[ "stream[0].bitrateLimit" ] = 4000000
_G._ENV[ "stream[1].bitrateLimit" ] = 4000000
_G._ENV[ "stream[2].bitrateLimit" ] = 4000000
_G._ENV[ "stream[0].enable_dynamic_bitrate" ] = 0
_G._ENV[ "stream[1].enable_dynamic_bitrate" ] = 0
_G._ENV[ "stream[2].enable_dynamic_bitrate" ] = 0

------------------------------------------------------------

local coaf		= require "coaf.core"
local _dbg		= require "coaf.debug"
local client	= require "coaf.client"
local funcs 	= require "nm_funcs"

------------------------------------------------------------

local stream_rtsp_server	= require "stream.rtsp_server"
local stream_kmp_server		= require "stream.kmp_server"
local stream_net_pusher		= require "stream.net_pusher"
local stream_rtmp_pusher	= require "stream.rtmp_pusher"
local stream_juns			= require "stream.juns"
local stream_sproxy_pusher	= require "stream.sproxy_pusher"
local stream_kmp_pusher		= require "stream.kmp_pusher"
local stream_sip			= require "stream.sip"
local stream_hls			= require "stream.hls"
local stream_omni			= require "stream.omni_pusher"

------------------------------------------------------------

local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli

	cli = client.New( service, loc, nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "STREAMER", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") .. "'" )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "STREAMER", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

------------------------------------------------------------
local streamSvcs = {
	[0] = { idr = "00", srcIdr = "00", bufId = 0 },
	[1] = { idr = "01", srcIdr = "01", bufId = 1 },
	[2] = { idr = "02", srcIdr = "02", bufId = 2 },
}

------------------------------------------------------------
-- The streaming module object.
------------------------------------------------------------
local _M = {
	iav_device = nil,	-- The IAV device.
	jpeg_snapper = nil,
	
	encoderProxies = {},
	stream_objs = {},
	tally_status = {},

	stream_bitrate_accounts = {
		[0] = {
			remain_thres = 5, --TODO: This value should auto detection
			last_remained = 0,
			last_bitrate = 0,
			stable_count = 0
		},
		[1] = {
			remain_thres = 5, --TODO: This value should auto detection
			last_remained = 0,
			last_bitrate = 0,
			stable_count = 0
		},
		[2] = {
			remain_thres = 5, --TODO: This value should auto detection
			last_remained = 0,
			last_bitrate = 0,
			stable_count = 0
		}
	}
}

---------------------------------------------------
-- source operations
---------------------------------------------------

local source_methods = {}
source_methods.__index = source_methods

function source_methods:GetStatus()
	local result = {}
	local cc = -1
	local rslt, arr
	rslt,arr = self:GetVideoStatus()
	if rslt >= 0 then
		cc = rslt
		result = arr
	end
	
	rslt,arr = self:GetAudioStatus()
	if rslt >= 0 then
		cc = (cc < 0 and 0 or cc) + rslt
		local k,v
		for k,v in pairs(arr) do
			result[k] = v
		end
	end
	
	return cc, result
end

function source_methods:GetVideoStatus()
	if not _M.iav_device then
		return -1, {}
	else
		local info = _M.iav_device:GetStatistic( self.bufId )
		if not info then
			return -1, {}
		else
			return 10, info	--TODO: Ensure the counter of info.
		end
	end
end

function source_methods:GetAudioStatus()
	--TODO:
	return -1, {}
end

---------------------------------------------------
-- Construct a stream (stream service)
---------------------------------------------------

local function ConstructStream( appId, streamId, srcUri, cfgId, bufId )

	local STREAM_SVC = {
		--Properties for iModule
		Identifier = "Stream_" .. appId .. "_" .. streamId,
		FriendlyName = "Stream server " .. appId .. "-" .. streamId .. " stream",

		--Properties for iStream
		StreamName = "Stream server " .. appId .. "-" .. streamId .. " stream",
		StreamID = streamId,
		
		ConfigPrefix = "streamer.stream[" .. cfgId .. "]",
		---
		appId = appId,
		streamId = streamId,
		bufId = bufId,

		vCodec = "",
		aCodec = "",
		status = "idle",

		cfgId = cfgId, 

		informations = {
			video = nil,
			audio = nil,
		},

		source = {
			SourceType = "ambarella/IAV",
			SourceName = "Internal encoded stream",
			SourceUri = srcUri,
			WithVideo = false,	--Default status, without video
			WithAudio = false,	--Default status, without audio
			bufId = bufId,
			mediaSource = nil,
		},

		maxDynamicServices = 9,
			
		streamServices = {
			Rtsp_server = {
				streamId = streamId,
				cfgId = cfgId,
				ServiceName = "Rtsp server of stream " .. appId .. "-" .. streamId,
				service = nil,
				currentPort = nil,
				ssm_config = {},
			},

			Kmp_pullingServer = {
				streamId = streamId,
				cfgId = cfgId,
				ServiceName = "Kiloview-Media-Protocol pulling server of stream " .. appId .. "-" .. streamId,
				service = nil,
			},

			-- ["Dynamic_nn"] = {}, total 'maxDynamicServices' items
		},

		--The recorder.
		recorder = nil
	}

	setmetatable( STREAM_SVC.source, source_methods )
	setmetatable( STREAM_SVC.streamServices.Rtsp_server, rtsp_server_m.ServiceMethods() )
	setmetatable( STREAM_SVC.streamServices.Kmp_pullingServer, kpull_server_m.ServiceMethods() )
	
	local i
	for i=0, STREAM_SVC.maxDynamicServices-1 do
		STREAM_SVC.streamServices["Dynamic_" .. string.format("%02d", i) ] = {
			appId = appId,
			streamId = streamId,
			cfgId = cfgId,
			dynamicIndex = i,
			session = nil,
			currentType = nil
		}
		setmetatable( STREAM_SVC.streamServices["Dynamic_" .. string.format("%02d", i) ] , nil_m.ServiceMethods() )	--Default, it's be nil.
	end
	
	---
	function STREAM_SVC:Activate()
	end

	---
	function STREAM_SVC:Start()
	end

	---
	function STREAM_SVC:Stop()
	end

	---
	function STREAM_SVC:Reset()
	end

	---
	function STREAM_SVC:Terminate()
	end

	---
	function STREAM_SVC:Set(params)
		local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
		return cnt
	end

	---
	function STREAM_SVC:Control(action,params)
		if action == "activate" then
			self:Activate()
			return 0, {}
		elseif action == "start" then
			self:Start()
			return 0, {}
		elseif action == "stop" then
			self:Stop()
			return 0, {}
		elseif action == "Reset" then
			self:Reset()
			return 0, {}
		elseif action == "terminate" then
			self:Terminate()
			return 0, {}
		elseif action == "snap" then --Snap the JPEG.
			if _M.jpeg_snapper and params and params["path"] and params["file"] then
				local msrc = _M.stream_objs["02"].source.mediaSource
				if msrc then
					local task = _M.jpeg_snapper:CreateSnapTask( msrc, params["path"], params["file"], 
								 								 { 
								 								 	count = math.floor(params["count"] or 1), 	--default take 1 snap.
								 								 	interval = math.floor(params["interval"] or 0),
																	suffixMask = params["suffix"] or ".jpeg"
																 } )
					if not params["nowait"] then --Default must wait.
						local r = task:WaitFinish(3000)	-- 3 seconds timeout
						task:Free()
						if r == "DONE" then
							return 0, {}
						end
					else --No wait, so return directly.
						return 0, {}
					end
				end
			end
			
			return -1, {}
		else
			return -1, {}
		end
	end

	---
	function STREAM_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
	end

	---
	function STREAM_SVC:Get(inParams)
		local cnt = getset.GetValues( self.ConfigPrefix, inParams )
		return cnt,inParams
	end

	---
	function STREAM_SVC:GetEnv(inParams)
		local cnt = getset.GetValues( "_ENV", inParams )
		return cnt,inParams
	end

	---
	function STREAM_SVC:RestoreDefault()
		--TODO:
		return 0
	end

	---
	--Property for iStream
	function STREAM_SVC:get_Source()	--Get source object id
		return "/" .. self.appId .. "/" .. self.streamId .. "/Source"
	end	

	---
	--Property for iStream
	function STREAM_SVC:get_Services()
		local result = {}
		local k,v
		for k,v in pairs(self.streamServices) do
			result[v:get_ServiceType()] = "/" .. self.appId .. "/" .. self.streamId .. "/" .. k
		end
		return result
	end

	---
	--Method for iStream	
	function STREAM_SVC:GetServiceByType(svcType)
		local k,v
		for k,v in pairs(self.streamServices) do
			if v:get_ServiceType() == svcType then
				return "/" .. self.appId .. "/" .. self.streamId .. "/" .. k
			end
		end
		return "/"	--Means none.
	end

	---
	--Method for iStream	
	function STREAM_SVC:GetAllServicesByType(svcType)
		local result = {}
		local k,v
		for k,v in pairs(self.streamServices) do
			if v:get_ServiceType() == svcType then
				table.insert( result, "/" .. self.appId .. "/" .. self.streamId .. "/" .. k )
			end
		end
		return result
	end

	---
	--Method for iStream	
	function STREAM_SVC:Update()
		_M.UpdateStreamService( self.StreamId, "*" )
	end

	return STREAM_SVC
end

----------------------------------------------------
-- Module initialization
----------------------------------------------------
function _M.Init( cosvc )
	
	recorder.Init() --Initialize the recorder service.

	----------------------------
	local k,v 
	for k,v in pairs(streamSvcs) do
		local svc_obj = ConstructStream( "DefaultApp", v.idr, v.srcIdr, k, v.bufId )
		if not svc_obj then
			print( "WARNING: Could not create stream instance of '" .. v.idr .. "', skip it and so it could not be online." )
		else
			_M.stream_objs[v.idr] = svc_obj

			print( "/DefaultApp/" .. v.idr .. " service online." )
			if cosvc then
				cosvc:ExposeObject( "/DefaultApp/" .. v.idr, svc_obj )
			end

			print( "/DefaultApp/" .. v.idr .. "/Source service online." )
			if cosvc then
				cosvc:ExposeObject( "/DefaultApp/" .. v.idr .. "/Source", svc_obj.source )
			end

			-- Each stream service.
			local n,obj
			for n,obj in pairs(svc_obj.streamServices) do
				print( "/DefaultApp/" .. v.idr .. "/" .. n .. " service online." )
				if cosvc then
					cosvc:ExposeObject( "/DefaultApp/" .. v.idr .. "/" .. n, obj )
				end
			end
		end
	end

	if cosvc then
		--For web service to generate .m3u8 file.
		cosvc:ExposeObject( "/HLS", stream_hls )
	end

	print( "Each encoder service online." )
	
	--Creating AMBA IAV device.
	_M.iav_device = KM_STREAMING.AmbaIAV.createNew( "/dev/iav" )
	if _M.iav_device then
		print( "Created IAV streaming device object." )
	else
		print( "*** ERROR: Fail to create IAV streaming device." )
	end

	if not _G.streamer then
		print( "*** ERROR: Please require 'application' module before initialize 'streams' module. Not found 'streamer' global configures." )
	else
		_M.jpeg_snapper = KM_STREAMING.JPEG.createSnapper( "jpeg-snapper" )
		--TODO: NOTE Here: configure SIP Endpoint.
		stream_sip.UpdateSipEndpoint()
		_M.UpdateConfigures()
	end
end

local openDone, dev
local function serialRxTask()
	if not openDone then
		local status, err = pcall(function()
			_M.serial_port = SERIAL( dev or "/dev/ttyUSB0", 9600)
		end)

		if not status then
			dev = ((dev == "/dev/ttyUSB0") and "/dev/ttyUSB1") or "/dev/ttyUSB0"
		else
			_dbg.Info("SERIAL", "Serial port open done!")
			openDone = true
		end
	else
		if _M.serial_port then
			local status, err = pcall(function()
				if _M.tally_status.TALLY == "on" then
					if _M.tally_status.MAIN == "on" then
						_M.serial_port:write("\x0f")
					else
						_M.serial_port:write("\xf0")
					end
				else
					_M.serial_port:write("\x00")
				end
			end)

			if not status then
				_dbg.Info("SERIAL", "Write to serial_port failed.")
				_M.serial_port = nil
				openDone = nil
			end
		end
	end
end

function _M.Modify_Old_NDI_HX(open)
	local proxy = GetCoAFProxy("/", "onvif", nil, 3)
	if proxy then
		_G.streamer.stream[0].Scrambling.enabled = open and 1 or 0
		_G.streamer.stream[1].Scrambling.enabled = open and 1 or 0

		local is_ok, skip = proxy:Set({
			["onvif.control.discoverPort"] = open and 3705 or 3702,
		})
		
		if is_ok and skip then
			if _M.stream_objs['00'].source.mediaSource then
				local enabled = _G.streamer.stream[0].enabled ~= 0
				_M.UpdateRtspServer( enabled, "00", open )
				_M.UpdateScrambling( "00" )
			end

			if _M.stream_objs['01'].source.mediaSource then
				local enabled = _G.streamer.stream[1].enabled ~= 0
				_M.UpdateRtspServer( enabled, "01", open )
				_M.UpdateScrambling( "01" )
			end
			stream_rtsp_server.EnableAuth( _G.streamer.rtsp_auth ~= 0 )
		end
	end
end

local sub_lock = {}
local ndi_ticket, tally_ticket
function _M.Start_NDI()
	local config = _G.streamer.stream[0].NDI_server
	if config.enabled == 2 then
		_M.Modify_Old_NDI_HX(true)
		return
	end
	_M.Modify_Old_NDI_HX(false)
	if config.enabled ~= 1 then
		return
	end

	local NDIservice = _M.stream_objs['00'].NDIservice
	if not NDIservice then
		NDIservice = KM_STREAMING.ndiMediaSender.createNew()
		if not NDIservice then
			_dbg.Error("NDI", "Could not create NDI service")
			return
		end
		_M.stream_objs['00'].NDIservice = NDIservice
	end

	local serverIP = config.server_addr ~= "" and config.server_addr or nil

	NDIservice:SetOption({
		ttl = "1",
		name = config.device_name,
		group = config.group_name,
		netmask = config.mcast_mask,
		connection = config.connection,
		netprefix = config.mcast_prefix,
		discovery_server = serverIP
	})

	if _M.stream_objs['00'].source.mediaSource then
		NDIservice:AddSession(0, _M.stream_objs['00'].source.mediaSource, {videoBuffer = 4*1024*1024, audioBuffer = 64*1024})
	end
	if _M.stream_objs['01'].source.mediaSource then
		NDIservice:AddSession(1, _M.stream_objs['01'].source.mediaSource, {videoBuffer = 2*1024*1024, audioBuffer = 64*1024})
	end

	NDIservice:StartAllSessions()


	NDIservice:HandleEvent("streamer", "UpdateIFrame")
	NDIservice:HandleEvent("streamer", "SetTallyLED")

	tally_ticket = _G.streamer_event:Subscribe("SetTallyLED", function(_, tbl)
		_M.tally_status[tbl.led] = tbl.state
	end)

	ndi_ticket = _G.streamer_event:Subscribe("UpdateIFrame", function(_, tbl)
		local stream = tbl.stream
		if sub_lock[stream] or sub_lock.all then
			return
		end
		sub_lock[stream] = true
		local idrs = {main ={0}, sub = {1}, all = {0, 1}}
		for _, idr in pairs(idrs[stream]) do
			local proxy = GetCoAFProxy("/Codec/Encoders/" .. string.format("%02d", idr), "codec", nil, 3)
			if proxy then
				proxy:UpdateEncoding("all")
				proxy:Destroy()
			end
		end
		sub_lock[stream] = nil
	end)
end

local old_ip = ""
function _M.UpdateWebUrl()
	local NDIservice = _M.stream_objs['00'].NDIservice
	if not NDIservice then
		return
	end

	local ip_info = funcs.GetLinkDetail("eth0")
	if not ip_info then
		_dbg.Info("NDI", "GetLinkDetail ERROR")
		return
	end

	local ip = ip_info.address 
	if not ip or ip == "" then
		_dbg.Info("NDI", "can not get ip")
		return
	end

	if ip == old_ip then
		return
	end

	local ap_proxy = GetCoAFProxy("/raw/HOSTAP", "nm_daemon")
	if not ap_proxy then
		_dbg.Info("NDI", "can not get nm_daemon proxy and re update ndi after 5 seconds")
		return
	end

	local is_ok, skip, data = ap_proxy:Get({["hostapm.localLink.ip"] = ""})
	ap_proxy:Destroy()
	if not is_ok or not skip then
		_dbg.Info("NDI", "can not get localLink ip and re update ndi after 5 seconds")
		return
	end

	if ip ~= data["hostapm.localLink.ip"] then
		NDIservice:SetWebUrl("http://" .. ip)
		old_ip = ip
	else
		_dbg.Info("NDI", "can not get eth0 ip and re update ndi after 5 seconds")
		return
	end
end
----------------------------------------------
---- Each second checking
function _M.Check()
	recorder.Check()
	if audioEngine then
		audioEngine.CheckAudioSource(1) --TODO: Now only checked channel 1.
	end
	_M.UpdateDynamicBitrate()
end

function _M.check_serial()
	if _G.FEATURE_NDI_HX and _G.streamer.stream[0].NDI_server.enabled == 1 then
		serialRxTask()
	end
end

function _M.check_web_url()
	if _G.FEATURE_NDI_HX and _G.streamer.stream[0].NDI_server.enabled == 1 then
		_M.UpdateWebUrl()
	end
end
------------------------
--[[ a test snap.
	if _M.jpeg_snapper and  _M.stream_objs["02"] and  _M.stream_objs["02"].source then
		local msrc = _M.stream_objs["02"].source.mediaSource	
		if msrc then
			cccc = (cccc + 1) % 10
			print( "****SNAP" )
			local task = _M.jpeg_snapper:CreateSnapTask( msrc, "/tmp", "" .. cccc, { suffixMask = ".jpeg" } )
			print( task:WaitFinish(1000) )
			task:Free()
			task = nil
		end
	end
]]
------------------------
function _M.CheckSip()
	stream_sip.CheckSip()
end
------------------------

function _M.PushControl()
	local pushstarted = false
	local idr,i
	for idr=0,1 do
		for i=0,7 do
			local serviceType = _G.streamer.stream[idr].Dynamic_services[i].serviceType
			if serviceType and serviceType ~= "" then
				local config = _G.streamer.stream[idr].Dynamic_services[i][serviceType]
				if config and config.enabled and config.control_enabled and
					config.control_enabled ~= 0 and config.enabled ~= 0 then
					pushstarted = true
					break
				end
			end
		end
		if pushstarted then
			break
		end
	end
	local en = not pushstarted
	print("Push enable set:",en)
	for idr=0,1 do
		for i=0,7 do
			local serviceType = _G.streamer.stream[idr].Dynamic_services[i].serviceType
			if serviceType and serviceType ~= "" then
				local config = _G.streamer.stream[idr].Dynamic_services[i][serviceType]
				-- pt.dump(config:GetStatus())
				if config and config.control_enabled and config.control_enabled ~= 0 then
					CONFIGS.BeginUpdate()
					config.enabled = en and 1 or 0
					CONFIGS.Submit()
				end
			end
		end
	end
end
------------------------
function _M.GetPushStatus()
	local pt = require "pl.pretty"
	local ret = "STOPED"
	local index,i
	for index=0,1 do
		if _G.streamer.stream[index].enabled ~= 0 then
			for i=0,7 do
				local serviceType = _G.streamer.stream[index].Dynamic_services[i].serviceType
				if serviceType and serviceType ~= "" then
					local config = _G.streamer.stream[index].Dynamic_services[i][serviceType]
					-- print(index, i, serviceType, config.enabled, config.control_enabled)
					if config and config.control_enabled and config.enabled and
						config.control_enabled ~= 0 and config.enabled ~= 0 then
						ret = "STARTED"
						local streamobj = _M.stream_objs[string.format("%02d", index)].streamServices["Dynamic_"..string.format("%02d", i)]
						if streamobj then
							local _,status = streamobj:GetStatus()
							-- pt.dump(status)
							if status and status["stream.status"] ~= "online" then
								return "FAILED"
							end
						end
					end
				end
			end --end for 0,7
		end --end if
	end
	return ret
end
------------------------

function _M.UpdateAudioChannel( thisIdr )
	local i = tonumber(thisIdr)
	--If this stream is enabled.
	if _G.streamer.stream[i].enabled ~= 0 then
		if _M.iav_device then --With valid IAV
			if _M.stream_objs[thisIdr] and _M.stream_objs[thisIdr].source then
				local vCodec
				if _G.DISABLE_VIDEO or _G.streamer.no_video ~= 0 then
					vCodec = ""
				else
					vCodec = "H264"
				end
				local vBufSize = 8*1024*1024
				if i==1 then --Sub stream, so small the video buffer.
					vBufSize = 2.5*1024*1024
				elseif i==2 then --The 'SNAP' channel, so it should be MJPEG
					vCodec = "MJPEG" --TODO: When no video?
				end
				
				local audioChannel = _G.streamer.stream[i].audioChannel or 1
				print( "*** Update to set audio channel = " .. audioChannel )
				if not audioEngine or not audioEngine.encodingChannels[audioChannel] or not audioEngine.encodingChannels[audioChannel].channel then
					_M.iav_device:InitChannel( streamSvcs[i].bufId, false, vCodec, vBufSize, nil )	--No audio
					print( "Initialized IAV channel of '" .. streamSvcs[i].bufId .. "', without audio!" )
				else
					_M.iav_device:InitChannel( streamSvcs[i].bufId, false, vCodec, vBufSize, audioEngine.encodingChannels[audioChannel].channel:GetEncoder() )
					print( "Initialized IAV channel of '" .. streamSvcs[i].bufId .. "', with audio [channel=" .. audioChannel .. "]" )
				end

				--Enable the buffer data-rate measuring.
				_M.iav_device:SetChannelMeasureDelta( streamSvcs[i].bufId, 1000 )

				_M.stream_objs[thisIdr].source.mediaSource = _M.iav_device:GetChannelMediaSource( streamSvcs[i].bufId )

				if not _M.stream_objs[thisIdr].source.mediaSource then
					print( "WARNING: Could not initialize stream channel[" .. i .. "] input source reader! This channel should be closed." )
				end

				_M.UpdateStreamService( thisIdr, "*" )
				
			else
				print( "WARNING: Update stream[" .. i .. "] but service object is incorrect." )
			end
		else
			print( "WARNING: Update streamer[" .. i .. "] but IAV device is not initialized. Nothing did." )
		end
	end
end

------------------------
function _M.UpdateEnableStreaming()
	if _G.NO_STREAMING_WITHOUT_SIGNAL then
		local i
		for i=0,1 do  --Don't disable stream 2 (for SNAP)
			local thisIdr = streamSvcs[i].idr
			if _M.stream_objs[thisIdr] and _M.stream_objs[thisIdr].source and _M.stream_objs[thisIdr].source.mediaSource then
				print( "** Enable/disable streaming:", thisIdr, _G.streamer.enable_streaming )
				_M.stream_objs[thisIdr].source.mediaSource:EnableOutput( _G.streamer.enable_streaming ~= 0 )
			end
		end
	end
end
------------------------
function _M.Update_NDI()
	if not _G.FEATURE_NDI_HX then
		return
	end
	local NDIservice = _M.stream_objs['00'].NDIservice
	if NDIservice then
		NDIservice:StopAllSessions()
		NDIservice:RemoveAllSessions()
		if ndi_ticket then
			_G.streamer_event:Unsubscribe(ndi_ticket)
			ndi_ticket = nil
		end
		if tally_ticket then
			_G.streamer_event:Unsubscribe(tally_ticket)
			tally_ticket = nil
		end
		old_ip = ""
	end
	if _M.stream_objs['00'].source.mediaSource then
		_M.Start_NDI()
	end
end

------------------------
function _M.UpdateConfigures()
	if not _G.streamer then
		return
	end

	print( "~~ Updating all streams configuration ~~" )

	local i
	for i=0,2,1 do

		local thisIdr = streamSvcs[i].idr
		
		--If this stream is enabled.
		if _G.streamer.stream[i].enabled ~= 0 then

			if _M.iav_device then --With valid IAV
				if _M.stream_objs[thisIdr] and _M.stream_objs[thisIdr].source then

					if not _M.stream_objs[thisIdr].source.mediaSource then	--Init and get the IAV media source
						local vCodec
						if _G.DISABLE_VIDEO or _G.streamer.no_video ~= 0 then
							vCodec = ""
						else
							vCodec = "H264"
						end
						local vBufSize = 8*1024*1024
						if i==1 then --Sub stream, so small the video buffer.
							vBufSize = 2.5*1024*1024
						elseif i==2 then --The 'SNAP' channel, so it should be MJPEG
							vCodec = "MJPEG" --TODO: When no video?
						end
						
						local audioChannel = _G.streamer.stream[i].audioChannel or 1
						
						if not audioEngine or not audioEngine.encodingChannels[audioChannel] or not audioEngine.encodingChannels[audioChannel].channel then
							_M.iav_device:InitChannel( streamSvcs[i].bufId, false, vCodec, vBufSize, nil )	--No audio
							print( "Initialized IAV channel of '" .. streamSvcs[i].bufId .. "', without audio!" )
						else
							_M.iav_device:InitChannel( streamSvcs[i].bufId, false, vCodec, vBufSize, audioEngine.encodingChannels[audioChannel].channel:GetEncoder() )
							print( "Initialized IAV channel of '" .. streamSvcs[i].bufId .. "', with audio [channel=" .. audioChannel .. "]" )
						end

						--Enable the buffer data-rate measuring.
						_M.iav_device:SetChannelMeasureDelta( streamSvcs[i].bufId, 1000 )

						_M.stream_objs[thisIdr].source.mediaSource = _M.iav_device:GetChannelMediaSource( streamSvcs[i].bufId )

						if not _M.stream_objs[thisIdr].source.mediaSource then
							print( "WARNING: Could not initialize stream channel[" .. i .. "] input source reader! This channel should be closed." )
						elseif _G.NO_STREAMING_WITHOUT_SIGNAL then
							print( "** Enable/disable streaming:", thisIdr, _G.streamer.enable_streaming )
							_M.stream_objs[thisIdr].source.mediaSource:EnableOutput( _G.streamer.enable_streaming ~= 0 )
						end
					end

					_M.UpdateStreamService( thisIdr, "*" )

				else
					print( "WARNING: Update stream[" .. i .. "] but service object is incorrect." )
				end
			else
				print( "WARNING: Update streamer[" .. i .. "] but IAV device is not initialized. Nothing did." )
			end
			
		else
			--Disabled streamer, so remove all services, then disable the IAV source.
			_M.UpdateStreamService( thisIdr, "*" )	--Update all services, so all services should be disabled.

			if _M.stream_objs[thisIdr].recorder then
				_M.stream_objs[thisIdr].recorder:StopRecord()
				recorder.free( _M.stream_objs[thisIdr].recorder )
				_M.stream_objs[thisIdr].recorder = nil
				--TODO: Notify recordManager.
			end

			if _M.iav_device and _M.stream_objs[thisIdr] and _M.stream_objs[thisIdr].source and _M.stream_objs[thisIdr].source.mediaSource then
				_M.iav_device:DestroyChannelMediaSource( streamSvcs[i].bufId )
				_M.stream_objs[thisIdr].source.mediaSource = nil
			end
		end
	end --For
	if _G.FEATURE_NDI_HX then
		_M.Update_NDI()
	end
	collectgarbage()
end

---------------
local function compareSSMConfigs( old, new )
	if old == new then
		return false
	else
		if not old or not new then
			return true
		end

		if old.ssm ~= new.ssm or
		   old.ttl ~= new.ttl or
		   old.videoAddr ~= new.videoAddr or
		   old.audioAddr ~= new.audioAddr or
		   old.videoPort ~= new.videoPort or
		   old.audioPort ~= new.audioPort
		then
			return true
		else
			return false
		end
	end
end

---------------
function _M.UpdateRtspServer( en, idr, open_ndi_hx )
	local thisSvc = _M.stream_objs[idr]
	local thisConfig = _G.streamer.stream[ tonumber(idr) ].Rtsp_server
	
	local ssm_config = {
		ssm = thisConfig.ssm,
		ttl = thisConfig.multicast_ttl,
		videoAddr = thisConfig.multicast_addr,
		audioAddr = thisConfig.multicast_addr,
		videoPort = thisConfig.multicast_video_port,
		audioPort = thisConfig.multicast_audio_port
	}

	--If changed RTSP port, only take effects after reset soft.
	if thisSvc.streamServices.Rtsp_server.service then
		if not en then
			print( "Disabled RTSP server for stream '" .. idr .. "'" )
			thisSvc.streamServices.Rtsp_server.service:Destroy()
			thisSvc.streamServices.Rtsp_server.service = nil
			thisSvc.streamServices.Rtsp_server.session = nil
			return
		elseif (open_ndi_hx ~= nil) or thisSvc.streamServices.Rtsp_server.session ~= thisConfig.session or compareSSMConfigs(thisSvc.streamServices.Rtsp_server.ssm_config, ssm_config)  then --Updated session
			thisSvc.streamServices.Rtsp_server.service:Destroy()
			thisSvc.streamServices.Rtsp_server.service = nil
			thisSvc.streamServices.Rtsp_server.session = nil
		else
			print( "Update dropping ADTS flag = ", thisConfig.no_adts )
			thisSvc.streamServices.Rtsp_server.service:SetDropADTSHeader(thisConfig.no_adts ~= 0)
			return --Nothing.
		end
	end

	if en then --Enabled
		if not thisSvc.source.mediaSource then
			print( "*** ERROR: Invalid IAV source, so could not start the RTSP server of stream '" .. idr .. "'" )
			return
		end

		if not thisConfig.session or thisConfig.session == "" then
			print( "*** WARNING: Invalid RTSP session of stream '" .. idr .. "', disabled." )
			return
		end

		thisSvc.streamServices.Rtsp_server.service = stream_rtsp_server.GetService( idr, thisConfig )
		if not thisSvc.streamServices.Rtsp_server.service then
			return
		end

		thisSvc.streamServices.Rtsp_server.ssm_config = ssm_config

		local session = thisSvc.streamServices.Rtsp_server.service:AddSession( "", thisSvc.source.mediaSource, thisConfig.session, ssm_config, open_ndi_hx )
		if session then
			thisSvc.streamServices.Rtsp_server.session = thisConfig.session
			thisSvc.streamServices.Rtsp_server.service:SetDropADTSHeader( thisConfig.no_adts ~= 0 )
			thisSvc.streamServices.Rtsp_server.service:StartSession()
		end
	end
end

----------

function _M.UpdateKmpPullingServer( en, idr )
	local thisSvc = _M.stream_objs[idr]
	local thisConfig = _G.streamer.stream[ tonumber(idr) ].Kmp_pullingServer
	en = en and thisConfig.enabled ~= 0

	if thisSvc.streamServices.Kmp_pullingServer.service then
		thisSvc.streamServices.Kmp_pullingServer.service:Destroy()
		thisSvc.streamServices.Kmp_pullingServer.service = nil
		if not en then
			print( "KMP pulling channel disabled for stream '" .. idr .. "'" )
			return
		end
	end
	
	if en then --Enabled
		if not thisSvc.source.mediaSource then
			print( "*** ERROR: Invalid IAV source, so could not start the KMP pulling of stream '" .. idr .. "'" )
			return
		end

		thisSvc.streamServices.Kmp_pullingServer.service = stream_kmp_server.GetService( idr, {bind="*", port="8580"})
		if not thisSvc.streamServices.Kmp_pullingServer.service then
			return
		end

		local did = thisSvc.streamServices.Kmp_pullingServer.service:AddSession( "", thisSvc.source.mediaSource, thisConfig.channel, thisConfig )
		if did then
			thisSvc.streamServices.Kmp_pullingServer.service:StartSession()
		end
	end
end

--------------------------------------------------------------

function _M.UpdateDynamicService( en, idr, thisServiceName, cfgId )
	local thisSvc = _M.stream_objs[idr]
	local thisConfigGroup = _G.streamer.stream[ tonumber(idr) ].Dynamic_services[cfgId]
	
	if not thisSvc.streamServices[ thisServiceName ] then
		print( "***ERROR: Invalid dynamic stream service of '" .. thisServiceName .. "' while updating it." )
		return
	end
	
	local streamService = thisSvc.streamServices[ thisServiceName ]
	
	--print( thisServiceName, "Current type is:", streamService.currentType )
	
	--Drop current stream service first
	if streamService.streamer then
		streamService.streamer:Destroy()
		streamService.streamer = nil
	end

	streamService.currentType = nil
	
	if not thisSvc.source.mediaSource then
		--print( "*** ERROR: Invalid IAV source, so could not start the dynamic streaming service of stream '" .. idr .. "'" )
		return
	end
	
	local serviceType = thisConfigGroup.serviceType
	local thisConfig = thisConfigGroup[ serviceType ]
	en = en and thisConfig.enabled ~= 0

	if serviceType == "Rtmp_push" then
		if en then
			_M._CreateRtmpPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, rtmp_push_m.ServiceMethods() )
	elseif serviceType == "Rtp_push" then
		if en then
			_M._CreateRtpPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, rtp_push_m.ServiceMethods() )
	elseif serviceType == "Net_push" then --New: General network pusher.
		if en then
			_M._CreateNetPushing( thisSvc, idr, streamService, thisConfig, thisConfigGroup )
		end
		setmetatable( streamService, net_push_m.ServiceMethods() )
	elseif serviceType == "Kmp_push" then
		if en then
			_M._CreateKmpPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, kmp_push_m.ServiceMethods() )
	elseif serviceType == "TS_push" then
		if en then
			_M._CreateTSPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, ts_push_m.ServiceMethods() )
	elseif serviceType == "HLS" then
		if en then
			_M._CreateHLS( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, hls_m.ServiceMethods() )
	elseif serviceType == "Sproxy_push" then
		if en then
			_M._CreateSproxyPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, sproxy_push_m.ServiceMethods() )
	elseif serviceType == "JUNSHI" then
		if en then
			_M._CreateJUNSHI( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, juns_m.ServiceMethods() )
	elseif serviceType == "Omnistream" then
		if en then
			_M._CreateOMNI( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, omni_push_m.ServiceMethods() )
	elseif serviceType == "SIP" then
		if en then
			_M._CreateSIP( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, sip_m.ServiceMethods() )
	elseif serviceType == "Srt_push" or serviceType == "SRT_push" then
		if en then
			_M._CreateSrtPushing( thisSvc, idr, streamService, thisConfig )
		end
		setmetatable( streamService, srt_push_m.ServiceMethods() )
	else
		print( "No service type specified or invalid service type, current dynamic stream service is disabled. [stream: " .. idr ..", service: " .. thisServiceName ..", type=" .. serviceType .. "]" )
		streamService.currentType = nil
		setmetatable( streamService, nil_m.ServiceMethods() )
		return
	end
	streamService.currentType = serviceType
end

-------------------------------------------------
--Create SIP pushing service
-------------------------------------------------
function _M._CreateSIP( thisSvc, idr, streamService, thisConfig )
	streamService.streamer = stream_sip.GetStreamer(idr)
	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.callAccount, {
			autoAnswer = thisConfig.autoAnswer
		})
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
--Create RTMP pushing service
-------------------------------------------------
function _M._CreateRtmpPushing( thisSvc, idr, streamService, thisConfig )
	if not thisConfig.pubPoint or thisConfig.pubPoint == "" then
		print( "*** WARNING: Invalid publish point of RTMP pushing, disabled." )
		return
	end

	--Compatible mode, so use the old version RTMP pusher.
	if thisConfig.compatible_mode ~= 0 then
		streamService.streamer = stream_rtmp_pusher.GetStreamer(idr)
	--Otherwise(default), use the new network pusher.
	else
		streamService.streamer = stream_net_pusher.GetStreamer(idr)
	end

	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
												user=thisConfig.username, pass=thisConfig.password, 
												connTimeout = thisConfig.timeout,
												connIntv = thisConfig.reconnect_interval, withMeta = true,
												enableNtp = thisConfig.ntp_timestamp or -1 })
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
-- Get the TS muxrate.
-------------------------------------------------

--Note here:
--Since the proxy is dynamic created and dynamic destroied, so it's not a problem
--for re-entry.
function _M.GetTsMuxrate( streamId )
	streamId = tonumber(streamId)
	local muxrate = -1
	local encoders_proxy = GetCoAFProxy( "/Codec/Encoders", "codec", "local", 5 )
	if encoders_proxy then
		local pfx = "encoder.stream[" .. streamId .. "].video."
		local r,cc,gets = encoders_proxy:Get({ 	[ pfx .. "br_ctrl" ] = "cbr",
												[ pfx .. "bitrate" ] = 2000000 })
		encoders_proxy:Destroy()
		if r and gets then
			--if gets[ pfx .. "br_ctrl" ] == "cbr" or gets[ pfx .. "br_ctrl" ] == "CBR" then
				muxrate = gets[pfx .. "bitrate"] or -1
				if muxrate > 0 then
					muxrate = muxrate + 128000 --Consider audio, and give it over spaces.
				end
			--end
		end
	end
	--print( "***** MuxRate=", muxrate )
	return muxrate
end

-------------------------------------------------
--Create general network pushing service
-------------------------------------------------
function _M._CreateNetPushing( thisSvc, idr, streamService, thisConfig, thisConfigGroup )
	if not thisConfig.pubPoint or thisConfig.pubPoint == "" then
		print( "*** WARNING: Invalid push point of general network pushing, disabled." )
		return
	end

	local pushType = thisConfig.pubPoint:match("^(.-)://")
	if not pushType then
		print( "*** WARNING: Invalid push point of general network pushing, disabled." )
		return
	end

	pushType = pushType:lower()

	streamService.streamer = stream_net_pusher.GetStreamer(idr)
	if streamService.streamer then
		if pushType == "udp" then
			local muxrate = _M.GetTsMuxrate(idr)
			local tsConfig = thisConfigGroup[ "TS_push" ]
			local ts_null_multiple = tsConfig.ts_null_multiple
			if not ts_null_multiple or ts_null_multiple <= 1 then
				muxrate = -1
				ts_null_multiple = 1.0
			end
			streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
												ttl = tsConfig.ttl,
												muxrate = muxrate,
												ts_transport_stream_id = tsConfig.ts_transport_stream_id,
												ts_pts_pcr_delay = tsConfig.ts_pts_pcr_delay,
												ts_pmt_start_pid = tsConfig.ts_pmt_start_pid,
												ts_start_pid = tsConfig.ts_start_pid,
												ts_tables_version = tsConfig.ts_tables_version,
												ts_service_name = tsConfig.ts_service_name,
												ts_service_provider = tsConfig.ts_service_provider,
												ts_null_multiple = tsConfig.ts_null_multiple,
												ts_pcr_period = tsConfig.ts_pcr_period })
		elseif pushType:match("^rtmp") then
			local rtmpConfig = thisConfigGroup[ "Rtmp_push" ]
			streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
												user=rtmpConfig.username, pass=rtmpConfig.password, 
												connTimeout = rtmpConfig.timeout,
												connIntv = rtmpConfig.reconnect_interval, withMeta = true,
												enableNtp = rtmpConfig.ntp_timestamp or -1 })
		elseif pushType == "rtp" then
			local rtpConfig = thisConfigGroup[ "Rtp_push" ]
			streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
				--TODO: what args?
			})
		elseif pushType == "srt" then
			local srtConfig = thisConfigGroup[ "SRT_push" ]
			if srtConfig.ts_null_multiple <= 1 then
				muxrate = -1
			end
			--add by kiloview@20204-23  srt streamid and  set ts pata
			streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
												srt_connect_mode = srtConfig.srt_connect_mode,
												srt_encryption_key_len = _M.GetAesKeyLen(srtConfig.srt_encryption_key_len),
												srt_encryption_key = srtConfig.srt_encryption_key,
												srt_latency = srtConfig.srt_latency,									
												srt_bandwidth_overhead = srtConfig.srt_bandwidth_overhead * 0.01,
												srt_payload_size = srtConfig.srt_payload_size,
												srt_stream_id = srtConfig.srt_stream_id,
												muxrate = muxrate,
												ts_null_multiple = srtConfig.ts_null_multiple,
												ts_pcr_period = srtConfig.ts_pcr_period,
												ts_pts_pcr_delay = srtConfig.ts_pts_pcr_delay,
												ts_transport_stream_id = srtConfig.ts_transport_stream_id
											}) 																					 
		else --TODO: rtsp push? ...
			streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
			})
		end
		streamService.streamer:StartSession()
	end
end
-------------------------------------------------
--Create RTP pushing service
-------------------------------------------------

function _M._CreateRtpPushing( thisSvc, idr, streamService, thisConfig )
	--TODO: RTP
end

-------------------------------------------------
--Create Sproxy pushing service
-------------------------------------------------

function _M._CreateSproxyPushing( thisSvc, idr, streamService, thisConfig )
	if not thisConfig.pubPoint or thisConfig.pubPoint == "" then
		print( "*** WARNING: Invalid publish point of Sproxy pushing, disabled." )
		return
	end

	streamService.streamer = stream_sproxy_pusher.GetStreamer(idr)
	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.pubPoint, { 
												user=thisConfig.username, pass=thisConfig.password, 
												connTimeout = thisConfig.timeout,
												connIntv = thisConfig.reconnect_interval })
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
--Create TS pushing service
-------------------------------------------------

function _M._CreateTSPushing( thisSvc, idr, streamService, thisConfig )

	local i, addr, port, ttl
	local ts_pts_pcr_delay, ts_transport_stream_id, ts_pmt_start_pid, ts_start_pid, ts_tables_version, ts_service_name, ts_service_provider, ts_null_multiple
	local ts_pcr_period
	for i=1,8 do
		if thisConfig.targets[i].address ~= "" and thisConfig.targets[i].port > 0 then
			addr = thisConfig.targets[i].address
			port = thisConfig.targets[i].port
			ttl = thisConfig.targets[i].ttl
			ts_pts_pcr_delay = thisConfig.targets[i].ts_pts_pcr_delay
			ts_transport_stream_id = thisConfig.targets[i].ts_transport_stream_id
			ts_pmt_start_pid = thisConfig.targets[i].ts_pmt_start_pid
			ts_start_pid = thisConfig.targets[i].ts_start_pid
			ts_tables_version = thisConfig.targets[i].ts_tables_version
			ts_service_name = thisConfig.targets[i].ts_service_name
			ts_service_provider = thisConfig.targets[i].ts_service_provider
			ts_null_multiple = thisConfig.targets[i].ts_null_multiple
			ts_pcr_period = thisConfig.targets[i].ts_pcr_period
			break --TODO: Now only support 1 target.
		end
	end

	if not addr then
		print( "*** WARNING: Invalid TS pushing target address/port, disabled." )
		return
	end

	local muxrate = _M.GetTsMuxrate(idr)
	if ts_null_multiple <= 1 then
		muxrate = -1
		ts_null_multiple = 1.2
	end

	streamService.streamer = stream_net_pusher.GetStreamer(idr)
		
	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, "udp://" .. addr .. ":" .. port, { 
												ttl = ttl,
												muxrate = muxrate,
												ts_pts_pcr_delay = ts_pts_pcr_delay,
												ts_transport_stream_id = ts_transport_stream_id,
												ts_pmt_start_pid = ts_pmt_start_pid,
												ts_start_pid = ts_start_pid,
												ts_tables_version = ts_tables_version,
												ts_service_name = ts_service_name,
												ts_service_provider = ts_service_provider,
												ts_null_multiple = ts_null_multiple,
												ts_pcr_period = ts_pcr_period
											})
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
-- Get the AES Key Len.
-------------------------------------------------

function _M.GetAesKeyLen( aesName )
	local aesKeyLen
	if aesName == "AES-128" then
		aesKeyLen = 16
	elseif aesName == "AES-192" then
		aesKeyLen = 24
	elseif aesName == "AES-256" then
		aesKeyLen = 32
	else
		aesKeyLen = 0
	end
	return aesKeyLen
end

-------------------------------------------------
--Create SRT pushing service --add by youji@2019-07-15
-------------------------------------------------

function _M._CreateSrtPushing( thisSvc, idr, streamService, thisConfig )

	local i, addr, port, srt_latency, key_lenth
	local srt_connect_mode, srt_encryption_key_len, srt_encryption_key, srt_bandwidth_overhead, srt_payload_size
	--Set Dev listener mode ,but address is null
	--if thisConfig.address ~= "" and thisConfig.port > 0 then
	if thisConfig.srt_connect_mode ~= "" and thisConfig.port > 0 then
		addr = thisConfig.address
		port = thisConfig.port
		srt_connect_mode = thisConfig.srt_connect_mode
		srt_encryption_key_len = thisConfig.srt_encryption_key_len
		srt_encryption_key = thisConfig.srt_encryption_key
		srt_latency = thisConfig.srt_latency
		srt_bandwidth_overhead = thisConfig.srt_bandwidth_overhead
		srt_payload_size = thisConfig.srt_payload_size
	end

	if not addr then
		print( "*** WARNING: Invalid SRT pushing target address/port, disabled." )
		return
	end
   
	key_lenth = #srt_encryption_key
    if srt_encryption_key_len ~="none" then 
		if key_lenth == 0 then
    		print( "*** WARNING: Invalid SRT PARA encryption_key/key_len, disabled. [key-Val is NULL]" )
    		srt_encryption_key_len = "none"
		elseif key_lenth < 10 then
			print( "*** ERROR: Invalid SRT PARA srt_encryption_key disabled. [Key-Val length less than 16]" )
			return
		end
	end
	
	local muxrate = _M.GetTsMuxrate(idr)
	local ts_null_multiple = thisConfig.ts_null_multiple
	if not ts_null_multiple or ts_null_multiple <= 1 then
		muxrate = -1
		ts_null_multiple = 1.0
	end

	streamService.streamer = stream_net_pusher.GetStreamer(idr)
	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, "srt://" .. addr .. ":" .. port, { 
												srt_connect_mode = srt_connect_mode,
												srt_encryption_key_len = _M.GetAesKeyLen(srt_encryption_key_len),
												srt_encryption_key = srt_encryption_key,
												srt_latency = srt_latency,									
												srt_bandwidth_overhead = srt_bandwidth_overhead * 0.01,
												srt_payload_size = srt_payload_size,
												srt_stream_id = thisConfig.srt_stream_id,
												muxrate = muxrate,
												ts_null_multiple = ts_null_multiple,
												ts_pcr_period = thisConfig.ts_pcr_period,
												ts_pts_pcr_delay = thisConfig.ts_pts_pcr_delay,
												ts_transport_stream_id = thisConfig.ts_transport_stream_id
											})
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
--Create HLS service
-------------------------------------------------
function _M._CreateHLS( thisSvc, idr, streamService, thisConfig )
	if not thisConfig.session or thisConfig.session == "" then
		print( "*** WARNING: Invalid HLS session name, disabled." )
		return
	end
	streamService.streamer = stream_hls.GetStreamer(idr)
	if streamService.streamer then
		--Default location is "/tmp/hls", or change global config of stream_hls.
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.session, { 
												segmentTime = thisConfig.segmentTime })
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
--Create JUNSHI interaction service
-------------------------------------------------

function _M._CreateJUNSHI( thisSvc, idr, streamService, thisConfig )
	if not thisConfig.host or thisConfig.host == "" then
		print( "*** WARNING: Invalid host for JUNS interaction, disabled." )
		return
	end

	streamService.streamer = stream_juns.GetStreamer(idr)
	if streamService.streamer then
		--Default location is "/tmp/hls", or change global config of stream_hls.
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, "", thisConfig )
		streamService.streamer:StartSession()
	end
end

-------------------------------------------------
--Create OMNI service
-------------------------------------------------

function _M._CreateOMNI( thisSvc, idr, streamService, thisConfig )
	if not thisConfig.address or thisConfig.address == "" then
		print( "*** WARNING: Invalid address for Omnistream, disabled." )
		return
	end

	streamService.streamer = stream_omni.GetStreamer(idr)
	if streamService.streamer then
		streamService.streamer:AddSession( streamService.dynamicIndex, thisSvc.source.mediaSource, thisConfig.address or "<none>", {
			PushMode = "NONE",
			udp_tar_ip = thisConfig.address,
			video_port = thisConfig.video_port,
			audio_port = thisConfig.audio_port
		})
		streamService.streamer:StartSession()
	end
end

----------
function _M.UpdateScrambling( idr )
	if _G.FEATURE_NEWTEK or _G.FEATURE_NDI_HX then
		local thisSvc = _M.stream_objs[idr]
		local thisConfig = _G.streamer.stream[ tonumber(idr) ].Scrambling
		if thisSvc and thisSvc.streamServices.Rtsp_server.service then
			thisSvc.streamServices.Rtsp_server.service:EnableScrambling( thisConfig.enabled ~= 0, { seed = thisConfig.seed, depth = thisConfig.depth } )
		end
	end
end

----------------------------------------------

function _M.UpdateStreamService( idr, serviceType, dynamicIndex )
	local enabled = _G.streamer.stream[tonumber(idr)].enabled ~= 0

	if not serviceType or serviceType == "*" or serviceType == "Rtsp_server" then
		_M.UpdateRtspServer( enabled, idr )
		stream_rtsp_server.EnableAuth( _G.streamer.rtsp_auth ~= 0 )
		-- _M.UpdateScrambling( idr ) --NOTE: Now scrambling is just only for RTSP service; not necessary after NDI-HX
	end
	
	if not serviceType or serviceType == "*" or serviceType == "Kmp_pullingServer" then
		_M.UpdateKmpPullingServer( enabled, idr )
	end

	if not serviceType or serviceType == "*" then
		local streamObj = _M.stream_objs[idr]
		if not streamObj then
			print( "***WARNING: Invalid stream object of '" .. idr .. "', could not update dynamic services!" )
			return
		end
		
		local i
		for i=0,streamObj.maxDynamicServices-1  do
			_M.UpdateDynamicService( enabled, idr, "Dynamic_" .. string.format( "%02d", i), i )
		end
	elseif serviceType:match("Dynamic_") then
		--print( "Update dynamic service: ", enabled, idr, serviceType, dynamicIndex )
		_M.UpdateDynamicService( enabled, idr, serviceType, dynamicIndex or 0 )
	end
end

----------------------------------------------
-- Dynamic bitrate calculation routines.
----------------------------------------------

local function AssignDynamicBitrate( stream_id, br )
	if br then
		br = math.ceil(br/4000) * 4000
		if br <= 16000 then --Keep 16Kbps at least.
			br = 16000
		end
	end

	if br == _M.stream_bitrate_accounts[stream_id].last_bitrate then
		return false, br
	end
	_M.stream_bitrate_accounts[stream_id].last_bitrate = br
	return true, br
end
----------------------------------------------
function _M.UpdateDynamicBitrate()
	if not _G.streamer then
		return
	end

	local i
	for i=0,2,1 do
		--local thisIdr = streamSvcs[i].idr
		--If this stream is enabled.
		local changed = false
		if _G.streamer.stream[i].enabled ~= 0 and _G._ENV[ "stream[" .. i .. "].enable_dynamic_bitrate" ] ~= 0 then --{
			if not _M.encoderProxies[i] then
				_M.encoderProxies[i] = GetCoAFProxy( "/Codec/Encoders/" .. string.format("%02d",i), "codec", "local", 2 ) -- 2 second timeout
				if not _M.encoderProxies[i] then
					break --So looks like with wrong to communicate with encoder.
				end
			end

			local vrate, arate, vremain, aremain = _M.iav_device:GetSlowestRateInfo(i)
			if vrate > 0 or vremain > 0 then --{
				vrate = vrate*8
				local br_limit = _G._ENV[ "stream[" .. i .. "].bitrateLimit" ]
				if not _M.stream_bitrate_accounts[i].last_bitrate or _M.stream_bitrate_accounts[i].last_bitrate <= 0 then
					changed = AssignDynamicBitrate(i, br_limit)
				end

				if vrate >= br_limit then --Looks no bottleneck, but check remained packets to verify:
				-- 1) If remained packets is large or incrementation, should decrement bitrate to avoid full.
				-- 2) If remained packets is stable, that no need ajdust.
					if _M.stream_bitrate_accounts[i].last_remained >= _M.stream_bitrate_accounts[i].remain_thres and vremain > (_M.stream_bitrate_accounts[i].last_remained+1) then
						local br = math.floor(br_limit*0.9) --Decrement 10%
						--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = br
						changed = AssignDynamicBitrate(i, br)
					else
						--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = br_limit
						changed = AssignDynamicBitrate(i, br_limit)
					end
					_M.stream_bitrate_accounts[i].stable_count = 0
				else --{
				--3 cases:
				-- 1) actual bitrate is slow than set (br_limit), so check remained packets to verify that.
				-- 2) bandwidth bottleneck, in this case, remained packets may be large or incrementation
				-- 3) If remained packets count is stable (and small), try increment bitrate to near br_limit.
						--??? if vremain >= 5 then --Min: (5-2):3*16.66ms ~= 50ms/5*16.66ms ~= 83ms, Max maybe: 5*40ms = 200ms.
					
					--Buffer is large, so reduce bitrate as soon as possible.
					if _M.stream_bitrate_accounts[i].last_remained >= _M.stream_bitrate_accounts[i].remain_thres then
						if vremain > (_M.stream_bitrate_accounts[i].last_remained+2) then --Incrementation!
							local br = _M.stream_bitrate_accounts[i].last_bitrate
							if br > vrate then
								br = vrate
							end
							--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = br
							changed = AssignDynamicBitrate(i, br)
						--otherwise, decrement or not increment, keep go on.
						end
						_M.stream_bitrate_accounts[i].stable_count = 0
					elseif vremain <= _M.stream_bitrate_accounts[i].last_remained then --Stable, maybe we need increment some.
						_M.stream_bitrate_accounts[i].stable_count = _M.stream_bitrate_accounts[i].stable_count + 1
						if _M.stream_bitrate_accounts[i].stable_count >= 3 then
							_M.stream_bitrate_accounts[i].stable_count = 1 --Next try increment, let it be more fast
							local br = _M.stream_bitrate_accounts[i].last_bitrate
							if br < vrate then
								br = vrate
							end
							br = math.floor(br*1.1) --Increment 10%
							if br > br_limit then br = br_limit end

							--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = br
							changed = AssignDynamicBitrate(i, br)
						end
					else
						_M.stream_bitrate_accounts[i].stable_count = 0
					end
				--}
				end

				--print( "**** Account " .. i .. ":", _M.stream_bitrate_accounts[i].last_bitrate, vrate, vremain )
				_M.stream_bitrate_accounts[i].last_remained = vremain
			--}
			else
				--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = _G._ENV[ "stream[" .. i .. "].bitrateLimit" ]
				changed = AssignDynamicBitrate(i, _G._ENV[ "stream[" .. i .. "].bitrateLimit" ])
				_M.stream_bitrate_accounts[i].last_remained = 0
				_M.stream_bitrate_accounts[i].stable_count = 0
			end

			if changed then
				--Don't wait ACK, so uses the _NOTIFY
				_M.encoderProxies[i]._NOTIFY:Control( "update-dynamic-bitrate", { ["bitrate"] = _M.stream_bitrate_accounts[i].last_bitrate } )
			end
		--}
		else
			--_G._ENV[ "stream[" .. i .. "].suggestBitrate" ] = _G._ENV[ "stream[" .. i .. "].bitrateLimit" ]
			_M.stream_bitrate_accounts[i].last_bitrate = _G._ENV[ "stream[" .. i .. "].bitrateLimit" ]
			_M.stream_bitrate_accounts[i].last_remained = 0
			_M.stream_bitrate_accounts[i].stable_count = 0
		end
	end
end
------------------------
return _M
