------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 添加Onvif用户
##ARGUMENTS{GET,POST}: 
    @@Username{*,STR}: 要添加的用户名
	@@Password{*,STR}: 添加用户的密码
 	@@Name{STR}: 用户的友好名称 (为空时，名称与Username相同)
	@@Role{STR}: 用户角色 (为空时，默认角色为 <b>user</b>)
		%%administrator: 管理员
		%%operator: Operator
		%%user: 普通用户
	~~

##RETURNS: 
	++Data{OBJ}:
		**Reason{STR}: 如果返回错误码为405，Reason说明添加用户失败的原因：
			%%reserved: 使用了系统保留的用户名(如admin)
			%%invalid: 无效的用户名
			%%exist: 指定的用户已存在
			%%full: 超出了系统最多支持的用户数
		~~
	~~

##NOTES:
>>{BOLD} 安全性说明：
>> 调用该API必须通过HTTP的身份认证。身份认证用户名和密码同Web登录。

##__OTHERS:
>> 在本页面的API Call测试中，给定的用户名为admin，密码为admin（指定HTTP Request header中Authorization:"Basic YWRtaW46YWRtaW4="）。如果需要修改，请在下面指定：
<script type="text/javascript">
ajaxAuth = "Basic YWRtaW46YWRtaW4=";
function change_auth(val)
{
	ajaxAuth = val;
	alert('DONE');
}
</script>
>> 修改Authorization值: <input type="text" id="test_auth" size="28" value="Basic YWRtaW46YWRtaW4=" /> <input type="button" value="修改" onclick='change_auth($("#test_auth")[0].value);' />
<br /><br />
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Username or not args.Password then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid arguments"
	}]] )
	return
end

local MAX_USERS=10

local function addOnvifUser( user, pass, name, role )
	if user == "admin" then
		return false, "reserved"
	elseif type(user) ~="string" or type(pass) ~= "string" then
		return false, "invalid"
	elseif user == "" then
		return false, "invalid"
	else
		local gets = {}
		local i
		
		for i=0, MAX_USERS-1 do
			gets[ "onvif.users[" .. i .. "].username" ] = ""
		end

		local r,cc
		r,cc,gets = rpc.Call( "BUS.Get", gets )
		if not r or not gets then
			return false, "system"
		end

		local validSpace = -1
		for i=0, MAX_USERS-1 do
			if gets[ "onvif.users[" .. i .. "].username" ] == user then
				return false, "exist"
			elseif validSpace == -1 and gets[ "onvif.users[" .. i .. "].username" ] == "" then
				validSpace = i
			end
		end
		
		if validSpace >= 0 then
			local sets = {["onvif.users[" .. validSpace .. "].username"] = user,
						  ["onvif.users[" .. validSpace .. "].password" ] = pass }
			if name and name ~= "" then
				sets[ "onvif.users[" .. validSpace .. "].name" ] = name
			else
				sets[ "onvif.users[" .. validSpace .. "].name" ] = user
			end

			if role and role ~= "" then
				sets[ "onvif.users[" .. validSpace .. "].role" ] = role
			else
				sets[ "onvif.users[" .. validSpace .. "].role" ] = "user"
			end
			rpc.Call( "BUS.Set", sets )
		else
			return false, "full"
		end

		return true
	end
end

local code, reason = addOnvifUser( args.Username, args.Password, args.Name, args.Role )
if code then
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK"
	}]] )
else
	cgilua.put([[
	{
		"Result":405,
		"Status":"ERROR: Fail to add user",
		"Data":{
			"Reason":"]] .. (reason or "unknown") .. [["
		}
	}]] )
end

