------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 添加一个WIFI连接
##ARGUMENTS{POST}: 
  @@DeviceObject: 如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Wifi接口。
  @@Title{*}: 连接名称
  @@SSID{*}: 连接到的SSID名称
  @@AutoConnect{INT,*}: 自动连接选项
    %%1: 表示当在范围内找到这个热点，将自动连接
    %%0: 不自动连接，需要手动进行连接（参见 ^^REF{connectWifi.lua}）
  ~~

  @@Security{*}: WIFI连接的安全性。为以下选项之一：
		%%none-none: 无密码
	    %%none-open: WEP 40/128位密钥(开放式)
	    %%none-share: WEP 40/128位密钥(共享密钥)
	    %%none-open-128: WEP 128位密码句(开放式)
	    %%none-share-128: WEP 128位密码句(共享密钥)
	    %%ieee8021x: IEEE 802.1X动态WEP（暂不支持）
		%%wpa-psk: WPA/PSK或WPA2/PSK
	    %%wpa-none: 面向AdHoc的WPA/WPA2
  ~~

  @@Psk{*}: 连接密码
  @@DHCP{INT,*}: 
    %%1: 使用DHCP获取地址
    %%0: 手动指定IP地址
  ~~
  @@G=Manual-IP: 本参数组包括IP,Mask,Gateway,Dns1和Dns2。当DHCP为0时（手动指定IP地址）需要指定以下参数
  @@IP{G=Manual-IP}: 手动指定的IP地址
  @@Mask{G=Manual-IP}: 手动指定子网掩码
  @@Gateway{G=Manual-IP}: 手动指定默认网关
  @@Dns1{G=Manual-IP}: 指定DNS1 (或为空)
  @@Dns2{G=Manual-IP}: 指定DNS2 (或为空)

##RETURNS{DEFAULT}: 

]]

local WIFI = require "M_WifiSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = WIFI.GetDefaultWifi()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get Wifi device"
	}
	]])
		return
	end
	objectId = device.Object
end

local r,msg = WIFI.SetConnection( objectId, true --[[New one]], args )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
}
]])



