------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 获得音频信号源信息
##ARGUMENTS: 
    无

##RETURNS: 
	++Data{OBJ}:
        **Sources{ARR}: 
        [ "DIGITAL|ANALOG|LINE|MIC|HDMI|SDI|...", ... ]
        >> 数组，返回所有设备支持的音频信号源列表。
            %%DIGITAL: 是一个通用的名称，对于HDMI/SDI信号源带有内嵌音频，DIGITAL代表了HDMI/SDI的内嵌音频。
            %%ANALOG: 同DIGITAL类似，ANALOG代表通常的模拟音频输入（对于大多数设备，它代表了Line in输入）
            %%LINE: 表示line in
            %%MIC: 表示Microphone输入
            %%HDMI: 表示HDMI内嵌音频
            %%SDI: 表示SDI内嵌音频

        **CurrentSource{STR}:"DIGITAL|ANALOG|LINE|MIC|HDMI|SDI|...|UNKNOWN|DETECTING",
        当前音频信号输入的源。参数值的含义同上（Sources）。如果为UNKNOWN，表示没有信号源输入；DETECTING表示正在检测（仅针对具有多个输入源的设备）

        **Sampling{INT}: 数值（如48000,44100,32000等）,音频采样率

        **Channels{INT}:数值(如2),声道数
	~~
]]

local cgilua = require "cgilua"
require("kiloview.globalConfig")()
local rpc = require "kiloview.ringRpc"

local r, info = rpc.Call( "BUS.GetSignalStatus" )
local sampling = 48000
local channels = 2
local asource = "UNKNOWN"
if r and info then
	asource = info.audio["audio.source"] or "UNKNOWN"
	--sampling = info.audio["audio.sampling"] or 0
	channels = info.audio["audio.channels"] or 0
end

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Sources":[
]])

local cnt = 0
if _G.FEATURE_AUDIO_SDI then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"SDI\"" )
	cnt = cnt + 1
end

if _G.FEATURE_AUDIO_HDMI then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"HDMI\"" )
	cnt = cnt + 1
end

if _G.FEATURE_AUDIO_LINEIN then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"LINE\"" )
	cnt = cnt + 1
end

cgilua.put([[
		],

		"CurrentSource":"]] .. asource .. [[",
		"Sampling":]] .. sampling .. [[,
		"Channels":]] .. channels .. [[
	}
}
]] )

