------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ETHERNET}: 获得当前（或指定的）以太网的详细状态信息
##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Ethernet接口
	
##RETURNS: 
	++Data{OBJ}:
		**Device: Ethernet设备名称 (如eth0, usb1, ...)
		**Object: Ethernet对象名(注：是DeviceObject)
		**State: 当前Ethernet的连接状态，为以下值之一： 
            %%disabled: 被禁用
            %%off: 未连接
            %%connecting: 正在连接
            %%connected: 已连接
        ~~
        **ConnectionObject: 当前以太网的Connection Object，在^^REF{setEthernet.lua}中需要用到
        **ConnectionUuid: 当前连接的Uuid，在^^REF{setEthernet.lua}中需要用到

        **Carrier{INT}: 
            %%0: 表示网线没有连接
            %%1: 表示有网线连接
        ~~

		**Speed{INT}:
            >> 连接速率（10/100/1000,单位Mbps)

        **HwAddress: 网络的MAC地址（可能是Clone的Mac地址）
        **PermHwAddress: 网络设备的原始Mac地址

        **DHCP{INT}: 
            %%0: 表示手动指定的IP地址；
            %%1: 表示动态获取IP地址
        ~~

		**IP: 当前获得的IP地址
		**Prefix: 以prefix形式表示的子网掩码（如24表示255.255.255.0, 16表示255.255.0.0)
		**Gateway: 默认网关
        **Dns1: DNS1
        **Dns2: DNS2
	~~
]]

local ETH = require "M_EthernetSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local objectId
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.DeviceObject
end

local device = ETH.GetDefaultEthernet(objectId)
if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get ethernet device"
}
]])
	return
end

local r = ETH.GetEthernetDetails( device )
if not r then
	cgilua.put([[
{
	"Result":400,
	"Status":"FAIL: Fail to get ethernet details"
}
]])

else
	local stateStr = "unknown"
	if device.State == 10 then
		stateStr = "disabled"
	elseif device.State <= 30 or device.State == 120 then
		stateStr = "off"
	elseif device.State < 80 then
		stateStr = "connecting"
	else
		stateStr = "connected"
	end
	
	cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Device":"]] .. device.Name .. [[",
		"Object":"]] .. device.Object .. [[",
		"State":"]] .. stateStr .. [[",
		"ConnectionObject":"]] .. r.ConnectionObject .. [[",
		"ConnectionUuid":"]] .. r.ConnectionUuid .. [[",
		"Carrier":]] .. r.Carrier .. [[,
		"Speed":]] .. r.Speed .. [[,
		"HwAddress":"]] .. r.HwAddress .. [[",
		"PermHwAddress":"]] .. r.PermHwAddress .. [[",
		"DHCP":]] .. ( r.DHCP and "1" or "0" ) .. [[,
		"IP":"]] .. r.IP .. [[",
		"Prefix":]] .. r.Prefix .. [[,
		"Gateway":"]] .. r.Gateway ..[[",
		"Dns1":"]] .. (r.Dns1 or "") .. [[",
		"Dns2":"]] .. (r.Dns2 or "") .. [["
	}
}
]])

end


