------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=NET-GENERAL}: 获得设备上的所有网络设备列表
##ARGUMENTS: 
	无
	
##RETURNS: 
	++Data{ARR}: 数组，每一项代表一个网络接口信息，内容如下：
		++{OBJ}:
			**Device: 网络接口名（如eth0, usb0, ...）
			**Object: 接口ObjectID。注意这个Object，它是后续对网络接口进行操作的唯一标识。
			**Type: 网络接口类型，为以下值之一： 
                %%ETHERNET: 802.3以太网
                %%WIFI: 802.11 WiFi
                %%MODEM: 3G/4G/LTE无线广域网
                %%BT: 蓝牙
                %%BOND: 多网络接口捆绑
                %%ADSL: ADSL
                %%VLAN: VLAN
                %%UNKNOWN: 未知
            ~~
			注：目前，除ETHERNET/WIFI/MODEM/BT以外，其它类型不支持。

			**State: 当前网络的连接状态，为以下值之一： 
                %%disabled: 被禁用
                %%off: 未连接
                %%connecting: 正在连接
                %%connected: 已连接
            ~~
		~~
	~~

##RELATED:
  要获得每个网络接口的详细状态（如IP地址，Wifi连接信息等），请判断其Type（ETHERNET/WIFI...），然后调用对应的 get****Detail.lua，
如 ^^REF{getWifiDetail.lua}, ^^REF{getEthernetDetail.lua}, ...
]]

local NET = require "M_NetworkGeneric"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local devices = NET.GetNetInterfaces()
if not devices then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: System internal error"
}
]])
	return
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":[
]])

local k,v
local cc = 0
for k,v in pairs(devices) do
	if v.Name ~= "lo" then
		local stateStr = "unknown"
		if v.State == 10 then
			stateStr = "disabled"
		elseif v.State <= 30 or v.State == 120 then
			stateStr = "off"
		elseif v.State < 80 then
			stateStr = "connecting"
		else
			stateStr = "connected"
		end
	
		if cc == 0 then
			cgilua.put( [[	{ ]] )
		else
			cgilua.put( [[	,{ ]] )
		end
		cgilua.put( [["Device":"]] .. v.Name .. [[",
	"Object":"]] .. v.Object .. [[",
	"Type":"]] .. v.TypeName .. [[",
	"State":"]] .. stateStr .. [[" }
]])
		cc = cc + 1
	end
end

cgilua.put( [[
	]
}
]])


