------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 获取Onvif配置参数
##ARGUMENTS:

##RETURNS:
  ++Data{OBJ}: 
  **Enable{INT}: Onvif协议支持使能标志
  	%%0: 禁用Onvif协议支持
	%%1: 开启Onvif协议支持
  ~~

  **ServicePort{INT}: Onvif HTTP服务端口（默认8080）
  **Discoverable{INT}: Onvif发现功能使能：
    %%0: 禁止Onvif发现功能
	%%1: 允许Onvif发现功能
  ~~

  **Location{STR}: Onvif基本信息中的位置信息。

  **DeviceName{STR}: Onvif基本信息中的设备名称。这个值等同于 ^^REF{deviceInfo.lua}中的FRIENDLY_NAME。

  **AuthRequirement: Onvif是否要求身份认证
    %%0: 关闭身份认证
	%%1: 要求身份认证
  ~~
  ~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local argsMap = {
	["Enable"] = "onvif.control.enabled",
	["ServicePort"] = "onvif.control.port",
	["Discoverable"] = "onvif.discoverable",
	["Location"] = "onvif.location",
	["DeviceName"] = "onvif.deviceName",
	["AuthRequirement"] = "onvif.auth_req"
}

local gets = {}
local k,v
for k,v in pairs(argsMap) do
	if k == "Location" or k == "DeviceName" then
		gets[v] = ""
	else
		gets[v] = 0
	end
end

local r, cc, gets = rpc.Call( "BUS.Get", gets )

if not r or not gets then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to get arguments"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
]])
	local counter=0
	local xx, isNumber
	for k,v in pairs(argsMap) do
		if counter > 0 then
			cgilua.put( ",\n" )
		end

		xx = gets[ v ]
		if k == "Location" or k == "DeviceName" then
			isNumber = false
		else
			isNumber = true
		end

		if not xx then
			xx = isNumber and 0 or ""
		end

		cgilua.put( "		\"" .. k .. "\":" .. ( isNumber and "" or "\"" ) .. xx .. (isNumber and "" or "\"") )
		counter = counter + 1
	end

	cgilua.put([[

	}
}
]])

end

