------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=PORTFWD}: Get port forwarding list
>> Get all configured port forwarding list.

##ARGUMENTS{GET,POST}:
NONE

##RETURNS: 
	++Data{ARR}: The port forwarding list, each item is a JSON object
	++{OBJ}:
		**Status{STR}: The port forwarding work status:
		  %%running: Normal case, port forwarding task is running.
		  %%idle: For some reason, the port forwarding task is not started. This is an abnormal case, you can assume this is an error.
		  %%error: Port forwarding start failed.
		  %%restart: The port forwarding task is terminated (since an exception) but now is restarting.
		  %%terminated: The port forwarding task is terminated since it's destroied.
		~~

		**Protocol{STR}: Forward protocol type, 'tcp' or 'udp'.
		**Listen{STR}: Device local listen address and port, the format is "address:port". Which address can be '*', that means listen at any network interface.
		**__Port{INT}: The listening port extracted from "Listen", for alternative using.
		**Target{STR}: Forward to network target, format is "address:port", wich address can be a domain name, i.e. 'www.google.com'.
		**Over{STR}: The data forwarding over which device's network interface. You can get all valid network interfaces by ^^REF{getNetInterface.lua}
		**MaxPacketSize{INT}: The maximum packet size for data transmission, in KB.
		**SocketRxBuffer{INT}: Socket receiving buffer size, in KB.
		**SocketTxBuffer{INT}: Socket sending buffer size, in KB.
	~~
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local r, items = rpc.Call( "PORT_FWD.GetAll" )

if not r or type(items) ~= "table" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"failed"
	}
]])
else
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK",
		"Data":[
]])

	local i,v
	for i,v in ipairs(items) do
		if i>1 then
			cgilua.put("		,\n" )
		end

		cgilua.put([[
		{
			"Status":"]] .. (v.Status or "idle") .. [[",
			"Protocol":"]] .. (v.Protocol or "") .. [[",
			"Listen":"]] .. (v.Listen or "") .. [[",
			"__Port":]] .. (v.__Port or "0") .. [[,
			"Target":"]] .. (v.Target or "") .. [[",
			"Over":"]] .. (v.Over or "") .. [[",
			"MaxPacketSize":]] .. (v.MaxPacketSize or 64) .. [[,
			"SocketRxBuffer":]] .. (v.SocketRxBuffer or 256) .. [[,
			"SocketTxBuffer":]] .. (v.SocketTxBuffer or 256) .. [[
		}
]])
	end

	cgilua.put([[
		]
	}]])
end

