------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=RECORD}: 获得指定码流的录像选项参数

##ARGUMENTS{GET,POST}:
    @@Stream{STR,*}:
        指定要获取录像选项参数的码流。
        %%main: 主码流
        %%sub: 子码流
	~~

##RETURNS: 
	++Data{OBJ}:
	**AutoRecord: 插入磁盘时自动开启录像
		%%1: 开启自动录像
		%%0: 不自动录像
	~~

	**DiskSelection: 选择录像磁盘的策略：
		%%first: 第一个插入的存储器
	- 不管存储器是什么类型，也不管可用空间大小，录像到第一个插入的存储器分区
		%%largest: 可用空间最大的存储器
	- 不管存储器是什么类型，录像到可用空间最大的那个存储器分区
		%%mark: 特别标记的存储器
	- 只录像到在根目录下创建有 RECORD_HERE 或 RECORD_HERE.txt 文件的存储器分区上
		%%usb: 可用空间最大的USB存储器
	- 录像到可用空间最大的USB存储器分区上
		%%usb-first: 第一个插入的USB存储器
	- 不管可用空间大小，录像到第一个插入的USB存储器分区上
		%%usb-mark: 特别标记的USB存储器
	- 只录像到在根目录下创建有 RECORD_HERE 或 RECORD_HERE.txt 文件的USB存储器分区上
		%%sdmmc: 可用空间最大的SD/MMC存储器
	- 录像到可用空间最大的SD/MMC存储器分区上
		%%sdmmc-first: 第一个插入的SD/MMC存储器
	- 不管可用空间大小，录像到第一个插入的SD/MMC存储器分区上
		%%sdmmc-mark: 特别标记的SD/MMC存储器
	- 只录像到在根目录下创建有 RECORD_HERE 或 RECORD_HERE.txt 文件的SD/MMC存储器分区上
	~~

	**Format: 文件录像格式
		%%ts: TS文件格式
		%%mp4: MP4文件格式（暂不支持）
	~~

	**Prefix: 录像文件名的前缀。录像文件的命名以前缀开头，后跟随8位顺序递增的数字。默认是"REC"。

	**Limitation: 文件录像限制：
		%%time: 限制录像时长，一旦时长限制到达，停止录像
		%%size: 限制文件大小，一旦文件大小到达，停止录像
		%%time-loop: 限制单个文件的时长，自动切割录像到多个文件
		%%size-loop: 限制单个文件的大小，自动切割录像到多个文件
		%%none: 无限制
	~~

	**LimitSize{INT}: 如果Limitation为size或size-loop，指定限制的文件大小（KB）
	**LimitTime{INT}: 如果Limitation为time或time-loop，指定限制的录像时长（秒）

	**Overwrite: 当磁盘空间不够时，是否覆盖已有的录像：
		%%overwrite: 覆盖已有录像
		%%full-stop: 当磁盘满时，停止录像
	~~
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local stream
local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid stream"
	}
]])
	return
end

stream = args.Stream

local streamId = 0
if stream == "sub" then
	streamId = 1
end

local argPrefix = "record_m.stream[" .. streamId .. "]"

local reqParams = {
	[argPrefix .. ".auto_record"] = 0,
	[argPrefix .. ".disk_type"] = "largest",
	[argPrefix .. ".format"] = "ts",
	[argPrefix .. ".file_prefix"] = "REC",
	[argPrefix .. ".limit_type"] = "size-loop",
	[argPrefix .. ".limit_size"] = 0,
	[argPrefix .. ".limit_time"] = 0,
	[argPrefix .. ".disk_policy"] = "overwrite"
}

local r, cc, gets = rpc.Call( "BUS.Get", reqParams )
if not r or not gets then
cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to get picture cropping configurations",
}]])
else
cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"AutoRecord":]] .. gets[argPrefix .. ".auto_record"] .. [[,
		"DiskSelection":"]] .. gets[argPrefix .. ".disk_type"] .. [[",
		"Format":"]] .. gets[argPrefix .. ".format"] .. [[",
		"Prefix":"]] .. gets[argPrefix .. ".file_prefix"] .. [[",
		"Limition":"]] .. gets[argPrefix .. ".limit_type"] .. [[",
		"Limitation":"]] .. gets[argPrefix .. ".limit_type"] .. [[",
		"LimitSize":]] .. gets[argPrefix .. ".limit_size"] .. [[,
		"LimitTime":]] .. gets[argPrefix .. ".limit_time"] .. [[,
		"Overwrite":"]] .. gets[argPrefix .. ".disk_policy"] .. [["
	}
}]])
end


