------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=TIME_LOCATION}: 获取设备的时间配置参数

##ARGUMENTS: 
  无

##RETURNS: 
	++Data{OBJ}:
		**LocalTime: 格式: yyyy-mm-dd hh:nn:ss
            >> 设备上的本地时间
        **UTCTime: 格式: yyyy-mm-dd hh:nn:ss
            >> 设备上的UTC时间
        **NTPEnable{INT}: 0|1, NTP同步使能。0为关闭，1为开启
        **NTPServers: NTP服务器地址列表，多项之间用空格隔开
        **NTPSync{INT}: 0|1，返回当前NTP同步状态。0为未同步，1为已同步
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )
local _x,timedateinfo = rpc.Call( "SYS_STATUS.GetTimeDateInfo" )
if not timedateinfo then
	local stime = os.date("*t")
	timedateinfo = {
		["local"] = string.format( "%04d-%02d-%02d %02d:%02d:%02d", stime.year,stime.month,stime.day,stime.hour,stime.min,stime.sec ),
		["utc"] = string.format( "%04d-%02d-%02d %02d:%02d:%02d", stime.year,stime.month,stime.day,stime.hour,stime.min,stime.sec ),
		["zone"] = "UTC",
		["ntp_enabled"] = "no",
		["ntp_sync"] = "no"
	}
end

local _x,ntpAddr = rpc.Call( "SYS_STATUS.GetNTPAddress" )

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"LocalTime":"]] .. timedateinfo["local"] .. [[",
		"UTCTime":"]] .. timedateinfo["utc"] .. [[",
		"NTPEnable":]] .. ( timedateinfo["ntp_enabled"] == "yes" and 1 or 0 ) .. [[,
		"NTPServers":"]] .. ( ntpAddr or "") .. [[",
		"NTPSync":]] .. ( timedateinfo["ntp_sync"] == "yes" and 1 or 0 ) .. [[
	}
}
]])
