------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 获得当前Wifi的详细状态信息
##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Wifi接口
	
##RETURNS: 
	++Data{OBJ}:
		**Device: Wifi设备名（如wlan0）
		**Object: Wifi对象名
		**State: 当前Wifi的连接状态，为以下值之一： 
            %%disabled: 被禁用
            %%off: 未连接
            %%connecting: 正在连接
            %%connected: 已连接
        ~~
		**Capabilities: 无线能力，标识其 2.4G/5.8G支持能力
		**ConnectedSSID: 已连接的SSID名称，如果未连接则为空
		**SignalStrength{INT}: 已连接AP的信号强度（0－100），未连接则为0
		**ConnectingSpeed{INT}: 当前AP的连接速度(Mbps），无连接则为0
		**IP: 当前获得的IP地址
		**Prefix{INT}: 以prefix形式表示的子网掩码（如24表示255.255.255.0, 16表示255.255.0.0）
		**Gateway: 默认网关
	~~
]]

local WIFI = require "M_WifiSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local objectId
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.DeviceObject
end

local device = WIFI.GetDefaultWifi(objectId)
if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get Wifi device"
}
]])
	return
end

local r = WIFI.GetWifiDetails( device )
if not r then
	cgilua.put([[
{
	"Result":400,
	"Status":"FAIL: Fail to get Wifi details"
}
]])

else
	local stateStr = "unknown"
	if device.State == 10 then
		stateStr = "disabled"
	elseif device.State <= 30 or device.State == 120 then
		stateStr = "off"
	elseif device.State < 80 then
		stateStr = "connecting"
	else
		stateStr = "connected"
	end
	
	cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Device":"]] .. device.Name .. [[",
		"Object":"]] .. device.Object .. [[",
		"State":"]] .. stateStr .. [[",
		"Capabilities":"]] .. r.Capabilities .. [[",
		"ConnectedSSID":"]] .. r.ConnectedSSID .. [[",
		"ConnectingSpeed":]] .. r.ConnectingSpeed .. [[,
		"SignalStrength":]] .. r.SignalStrength .. [[,
		"IP":"]] .. r.IP .. [[",
		"Prefix":]] .. r.Prefix .. [[,
		"Gateway":"]] .. r.Gateway ..[["
	}
}
]])

end


