
------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=BASIC}: 列出当前的HTTP登录用户列表
##ARGUMENTS: 
    无
    	
##RETURNS: 
	++Data{OBJ}:
		**Users{ARR}:   ["user1","user2",...] 每一项为一个用户名（字符串）
			>>{BOLD} 注意：超级管理员admin将不会列出在返回结果中。
	~~

##NOTES:
>>{BOLD} 安全性说明：
>> 调用该API必须通过HTTP的身份认证。身份认证用户名和密码同Web登录。

##__OTHERS:
>> 在本页面的API Call测试中，给定的用户名为admin，密码为admin（指定HTTP Request header中Authorization:"Basic YWRtaW46YWRtaW4="）。如果需要修改，请在下面指定：
<script type="text/javascript">
ajaxAuth = "Basic YWRtaW46YWRtaW4=";
function change_auth(val)
{
	ajaxAuth = val;
	alert('DONE');
}
</script>
>> 修改Authorization值: <input type="text" id="test_auth" size="28" value="Basic YWRtaW46YWRtaW4=" /> <input type="button" value="修改" onclick='change_auth($("#test_auth")[0].value);' />
<br /><br />
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local r, users = rpc.Call( "SERVER.GetUsers" )
if not r or not users then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Fail to get users"
	}]] )
else
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK",
		"Data":{
			"Users":[
]] )
	local i
	for i=1, #users do
		if i == 1 then
			cgilua.put( '\t\t\t\t"' .. users[i].username .. '"\n' )
		else
			cgilua.put( '\t\t\t\t,"' .. users[i].username .. '"\n' )
		end
	end

	cgilua.put( [[
			]
		}
	}]] )
end
