------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=WIFI}: 列出所有已保存的Wifi连接列表
##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}）；如果没有指定，则为默为的Wifi接口

##RETURNS: 
	++Data{OBJ}:
		**Device: Wifi设备名(如wlan0)
		**Object: Wifi对象名
		++CONNECTION_list{ARR}: 每一项为一个已保存的Wifi连接项，内容如下：
			++{OBJ}:
            	**Title: 连接名称
				**Security: 连接的安全设置
                    %%open: 开放
                    %%wep: WEP
					%%dynamic-wep: 动态WEP(IEEE 802.1x)
                    %%adhoc-psk: AdHoc的PSK加密
                    %%wpa-psk: WPA/WPA2-PSK
                    %%wpa-eap: WPA-EAP
                    %%unknown: 未知
                ~~
				**IsCurrent{INT}: 0|1, 该连接是否为当前正在使用的连接。1为true, 0为false。
				**AddressResolve: 地址获取方式
                    %%dhcp: DHCP
                    %%manual: 手动指定
                ~~

				**Object:连接的对象名称。
                >> 注意这个名称，这是后续对该连接进行操作的依据，形式为 /org/freedesktop/NetworkManager/Connections/3 。

				**Uuid: 该连接的Uuid。如果需要修改一个Wifi连接，该Uuid应该提供作为修改连接的参数
			~~
		~~
	~~
]]

local cgilua = require "cgilua"
local WIFI = require "M_WifiSet"

local objectId
local args = require("M_Request").VerifyAndGetArgs()
if args then
	objectId = args.DeviceObject
end

local device = WIFI.GetDefaultWifi(objectId)
local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

if not device then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get Wifi device"
}
]])
	return
end

local conns = WIFI.GetConnections( device.Object )
if not conns then
	cgilua.put([[
{
	"Result":500,
	"Status":"FAIL: Fault to get connection list"
}
]])
	return
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data": {
		"Device":"]] .. device.Name .. [[",
		"Object":"]] .. device.Object .. [[",
		"CONNECTION_list":[
]] )

		local k,v
		local cc = 0
		for k,v in pairs( conns ) do
			if cc > 0 then
				cgilua.put( ", " )
			end
			
			cgilua.put([[
			{ "Title":"]] .. v.title .. [[",
			"Security":"]] .. v.secinfo .. [[",
			"IsCurrent":]] .. (v.isCurrent and 1 or 0 ) .. [[,
			"AddressResolve":"]] .. v.addrResolv .. [[",
			"Object":"]] .. v.connObj .. [[",
			"Uuid":"]] .. v.uuid .. [["
			}
			]] )
			
			cc = cc + 1
		end

		cgilua.put([[
		]
	}
}
]])

