------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=SERIAL_PTZ}: PTZ控制
##ARGUMENTS{GET,POST}: 
    @@Action{STR}: 必须参数。指定PTZ控制动作：
		%%left: 向左移动
		%%right: 向右移动
		%%up: 向上移动
		%%down: 向下移动
		%%left-up: 向左上移动
		%%left-down: 向左下移动
		%%right-up: 向右上移动
		%%right-down: 向右下移动
		%%stop-move: 停止移动
		%%zoom-in: 放大
		%%zoom-out: 缩小
		%%stop-zoom: 停止缩放
		%%focus-near: 聚焦近
		%%focus-far: 聚焦远
		%%stop-focus: 停止聚集
		%%iris-up: 光圈开
		%%iris-down: 光圈关
		%%stop-iris: 停止光圈控制
		%%stop-all: 停止所有动作（P/T/Z和聚焦、光圈）
		%%home: 回到HOME位置
		%%save-preset: 保存预置位
		%%remove-preset: 删除预置位
		%%load-preset: 调用预置位
	
	@@Speed{NUM}: 可选。对于left/right/up/down/zoom-/focus-/iris-操作，指定动作的速度，值范围0-1（小数）。如未指定，使用系统默认配置的速度。
	@@Speed2{NUM}: 可选。对于left-up/left-down/right-up/right-down操作，指定垂直移动的速度（水平移动的速度由参数Speed指定）。值范围0-1（小数）。如未指定，使用系统默认配置的速度。
	@@Id{INT}: 当执行预置位操作时，必须指定该参数。代表要操作的预置位ID；其它操作无需此参数。

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
local action = nil
if args and args.Action and args.Action ~= "" then
	action = args.Action:lower()
end

if not action then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local controlArgs = {
}

if action:match("preset") then
	local id = tonumber(args.Id)
	if not id then
		cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
		return
	end
	controlArgs.id = id
else
	controlArgs.speed = tonumber(args.Speed)
	controlArgs.tilt_speed = tonumber(args.Speed2)
end

local r,x
r,x = rpc.Call( "BUS.PTZControl", action, controlArgs )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to control PTZ") .. [["
}
]])

