------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=RECORD}: 重命名录像文件

##ARGUMENTS{GET,POST}:
	@@Device{STR,*}: 要重命名录像文件的磁盘设备号（请参考^^REF{getRecordList.lua}返回的RecordDevice字段）
	@@SequenceID{STR,*}: 要重命名文件所在的录像Sequence ID（请参考^^REF{getRecordList.lua}返回的SequenceID字段）
	@@From{STR,*}: 要重命名的原文件名
	@@To{STR,*}: 重命名到新的文件名

##RETURNS{DEFAULT}: 

##NOTES:
	<b>注意！</b>不可对正在录像中的文件进行重命名！如果尝试对正在录像中的文件进行重命名，将返回500错误，错误消息为BUSY。
]]


local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local dev, filter
local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Device or args.Device == "" or not args.SequenceID or args.SequenceID == "" or not args.From or args.From == "" or not args.To or args.To == "" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid arguments"
	}
]])
	return
end

local r,cc,info = rpc.Call( "BUS.DiskRecordList", args.Device, args.SequenceID )
if not r or not cc or type(info) ~= "table" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: No valid recording list"
	}
]])
	return
end

local path = info[1] and info[1].PATH
if not path or type(info[1].FILES) ~= "table" or #info[1].FILES <= 0 then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: No valid recording files"
	}
]])
	return
end

if info[1]._CURRENT then
	if args.From == info[1].FILES[ #info[1].FILES ].FILENAME then
		cgilua.put([[
		{
			"Result":500,
			"Status":"BUSY"
		}
]])
		return
	end
end

local fp = io.open( path .. "/" .. args.From, "r" )
if not fp then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid source file"
	}
]])
	return
else
	io.close(fp)
end

--os.execute( "/bin/mv -f " .. path .. "/" .. args.From .. " " .. path .. "/" .. args.To )
rpc.Call( "BUS.RenameRecordFile", path .. "/" .. args.From, path .. "/" .. args.To )

cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

