------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 选择音频信号源
##ARGUMENTS{GET,POST}: 
    @@Source{*}: 指定一个音频信号源名称（来自于 ^^REF{getAudioSource.lua} 中返回的 Sources 列表）
            >> 注：不同产品型号，数字音频源的名称可能不一样，例如HDMI/SDI/...。
            >> 为了软件编程的一致，无论HDMI/SDI/...哪种
            音频源，均可以用DIGITAL来代表；同样，模拟音频源可能为LINE/MIC/...，可以用ANALOG来代表。

			>> 如果指定Source为 <b>AUTO</b>，表示让设备自动选择音频源。自动选择的原则是根据视频信号
			输入的来源，如果来自于数字视频接口（如HDMI/SDI/DVI/...），则音频也从该接口获取；否则，
			将选择模拟音频（Line in）输入。

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Source or args.Source == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local r, x = rpc.Call( "BUS.Set", { ["avio.audio.source"] = args.Source } )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to select audio source") .. [["
}
]])



