------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 选择视频信号源（仅针对具有多个视频输入信号源的设备有效）
##ARGUMENTS{GET,POST}: 
    @@Source: 指定一个视频信号源名称（来自于^^REF{getVideoSource.lua}中返回的 Sources 列表）
	>> 有几个特殊的Source选项：
		%%AUTO: 自动侦测。对于具有多个视频输入接口的设备，让设备自动侦测哪个接口具有视频输入信号，
				并且自动锁定该接口。
				>> 以KILOVIEW M2设备为例，该设备具有HDMI和VGA两个输入接口，当指定Source为<b>AUTO</b>
				时，如果检测到HDMI有信号，则自动选择HDMI作为输入；如果检测到VGA有信号，自动选择VGA
				作为输入。如果两者同时有信号，那么将优先选择HDMI。
				>> 如果在工作过程中，某个信号突然丢失，AUTO模式将自动侦测另外一个接口。
				<br />
				>> 对于单一视频接口的编码设备，AUTO将使用默认接口，不进行侦测。
		%%ONESHOT: 一次性自动侦测。自动侦测的机制与 AUTO 是相同的，ONESHOT机制与AUTO机制的不同之处在于，
				一但侦测到某个视频接口有信号，设备将持续锁定这个接口作为输入源。即使在工作过程中，该
				接口的信号丢失，也会一直锁定，直到设备重启或软复位。
		~~

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Source or args.Source == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local r, x = rpc.Call( "BUS.Set", { ["avio.video.source"] = args.Source } )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to select video source") .. [["
}
]])

