------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ENCODING}: 设置指定音频编码通道的编码参数

>>  KILOVIEW编码器设备有一个独立的音频编码引擎。这个引擎可以支持1个或多个通道。每个通道可以
从不同的音频输入设备采集音频，并压缩成不同的格式。对于一个视频编码码流，它可以绑定到音频
编码引擎上的指定通道，从而实现视频和音频的合并码流。

##ARGUMENTS{POST}: 
    @@ID{INT,*}: 指定要设置的音频编码通道ID。通道ID的获取参见 ^^REF{getAudioEngChannel.lua} 的返回结果。
    @@Enable{INT}:
        %%0: 该通道被禁用
        %%1: 启用

    @@Device: "..."
        音频采集设备名称，形式如 alsa:default。目前请忽略该值的作用。
    
    @@Codec: 
    音频编码算法，目前支持AAC(Low-Cost)和G711算法。
        %%AAC: AAC(Low-Coast)
        %%G711: G.711
    ~~
    @@Sampling{INT}: 采样率，对AAC有效。对G711算法，恒为8KHz，不必设定。
    
    @@Channels{INT}:
        声道，值为1或2。对AAC有效。对G711算法，恒为1，不必设定。

    @@Bitrate{INT}: 编码码率，单位bps（如64000表示64Kbps）。对AAC有效。对G711算法，恒为64000，不必设定。

    @@AAC_Format: AAC格式。
        %%RAW: 表示原生的AAC格式；
        %%ADTS: 表示带有ADTS头。
                >>{BOLD} 对于TS流应用，必须指定AAC_Format为ADTS
    ~~
    @@G711_Format: G.711格式。
        %%ULAW: G.711 uLaw格式
        %%ALAW: G.711 aLaw格式
    ~~

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"


local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not tonumber(args.ID) then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local thisChannel = "audioEngine.encoders[" .. tonumber(args.ID) .. "]"
local cc = 0
local sets = {
}

if args.Enable and args.Enable ~= "" then
	sets[ thisChannel .. ".enable" ] = tonumber(args.Enable) or 0
	cc = cc + 1
end

--TODO: Ignored device.

if args.Codec and args.Codec ~= "" then
	sets[ thisChannel .. ".encoder.codec" ] = args.Codec
	cc = cc + 1
end

if args.Sampling and args.Sampling ~= "" then
	sets[ thisChannel .. ".encoder.sampling" ] = tonumber(args.Sampling) or 8000

	cc = cc + 1
end

if args.Channels and args.Channels ~= "" then
	sets[ thisChannel .. ".encoder.channels" ] = tonumber(args.Channels) or 1
	cc = cc + 1
end

if args.Bitrate and args.Bitrate ~= "" then
	sets[ thisChannel .. ".encoder.bitrate" ] = tonumber(args.Bitrate) or 64000
	cc = cc + 1
end

if args.AAC_Format and args.AAC_Format ~= "" then
	sets[ thisChannel .. ".encoder.aac.format" ] = ( args.AAC_Format:upper() == "ADTS" and "ADTS" or "RAW" )
	cc = cc + 1
end

if args.G711_Format and args.G711_Format ~= "" then
	sets[ thisChannel .. ".encoder.g711.law" ] = ( args.G711_Format:upper() == "ALAW" and "ALAW" or "ULAW" )
	cc = cc + 1
end

if cc == 0 then
	cgilua.put([[
{
	"Result":201,
	"Status":"OK: But no any arguments specified"
}
]])
	return
end

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to configure the specified audio engine channel"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end

