------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ETHERNET}: 修改一个有线以太网连接
##ARGUMENTS{POST}: 
  @@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Ethernet接口
  @@Object{*}: 要修改Ethernet连接的Object名称（通过^^REF{getEthernetDetail.lua}可以获得 [ConnectionObject参数]）
  @@Uuid{*}: 该连接的Uuid（通过^^REF{getEthernetDetail.lua} 可以获得）
  @@CloneMac: 可选参数。如果指定了CloneMac（且不为空字符串），则可以修改以太网的MAC地址。
  @@DHCP{INT}: 
    %%1: 使用DHCP获取地址
    %%0: 手动指定IP地址。如果指定DHCP=0，则以下参数（参数组Manual-IP）需要指定。

  @@G=Manual-IP: 手动指定IP地址参数组。本组参数包括：IP, Mask, Prefix, Gateway, Dns1, Dns2
  @@IP{G=Manual-IP}: 手动指定的IP地址
  @@Mask{G=Manual-IP}: 手动指定子网掩码
  @@Prefix{G=Manual-IP}: 以prefix形式指定的子网掩码（注：可选，与Mask参数二选一即可）
  @@Gateway{G=Manual-IP}: 手动指定默认网关
  @@Dns1{G=Manual-IP}: 指定DNS1 (或为空)
  @@Dns2{G=Manual-IP}: 指定DNS2 (或为空)

##RETURNS{DEFAULT}: 

##NOTES:
  调用本API修改以太网连接参数后，参数不会立即生效，除非重启设备或调用^^REF{connectEthernet.lua}重新连接。
]]

local ETH = require "M_EthernetSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = ETH.GetDefaultEthernet()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get ethernet device"
	}
	]])
		return
	end
	objectId = device.Object
end

local r,msg = ETH.SetConnection( objectId, false --[[Change one]], args )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
}
]])


