------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 设置视频的旋转/翻转参数配置
>> KILOVIEW全系列编码器产品，支持视频图像在编码时进行90度/180度/270度旋转，以及水平翻转（镜像）和垂直翻转功能。
>> <b>注意</b>：如果已开启文字叠加功能，随着视频图像的翻转，文字叠加在图像上是否同时翻转，这是可以配置的。请参见 ^^REF{setOSDEnable.lua} 中的NoRotate参数。

##ARGUMENTS{POST}: 
@@Mode{STR}: 设置翻转状态（模式）：
	%%0: 正常，不旋转，不翻转
	%%90: 顺时针旋转90度
	%%180: 顺时针旋转180度
	%%270: 顺时针旋转270度（逆时针旋转90度）
	%%mirror: 水平翻转（镜像）
	%%flip: 垂直翻转（倒像）
~~

##RETURNS{DEFAULT}:
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Mode or args.Mode == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end


local r,x = rpc.Call( "BUS.Set", {["avio.video.RMF"] = args.Mode} )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set flipping parameters"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end
