------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=OSD}: 设置主码流/子码流的字符叠加功能使能

##ARGUMENTS{GET,POST}: 
   @@Stream{*}: 指定码流
        %%main: 主码流
        %%sub: 子码流
    ~~
   @@Enable: OSD使能。可选。
   >> 如果没有指定，将返回当前OSD的使能状态（见返回值说明）
    %%0: 表示禁止OSD
    %%1: 表示启用OSD
    ~~
    >>{BOLD} 注意：
    >>    此处的Enable参数用于控制整个码流的OSD是否开启，相当于一个“总开关”。每个码流可以
    最多支持4个不同的独立叠加项，每个独立叠加项也可以控制开启或关闭。请参考^^REF{setOSDItem.lua}

   @@NoRotate: 可选。指定OSD内容是否随着视频的旋转而旋转。
    %%0: 跟随旋转
    %%1: 不旋转
    ~~
##RETURNS: 

    ++Data{OBJ}:
        **Enable{INT}: 0|1, 执行完本操作后，当前的OSD使能状态
        **NoRotate{INT}: 0|1, 执行完本操作后，当前的旋转控制状态
    ~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local stream = 0
if args.Stream == "sub" then
	stream = 1
end

local args = {
	[ "osd.stream[" .. stream .. "].enabled" ] = tonumber(args.Enable),
	[ "osd.stream[" .. stream .. "].norotate" ] = tonumber(args.NoRotate)
}

rpc.Call( "BUS.Set", args )

args = {
	[ "osd.stream[" .. stream .. "].enabled" ] = 0,
	[ "osd.stream[" .. stream .. "].norotate" ] = 0
}

local r,cc,args = rpc.Call( "BUS.Get", args )
if r and args then
	cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Enable":]] .. args[ "osd.stream[" .. stream .. "].enabled" ] .. [[,
		"NoRotate":]] .. args[ "osd.stream[" .. stream .. "].norotate" ] .. [[
	}
}
]])
else
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set and get OSD enabling"
}
]])
end
