------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 配置Onvif参数
##ARGUMENTS{GET,POST}:
  @@Enable{INT}: Onvif协议支持使能：
  	%%0: 禁用Onvif协议支持
	%%1: 开启Onvif协议支持
  ~~

  @@ServicePort{INT}: Onvif HTTP服务端口（默认8080）
  @@Discoverable{INT}: Onvif发现功能使能：
    %%0: 禁止Onvif发现功能
	%%1: 允许Onvif发现功能
  ~~

  @@Location{STR}: Onvif基本信息中的位置信息。
  >> <b>注意</b>：Onvif中的位置信息与 ^^REF{getLocation.lua} / ^^REF{setLocation.lua} 中的时区信息不完全相同，因此
  用户需要自行定义Location的值。一般的形式为 "country/china"。

  @@DeviceName{STR}: Onvif基本信息中的设备名称。这个值等同于 ^^REF{setDeviceNames.lua}中的FRIENDLY_NAME。

  @@AuthRequirement: Onvif是否要求身份认证：
    %%0: 关闭身份认证
	%%1: 要求身份认证
  ~~

##RETURNS: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local argsMap = {
	["Enable"] = "control.enabled",
	["ServicePort"] = "control.port",
	["Discoverable"] = "discoverable",
	["Location"] = "location",
	["DeviceName"] = "deviceName",
	["AuthRequirement"] = "auth_req"
}

local sets = {}
local k,v
for k,v in pairs(args) do
	if v and v ~= "" then
		if k == "Location" or k == "DeviceName" then
			sets[ "onvif." .. argsMap[k] ] = v
		else
			sets[ "onvif." .. argsMap[k] ] = tonumber(v)
		end
	end
end

local r,x = rpc.Call( "BUS.Set", sets )
if r and x then
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

else
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set arguments"
}
]])

end

