------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ENCODING}: 设置指定码流（主码流/子码流）的编码参数

##ARGUMENTS{POST}: 
    @@Stream{*}:指定要设置的码流
        %%main: 主码流
        %%sub: 子码流
    ~~

    @@Scaling: 
    编码缩放大小。
    %%default: 表示不缩放，对主码流而言，输入视频是什么格式，编码大小就是多大；对子码流而言，默认大小是720x576。
    %%(w)x(h): w表示宽度，h表示高度，指定的缩放大小。
    ~~

    @@Chrome:
    %%0: 彩色
    %%1: 灰度
    ~~

    @@Codec: H264|H265, 
    视频编码算法（目前仅支持H264）

    @@Profile: base|main|high，指定H.264编码的Profile

    @@BRCtrl: cbr|vbr。Bitrate控制方式，cbr或vbr

    @@Bitrate{INT}: 编码码率，单位bps。如1000000表示1Mbps

    @@FrameRateMode: 编码帧率模式
        %%default: 表示全帧率，即输入视频帧率为多少，编码帧率为多少
        %%half: 表示当输入视频的帧率 >=50 时，自动降半帧率
        %%custom: 表示手动指定帧率
    ~~

    @@FrameRate{NUM}: 指定的编码帧率（如果FrameRateMode为default/half，则不必要指定FrameRate值）

    @@GOPSize{INT}: H.264 GOP大小（决定I帧之间的间隔），以帧数为单位

    @@RefFrame{INT}: 1|2, 参数帧。1表示单一参考帧，2表示多参考帧

    @@BindAudioChannel{INT}: 绑定的音频编码引擎通道号, -1表示无音频绑定    

##RETURNS{DEFAULT}:
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local stream = 0
if args.Stream:lower() == "sub" then
	stream = 1
end

local argPrefix = "encoder.stream[" .. stream .. "]"

local sets={}
local cc = 0
if args.Scaling and args.Scaling ~= "" then
	cc = cc + 1
	sets[argPrefix .. ".video.scale"] = args.Scaling
end

if args.Chrome and tonumber(args.Chrome) then
	cc = cc + 1
	sets[argPrefix .. ".video.chrome"] = tonumber(args.Chrome)
end

if args.Profile and args.Profile ~= "" then
	cc = cc + 1
	sets[argPrefix .. ".video.profile"] = args.Profile
end

if args.BRCtrl and args.BRCtrl ~= "" then
	cc = cc + 1
	sets[argPrefix .. ".video.br_ctrl"] = args.BRCtrl
end

if args.Bitrate and tonumber(args.Bitrate) then
	cc = cc + 1
	sets[argPrefix .. ".video.bitrate"] = tonumber(args.Bitrate)
end

local fmode = nil
if args.FrameRateMode and args.FrameRateMode ~= "" then
	cc = cc + 1
	fmode = args.FrameRateMode
	sets[argPrefix .. ".video.framerate_mode"] = args.FrameRateMode
end

if args.FrameRate and tonumber(args.FrameRate) and fmode and fmode ~= "full" and fmode ~= "half" then
	cc = cc + 1
	sets[argPrefix .. ".video.framerate"] = args.FrameRate
end

if args.GOPSize and tonumber(args.GOPSize) then
	cc = cc + 1
	sets[argPrefix .. ".video.gop_size"] = tonumber(args.GOPSize)
end

if args.RefFrame and tonumber(args.RefFrame) then
	cc = cc + 1
	sets[argPrefix .. ".video.ref_frames"] = tonumber(args.RefFrame)
end

if args.BindAudioChannel and tonumber( args.BindAudioChannel ) then
	cc = cc + 1
	sets["streamer.stream[" .. stream .."].audioChannel"] = tonumber(args.BindAudioChannel)
end

if cc == 0 then
	cgilua.put([[
{
	"Result":201,
	"Status":"OK: But no any arguments specified"
}
]])
	return
end

local r,x = rpc.Call( "BUS.Set", sets )
if not r or not x then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set video encoding parameters"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
end

