local _dbg = require "coaf.debug"
local client = require "kiloview.coafClientInit"

local _M = {}
local busOprs = require "kiloview.busOperations"
local devNames = require "kiloview.netdevNames"

local call_groups = {
	["BUS"] = { method="local", object = busOprs },
	["DEVICE_NAMES"] = { method="local", object = devNames },
	["SERVER"] = { method="coaf", path="/SERVER", service="web_daemon", timeout=3 },
	["SYS_STATUS"] = { method="coaf", path="/SYS_STATUS", service="web_daemon", timeout=5 },
	["HOSTAP_CTRL"] = { method="coaf", path="/HOSTAP_CTRL", service="web_daemon", timeout=10 },
	["CONFIG_MANAGER"] = { method="coaf", path="/CONFIG_MANAGER", service="web_daemon", timeout=3 },
	["PRESET"] = { method="coaf", path="/PRESET", service="web_daemon", timeout=10 },
	["UPGRADE"] = { method="coaf", path="/UPGRADE", service="web_daemon", timeout=60 },
	["NETWORK_MANAGER"] = { method="coaf", path="/raw/NETWORK_MANAGER", service="nm_daemon", timeout=3 },
	["WIFI"] = { method="coaf", path="/raw/WIFI", service="nm_daemon", timeout=3 },
	["NET_STATUS"] = { method="coaf", path="/NET_STATUS", service="nm_daemon", timeout=10 },
	["PORT_FWD"] = { method="coaf", path="/PORT_FWD", service="nm_daemon", timeout=5 },
	["ARP"] = { method="coaf", path="/ARP", service="nm_daemon", timeout=5 },
}

local function GetCoAFProxy(objPath, service, loc, timeout, addr)
	local proxy
	local cli
	cli = client.New( service, loc, addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "RPC", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "RPC", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

function _M.Call( target, ... )
	local dotCall = true
	local group, call = target:match("([%w%d_]*)%.(.+)$")
	if not group then
		group, call = target:match("([%w%d_]*):(.+)$")
		if not group then
			return false, "Invalid call target"
		else
			dotCall = false
		end
	end

	local caller = call_groups[group]
	if not caller then
		return false, "Invalid call target"
	end

	if caller.method == "local" then
		local func = caller.object[call]
		if not func then
			return false, "Invalid call function"
		end
		if dotCall then
			return true, func(...)
		else
			return true, func(caller.object, ...)
		end
	else --Currently, all are CoAF calling.
		local proxy = GetCoAFProxy( caller.path, caller.service, caller.location or ngx.var.coaf_location or "local", caller.timeout or 3 )
		if not proxy then
			return false, "Fail to connect remote service"
		else
			local func = proxy[call]
			if dotCall then
				local rets = { func(...) }
				proxy:Destroy()
				return table.unpack(rets)
			else
				local rets = { func(proxy,...) }
				proxy:Destroy()
				return table.unpack(rets)
			end
		end
	end
end

return _M

