--Controller of wifi
local _M = require( "controller.generic" )()
local cgilua = require "cgilua"

----------------------- SAJAX functions ----------------------------

local function DeleteConnection( obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_WIFI_OBJ .. [["}]]
	end

	if not conPath then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_CONNECTION .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.DeleteConnection", conPath )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_DELCONN_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_DELCONN_FAIL .. [["}]]
	end
end

-------------------------------------------------------

local function ConnectConnection(obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_WIFI_OBJ .. [["}]]
	end

	if not conPath then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_CONNECTION .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.ConnectDevice", obj, conPath )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_CON_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_CON_FAIL .. [["}]]
	end

end

-------------------------------------------------------

local function deactiveConnection( obj, conPath )
	if not obj then
		return [[{ "Result":400, "Msg":"]] .. _M.langs.INVALID_WIFI_OBJ .. [["}]]
	end

	local r, code = _M.MasterCall( "NETWORK_MANAGER.DeactivateDevice", obj )
	if r and code then
		return [[{ "Result":200, "Msg":"]] .. _M.langs.MSG_DCON_DONE .. [["}]]
	else
		return [[{ "Result":400, "Msg":"]] .. _M.langs.ERRMSG_DCON_FAIL .. [["}]]
	end

end

-------------------------------------------------------

local function asyncWifiScan( obj )
	_M.MasterCall( "WIFI.WifiScan", obj )
	local r,wifiList = _M.MasterCall( "WIFI.WifiScanList", obj )
	local counters = 0
	local k,v
	local objPath = _M.utils.url_encode(obj)
	local returns = ""
	
	if r and wifiList then
		for k,v in pairs(wifiList) do
			local sigClass = "sig_3"
			local title
			local opera
			counters = counters + 1
			if v.Strength >= 80 then
				sigClass = "sig_5"
			elseif v.Strength >= 60 then
				sigClass = "sig_4"
			elseif v.Strength >= 40 then
				sigClass = "sig_3"
			elseif v.Strength >= 20 then
				sigClass = "sig_2"
			else
				sigClass = "sig_1"
			end

			title = v.Ssid

			if not v.security or v.security["OPEN"] then
				title = title .. " <img src='/images/nosafe.png' alt='" .. _M.langs.NOSAFE .. "'></img>"
			end

			opera = [[<a href="#" onclick="ShowPage('wifi_set.lp?obj=]] .. objPath .. [[&newcon=1&ap=]] .. _M.utils.url_encode(v.object) .. [['); return false;">]] .. _M.langs.CONNECT .. [[</a>]]
			returns = returns .. string.format( [[<div class="wifi_scan %s">
				<div class="title">%s</div>
				<div class="opera">%s</div>
			</div>]], sigClass, title, opera )
		end --For
	end --If scanlist

	if counters <= 0 then
		returns = "<div><i>" .. _M.langs.NO_WIFI_SCANLIST .. "</i></div>"
	end
	
	return returns
end --of asyncWifiScan

--------------------------------------------------------------

local function SaveSet( args )
	if not args or type(args) ~= "table" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end
	local r,x = _M.MasterCall( "BUS.Set", args )
	if r and x then 
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [[","Refresh":1}]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [[","Refresh":1}]]
	end
end

---------------------------------------------------------------------

_M.AjaxExport ("DeleteConnection", DeleteConnection, "json") 
_M.AjaxExport ("ConnectConnection", ConnectConnection, "json" )
_M.AjaxExport ("deactiveConnection", deactiveConnection, "json" )
_M.AjaxExport ("SaveSet", SaveSet, "json" )
_M.AjaxExport ("asyncWifiScan", asyncWifiScan, "html" )

---------------------------------------------------------------------
-- Check the WIFI arguments
---------------------------------------------------------------------

function _M.Check()
	local obj, objPath

	if not cgilua.QUERY or not cgilua.QUERY.obj then
		cgilua.put( "<h2>" .. _M.langs.INVALID_WIFI_OBJ .. "</h2>" )
		return nil
	end

	obj = cgilua.QUERY.obj
	objPath = _M.utils.url_encode(obj)

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", obj, {GENERIC=true} )	--No options, so only get base informations.
	if not r or type(devInfo) ~= "table" then
		cgilua.put( "<h2>" .. _M.langs.INTERNAL_ERROR .. "</h2>" )
		if devInfo and type(devInfo) == "string" then
			cgilua.put( "<p>" .. devInfo .. "</p>" )
		end
		return nil
	end

	local fname
	r, fname = _M.MasterCall( "DEVICE_NAMES.GetDeviceName", devInfo.Name, _M.pageLang )
		
	local activeUuid = ""
	if devInfo.ActiveConnection and devInfo.ActiveConnection ~= "" and devInfo.ActiveConnection ~= "/" then
		local activeConnInfo
		r, activeConnInfo = _M.MasterCall( "NETWORK_MANAGER.GetActivateConnectionDetails", devInfo.ActiveConnection )
		if r and activeConnInfo then
			activeUuid = activeConnInfo[devInfo.ActiveConnection].Uuid
		end
	end

	-- The results.
	local rTable = {
		obj = obj,
		objPath = objPath,
		fname = fname,
		activeUuid = activeUuid,
		
		connections = {}
	}

	if type(devInfo.AvailableConnections) == "table" then
		local connDetails
		--r, connDetails = _M.MasterCall( "NETWORK_MANAGER.GetConnectionDetails", devInfo.AvailableConnections )
		r, connDetails = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", { interface=devInfo.Name } )
		if r and connDetails then
			local k,v 
			for k,v in pairs(connDetails) do
				local title
				local secinfo
				local addrResolv
				local isCurrent = false
				
				title = v["connection"]["id"]
				if not title or title == "" then
					title = v["802-11-wireless"]["ssid"] or _M.langs.UNKNOWN
				end

				if v["connection"]["uuid"] and v["connection"]["uuid"] == activeUuid then
					isCurrent = true
				end

				if v["802-11-wireless-security"] then
					local kmgmt = v["802-11-wireless-security"]["key-mgmt"] or "none"
					if kmgmt == "none" then
						secinfo = _M.langs.WIFI_WEP
					elseif kmgmt == "ieee8021x" then
						secinfo = _M.langs.WIFI_WEP
					elseif kmgmt == "wpa-none" then
						secinfo = _M.langs.WIFI_ADHOC_PSK
					elseif kmgmt == "wpa-psk" then
						secinfo = _M.langs.WIFI_WPA_PSK
					elseif kmgmt == "wpa-eap" then
						secinfo = _M.langs.WIFI_WPA_EAP
					else
						secinfo = _M.langs.WIFI_UNKNOWN_SEC
					end
				else
					secinfo = _M.langs.WIFI_OPEN -- _M.langs.INVALID_WIFI_SEC
				end

				if v["ipv4"] and v["ipv4"]["method"] then
					if v["ipv4"]["method"] == "auto" then
						addrResolv = _M.langs.DHCP_ADDRESS_SOLV
					else
						addrResolv = _M.langs.MANUAL_ADDRESS_SOLV
					end
				end

				local kPath = _M.utils.url_encode(k)
				
				table.insert( rTable.connections, {
					title = title,
					secinfo = secinfo,
					addrResolv = addrResolv,
					isCurrent = isCurrent,
					connObj = k,
					connObjPath = kPath
				})
			end	--For
		end --If get details
	end -- If Avaliable connections got.
	
	return rTable
end

---------------------------------------------------------------------

return _M
