--Controller of wifi_set
local _M = require( "controller.generic" )()
local nu = require "kiloview.netUtil"
local cgilua = require "cgilua"

---------- AJAX Call -----------

local function SaveSet( args )
	if not args or type(args) ~= "table" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	if not args.obj then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", args.obj, {WIFI = true} )	--Get WIFI informations, so I can get MAC.
	if not r or not devInfo then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.GET_DEVINFO_ERROR .. [["}]]
	end

	local devName = devInfo.Name

	local connSettings = {
		["connection"] = {},
		["802-11-wireless"] = {},
		["ipv4"] = {}
	}

	connSettings["connection"]["type"] = "802-11-wireless"
	connSettings["connection"]["autoconnect"] = (args.autoConn == "y")

	if args.newConn and args.newConn == "y" then	--New a connection.
		if args.specAp and type(args.specAp) == "string" and args.specAp ~= "" and args.specAp ~="/" then	--Specical AP connection
			local r, apInfo = _M.MasterCall( "NETWORK_MANAGER.GetWifiAPInfo", args.specAp )
			if not r or not apInfo then
				return [[ {"Result":400, "Msg":"]] .. _M.langs.INVALID_AP .. [["}]]
			end

			connSettings["connection"]["id"] = apInfo.Ssid
			connSettings["802-11-wireless"]["ssid"] = apInfo.Ssid_Array

			if args.bssid and args.bssid ~= "" then
				connSettings["802-11-wireless"]["bssid"] = nu.MACToArray( args.bssid )
			end

		else --Manual set AP
			if not args.ssid or args.ssid == "" then
				return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_SSID .. [["}]]
			end

			connSettings["connection"]["id"] = args.ssid
			connSettings["802-11-wireless"]["ssid"] = args.ssid

			if args.bssid and args.bssid ~= "" then
				connSettings["802-11-wireless"]["bssid"] = nu.MACToArray( args.bssid )
			end
		end

	else --Change a connection
		if args.uuid and args.uuid ~= "" then
			connSettings["connection"]["uuid"] = args.uuid
			local r, detail = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", {uuid=args.uuid} )
			if r and detail then
				local k,v
				local found
				for k,v in pairs(detail) do
					found = v
					break
				end
				
				if found then
					connSettings["connection"]["id"] = found["connection"]["id"]
					connSettings["802-11-wireless"]["ssid"] = found["802-11-wireless"]["ssid"]
					if args.bssid and args.bssid ~= "" then
						connSettings["802-11-wireless"]["bssid"] = nu.MACToArray( args.bssid )
					end
				end
			end
		end
	end

	connSettings["connection"]["interface-name"] = devName

	local psk = args.psk
	if psk == "~dont~change" then
		psk = nil
	end

	if args.secrets then
		if args.secrets == "none-none" then
			connSettings["802-11-wireless-security"] = nil	--No security setting
		elseif args.secrets == "none-open" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "none",
				["auth-alg"] = "open",
				["wep-key-type"] = 1,
				["wep-key0"] = psk
			}
		elseif args.secrets == "none-share" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "none",
				["auth-alg"] = "shared",
				["wep-key-type"] = 1,
				["wep-key0"] = psk
			}
		elseif args.secrets == "none-open-128" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "none",
				["auth-alg"] = "open",
				["wep-key-type"] = 2,
				["wep-key0"] = psk
			}
		elseif args.secrets == "none-share-128" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "none",
				["auth-alg"] = "shared",
				["wep-key-type"] = 2,
				["wep-key0"] = psk
			}
		--[[
		elseif args.secrets == "ieee8021x" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "ieee8021x",
				["auth-alg"] = "shared",
				["wep-key-type"] = 1,
				["wep-key0"] = psk
			}
		]]
		elseif args.secrets == "wpa-none" then
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "wpa-none",
				["psk"] = psk,
				["pairwise"] = { "none" }
			}
		else
			connSettings["802-11-wireless-security"] = {
				["key-mgmt"] = "wpa-psk",
				["psk"] = psk,
				["pairwise"] = { "ccmp", "tkip" }
			}
		end
	end

	if args.dynamic and args.dynamic == "y" then	--DHCP
		connSettings["ipv4"]["method"] = "auto"
	else
		connSettings["ipv4"]["method"] = "manual"
		if args.ip and args.ip ~= "" then
			connSettings["ipv4"]["address-data"] = {{address=args.ip, prefix=nu.ip4MaskToPrefix(args.mask)}}
		end

		connSettings["ipv4"]["gateway"] = args.gw or ""
		if args.dns1 and args.dns1 == "" then
			args.dns1 = nil
		end

		if args.dns2 and args.dns2 == "" then
			args.dns2 = nil
		end

		connSettings["ipv4"]["dns"] = { args.dns1 or nil, args.dns2 or nil }
		if #connSettings["ipv4"]["dns"] < 1 then
			connSettings["ipv4"]["dns"] = nil
		end

	end


	if args.newConn and args.newConn == "y" then
		local r, o = _M.MasterCall( "NETWORK_MANAGER.AddConnection", connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	else
		local r, o = _M.MasterCall( "NETWORK_MANAGER.ChangeConnectionDetails", args.specConn, connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	end
end

--------------------------------
_M.AjaxExport ("SaveSet", SaveSet, "json") 
--------------------------------

-- Check the request arguments

function _M.Check()
	local obj, objPath
	local newOne = false
	local specConn
	local specAp

	if not cgilua.QUERY or not cgilua.QUERY.obj then
		cgilua.put( "<h2>" .. _M.langs.INVALID_WIFISET_PARAM .. "</h2>" )
		return nil
	else
		obj = cgilua.QUERY.obj

		if cgilua.QUERY.newcon then	--New connection
			newOne = true
			if cgilua.QUERY.ap then --Specified AP
				specAp = cgilua.QUERY.ap
			end
		else
			if cgilua.QUERY.con then --Specified connection (Modify)
				specConn = cgilua.QUERY.con
			else
				cgilua.put( "<h2>" .. _M.langs.INVALID_WIFICONN_PARAM .. "</h2>" )
				return nil
			end
		end
	end

	objPath = _M.utils.url_encode(obj)

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", obj, {WIFI = true} )	--Get WIFI informations, so I can get MAC.
	if not r or not devInfo then
		cgilua.put( [[<h2>]] .. _M.langs.GET_DEVINFO_ERROR .. [[<hr class="title_line" /></h2>
						<p>]] .. _M.langs.GET_DEVINFO_ERROR_DETAIL .. [[</p>
						<a href="#" onclick="ShowPage('netlist.lp'); return false;">]] .. _M.langs.RETURN_TO_NETLIST .. [[</a> ]] )
		return nil
	end

	local devMac = devInfo.WIFI.HwAddress	--Identify the WIFI device.
	local ssid = ""
	local bssid = ""
	local dhcp = true
	local ip4addr = ""
	local ip4mask = ""
	local ip4gateway = ""
	local ip4dns1 = ""
	local ip4dns2 = ""
	local psk = ""
	local autoconn = true
	local secrets = "wpa-psk"
	local uuid = ""

	if newOne and specAp then	--Get AP information, and put to parameters.
		local r, apInfo = _M.MasterCall( "NETWORK_MANAGER.GetWifiAPInfo", specAp )
		if not r or not apInfo then
			cgilua.put( [[<h2>]] .. _M.langs.INVALID_AP .. [[<hr class="title_line" /></h2>
							<p>]] .. _M.langs.INVALID_AP_DETAIL .. [[</p>
							<a href="#" onclick="ShowPage('wifi.lp?obj=]] .. objPath .. [['); return false;">]] .. _M.langs.RETURN_TO_WIFI_MAN .. [[</a> ]] )
			return nil
		end

		ssid = apInfo.Ssid

		if not apInfo.security then
			secrets = "none-none"
		elseif apInfo.security["PSK"] then
			if apInfo.Mode == "ADHOC" then
				secrets = "wpa-none"
			else
				secrets = "wpa-psk"
			end
		elseif apInfo.security["8021X"] then
			secrets = "ieee8021x"
		elseif apInfo.security["WEP"] then
			secrets = "none-share"
		else
			secrets = "none-none"
		end

	elseif specConn then	--Modify the connection

		local r, connInfo = _M.MasterCall( "NETWORK_MANAGER.GetConnectionDetails", specConn )
		if not r or not connInfo or not connInfo[specConn] then
			cgilua.put( [[<h2>]] .. _M.langs.INVALID_SAVED_CONN .. [[<hr class="title_line" /></h2>
							<p>]] .. _M.langs.INVALID_SAVED_CONN_DETAIL .. [[</p>
							<a href="#" onclick="ShowPage('wifi.lp?obj=]] .. objPath .. [['); return false;">]] .. _M.langs.RETURN_TO_WIFI_MAN .. [[</a> ]] )
			return nil
		end

		connInfo = connInfo[specConn]

		ssid = connInfo["802-11-wireless"]["ssid"]
		if type(ssid) == "table" then	--Bytes array
			ssid = string.char( table.unpack(ssid) )
		end

		bssid = connInfo["802-11-wireless"]["bssid"]
		if type(bssid) == "table" and #bssid >= 6 then
			bssid = string.format( "%02X:%02X:%02X:%02X:%02X:%02X", bssid[1],bssid[2],bssid[3],bssid[4],bssid[5],bssid[6] )
		else
			bssid = ""
		end

		local kmgt = connInfo["802-11-wireless-security"] and connInfo["802-11-wireless-security"]["key-mgmt"] or "none"
		local auth_alg = connInfo["802-11-wireless-security"] and connInfo["802-11-wireless-security"]["auth-alg"] or ""
		local wep_key_type = connInfo["802-11-wireless-security"] and connInfo["802-11-wireless-security"]["wep-key-type"] or 0

		if kmgt == "wpa-psk" then
			secrets = "wpa-psk"
		elseif kmgt == "wpa-none" then
			secrets = "wpa-none"
		elseif kmgt == "ieee8021x" then
			secrets = "ieee8021x"
		elseif kmgt == "none" then
			if auth_alg == "shared" then
				secrets = "none-share"
			elseif auth_alg == "open" then
				secrets = "none-open"
			else
				secrets = "none-none"
			end

			if wep_key_type == 2 then
				secrets = secrets .. "-128"
			end
		end

		psk = "~dont~change"	--Don't change default.

		uuid = connInfo["connection"]["uuid"]
		if type(connInfo["connection"]["autoconnect"]) ~= "nil" then
			--remotedostring("print(...)", "XXX", type(connInfo["connection"]["autoconnect"]) );
			autoconn = connInfo["connection"]["autoconnect"]
		else
			autoconn = true
		end
		--remotedostring("print(...)", 1, autoconn )

		if autoconn and type(autoconn) ~= "boolean" then
			autoconn = (autoconn ~= 0) and (autoconn ~= "false") and (autoconn ~= "no" )
		end

		--remotedostring("print(...)", 2, autoconn )

		dhcp = connInfo["ipv4"]["method"] and connInfo["ipv4"]["method"] == "auto" or false

		if connInfo["ipv4"]["address-data"] and #connInfo["ipv4"]["address-data"] > 0 then
			ip4addr = connInfo["ipv4"]["address-data"][1]["address"]
			local pfx = connInfo["ipv4"]["address-data"][1]["prefix"]
			ip4mask = nu.ip4PrefixToMask(pfx)
			ip4gateway = connInfo["ipv4"]["gateway"] or ""

			if connInfo["ipv4"]["dns"] and #connInfo["ipv4"]["dns"] > 0 then
				ip4dns1 = connInfo["ipv4"]["dns"][1] or ""
				ip4dns2 = connInfo["ipv4"]["dns"][2] or ""
			end
		end
	end
	
	return {
		obj = obj,
		objPath = objPath,
		newOne = newOne,
		specConn = specConn,
		specAp = specAp,
		devMac = devMac,
		ssid = ssid,
		bssid = bssid,
		dhcp = dhcp,
		ip4addr = ip4addr,
		ip4mask = ip4mask,
		ip4gateway = ip4gateway,
		ip4dns1 = ip4dns1,
		ip4dns2 = ip4dns2,
		psk = psk,
		autoconn = autoconn,
		secrets = secrets,
		uuid = uuid
	}
end

---------------------------------

return _M

