--Controller of wired
local _M = require( "controller.generic" )()
local nu = require "kiloview.netUtil"
local cgilua = require "cgilua"

---------- AJAX Call -----------

local function SaveSet( args )

	if not args or type(args) ~= "table" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	if not args.obj then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", args.obj, {WIRE = true} )	--Get WIRE informations, so I can get MAC.
	if not r or not devInfo then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.GET_DEVINFO_ERROR .. [["}]]
	end

	local devName = devInfo.Name

	local connSettings = {
		["connection"] = {},
		["802-3-ethernet"] = {},
		["ipv4"] = {}
	}

	connSettings["connection"]["type"] = "802-3-ethernet"
	connSettings["connection"]["autoconnect"] = true	--Force auto connection.

	if args.uuid and args.uuid ~= "" then
		connSettings["connection"]["uuid"] = args.uuid
		local r, detail = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", {uuid=args.uuid} )
		if r and detail then
			local k,v
			local found
			for k,v in pairs(detail) do
				found = v
				break
			end
			
			if found then
				connSettings["connection"]["id"] = found["connection"]["id"]
			end
		end
	end

	if not connSettings["connection"]["id"] then
		connSettings["connection"]["id"] = devName
	end
	
	if args.cloneMac then
		if args.cloneMac ~= "" then
			connSettings["802-3-ethernet"]["cloned-mac-address"] = nu.MACToArray(args.cloneMac)
		--else
		--	connSettings["802-3-ethernet"]["cloned-mac-address"] = nu.MACToArray(devInfo.WIRE.PermHwAddress)
		end
	end

	connSettings["connection"]["interface-name"] = devName

	if args.dynamic and args.dynamic == "y" then	--DHCP
		connSettings["ipv4"]["method"] = "auto"
	else
		connSettings["ipv4"]["method"] = "manual"
		if args.ip and args.ip ~= "" then
			connSettings["ipv4"]["address-data"] = {{address=args.ip, prefix=nu.ip4MaskToPrefix(args.mask)}}
		end

		connSettings["ipv4"]["gateway"] = args.gw or ""

		if args.dns1 and args.dns1 == "" then
			args.dns1 = nil
		end

		if args.dns2 and args.dns2 == "" then
			args.dns2 = nil
		end

		connSettings["ipv4"]["dns"] = { args.dns1 or nil, args.dns2 or nil }
		if #connSettings["ipv4"]["dns"] < 1 then
			connSettings["ipv4"]["dns"] = nil
		end
	end


	if args.newConn and args.newConn == "y" then
		local r, o = _M.MasterCall( "NETWORK_MANAGER.AddConnection", connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	else
		local r, o = _M.MasterCall( "NETWORK_MANAGER.ChangeConnectionDetails", args.specConn, connSettings )
		if r and o then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_SAVE_DONE .. [["}]]
		else
			return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_SAVE_FAIL .. [["}]]
		end
	end
end

--------------------------------

local function ApplySet(obj)
	if not obj then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_POST_ARGS .. [["}]]
	end

	local r,code = _M.MasterCall( "NETWORK_MANAGER.ConnectDevice", obj )
	if r and code then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_APPLY_DONE .. [["}]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_APPLY_FAIL .. [["}]]
	end
end

--------------------------------

_M.AjaxExport ("SaveSet", SaveSet, "json") 
_M.AjaxExport ("ApplySet", ApplySet, "json") 

--------------------------------
-- Check the request arguments
--------------------------------
function _M.Check()
	local obj
	local newOne = false
	local specConn

	if not cgilua.QUERY or not cgilua.QUERY.obj then
		cgilua.put( "<h2>" .. _M.langs.INVALID_WIREDSET_PARAM .. "</h2>" )
		return nil
	end

	obj = cgilua.QUERY.obj

	local r, devInfo = _M.MasterCall( "NETWORK_MANAGER.GetDeviceInfo", obj, {WIRE = true} )	--Get WIRE informations, so I can get MAC.
	if not r or not devInfo then
		cgilua.put( [[<h2>]] .. _M.langs.GET_DEVINFO_ERROR .. [[<hr class="title_line" /></h2>
						<p>]] .. _M.langs.GET_DEVINFO_ERROR_DETAIL .. [[</p>
						<a href="#" onclick="ShowPage('netlist.lp'); return false;">]] .. _M.langs.RETURN_TO_NETLIST .. [[</a> ]] )
		return nil
	end

	if cgilua.QUERY.newcon then	--New connection
		newOne = true
	end

	if not newOne then
		if devInfo.ActiveConnection and devInfo.ActiveConnection ~= "" and devInfo.ActiveConnection ~= "/" then
			local r, ac = _M.MasterCall( "NETWORK_MANAGER.GetActivateConnectionDetails", devInfo.ActiveConnection )
			if r and ac then
				specConn = ac[devInfo.ActiveConnection].Connection
			end
		end

		if not specConn then
			specConn = devInfo.AvailableConnections[1]
			if not specConn then
				local r, allConns = _M.MasterCall( "NETWORK_MANAGER.ListConnectionAndGetDetails", { interface=devInfo.Name } )
				if r and allConns then
					local k,v
					for k,v in pairs(allConns) do
						specConn = k	--Get first one.
						break
					end
				end

				if not specConn then
					newOne = true
				end
			end
		end
	end

	local devMac = devInfo.WIRE.PermHwAddress	--Identify the device.
	local dhcp = true
	local ip4addr = ""
	local ip4mask = ""
	local ip4gateway = ""
	local ip4dns1 = ""
	local ip4dns2 = ""
	local uuid = ""
	local cloneMac = ""

	if specConn then
		local r, connInfo = _M.MasterCall( "NETWORK_MANAGER.GetConnectionDetails", specConn )
		if not r or not connInfo or not connInfo[specConn] then
			cgilua.put( [[<h2>]] .. _M.langs.INVALID_SAVED_CONN .. [[<hr class="title_line" /></h2>
							<p>]] .. _M.langs.INVALID_SAVED_CONN_DETAIL .. [[</p>
							<a href="#" onclick="ShowPage('netlist.lp'); return false;">]] .. _M.langs.RETURN_TO_NETLIST .. [[</a> ]] )
			return
		end

		connInfo = connInfo[specConn]

		uuid = connInfo["connection"]["uuid"]
		dhcp = connInfo["ipv4"]["method"] and connInfo["ipv4"]["method"] == "auto" or false
		cloneMac = connInfo["802-3-ethernet"]["cloned-mac-address"] or connInfo["802-3-ethernet"]["mac-address"]

		if cloneMac and #cloneMac >= 6 then
			cloneMac = string.format( "%02X:%02X:%02X:%02X:%02X:%02X", cloneMac[1],cloneMac[2],cloneMac[3],cloneMac[4],cloneMac[5],cloneMac[6] )
		else
			cloneMac = ""
		end

		if connInfo["ipv4"]["address-data"] and #connInfo["ipv4"]["address-data"] > 0 then
			ip4addr = connInfo["ipv4"]["address-data"][1]["address"]
			local pfx = connInfo["ipv4"]["address-data"][1]["prefix"]
			ip4mask = nu.ip4PrefixToMask(pfx)
			ip4gateway = connInfo["ipv4"]["gateway"] or ""

			if connInfo["ipv4"]["dns"] and #connInfo["ipv4"]["dns"] > 0 then
				ip4dns1 = connInfo["ipv4"]["dns"][1] or ""
				ip4dns2 = connInfo["ipv4"]["dns"][2] or ""
			end
		end
	end
	
	return {
		obj = obj,
		newOne = newOne,
		specConn = specConn,
		devMac = devMac,
		dhcp = dhcp,
		ip4addr = ip4addr,
		ip4mask = ip4mask,
		ip4gateway = ip4gateway,
		ip4dns1 = ip4dns1,
		ip4dns2 = ip4dns2,
		uuid = uuid,
		cloneMac = cloneMac
	}
	
end

--------------------------------

return _M
