-- The roll file logger.
local _M = {}

function _M.Create(filename, maxHis, compress)
	local path, file = filename:match("(.+)/(.*)$")
	local delm = "/"
	if not path then
		-- Try windows format.
		path, file = filename:match("(.+)\\(.*)$")
		if path then
			delm = "\\"
		else
			file = filename
		end
	end

	local base, ext = file:match("(.+)%.(.*)$")
	if not base then
		base = file
	end

	if not compress or (compress ~= "none" and compress ~= "" ) then
		--TODO: Support more compress format.
		compress = "tar.gz"
	end

	local writer = {
		location = path,
		basename = base,
		delimiter = delm,
		file_ext = ext,
		numberFormat = "-%03d",
		maxCount = maxHis or 3,
		compress = compress,
		flushLimit = 0,
		maxFileSize = 1024*1024,
		currentFile = nil,
		current_size = 0,
		current_buffered = 0
	}

	function writer:get_compress_ext()
		if self.compress == "" or self.compress == "none"  then
			return (self.file_ext and ("." .. self.file_ext) or "")
		else
			return "." .. self.compress
		end
	end

	function writer:compress_file(f,to)
		if self.compress == "" or self.compress == "none" then
			os.rename(f, to)	
		else
			os.execute( "/bin/tar czf " .. to .. " " .. f )
			os.remove(f)
		end
	end

	function writer:init(forNew)
		local fname
		if forNew then
			local cnt, prev_fname, fobj
			for cnt = self.maxCount, 1, -1 do
				fname = self.basename .. string.format(self.numberFormat, cnt) .. self:get_compress_ext()
				if self.location then
					fname = self.location .. self.delimiter .. fname
				end
				fobj = io.open(fname, "rb")
				if fobj then
					fobj:close()
					if not prev_fname then --Remove last file.
						os.remove(fname)
					else
						os.rename(fname, prev_fname)
					end
				end
				prev_fname = fname
			end

			if not prev_fname then
				prev_fname = self.basename .. string.format(self.numberFormat, 1) .. self:get_compress_ext()
				if self.location then
					prev_fname = self.location .. self.delimiter .. prev_fname
				end
			end

			fname = self.basename .. (self.file_ext and ("." .. self.file_ext) or "")
			if self.location then
				fname = self.location .. self.delimiter .. fname
			end

			self:compress_file(fname, prev_fname)
		else
			fname = self.basename .. (self.file_ext and ("." .. self.file_ext) or "")
			if self.location then
				fname = self.location .. self.delimiter .. fname
			end
		end

		self.currentFile = io.open(fname, "a+") --append
		if not self.currentFile then
			return false, "File to open file for writting"
		else
			self.currentFile:write("\n\n---- cut line (" .. os.date("%Y-%m-%d %H:%M:%S") .. ") ----\n\n")
			self.current_size = self.currentFile:seek('end')
			self.current_buffered = 0
			return true
		end
	end

	function writer:close()
		if self.currentFile then
			self.currentFile:close()
			self.currentFile = nil
		end
	end

	function writer:write(data)
		if not self.currentFile then
			if not self:init() then
				return -1
			end
		end
		local size = #data
		self.currentFile:write(data)
		self.current_size = self.current_size + size
		self.current_buffered = self.current_buffered + size
		if self.current_size >= self.maxFileSize then
			self.currentFile:close()
			self.currentFile = nil
			self:init(true)
		elseif self.current_buffered >= self.flushLimit then
			self.currentFile:flush()
			self.current_buffered = 0
		end
	end

	return writer
end

return _M
