local _dbg = require "coaf.debug"
local client = require "coaf.client"

local _M = {}

-----------------------------------------
local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "HOSTAP_CTRL", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "HOSTAP_CTRL", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end
-----------------------------------------

function _M.Init()
	return true
end

function _M.GetHostAPInfo()
	local hostapm_proxy = GetCoAFProxy( "/raw/HOSTAP", "nm_daemon" )
	if not hostapm_proxy then
		return nil, "Not initialized"
	end
	
	local gets =  { hostapm_proxy._P( "Device", "Enable", "Started", "HostAP", "HideSSID", "ShareKey", "ShareDevice", "SharedAddress", "ShareAddressRange", "WifiChannel", "ShareIPRange" ) }
	hostapm_proxy:Destroy()

	--TODO: Old reference.
	--	Device = _M.hostapm_proxy.p.Device,
	--	Enable = _M.hostapm_proxy.p.Enable,
	--	Started = _M.hostapm_proxy.p.Started,
	--	HostAP = _M.hostapm_proxy.p.HostAP,
	--	HideSSID = _M.hostapm_proxy.p.HideSSID,
	--	ShareKey = _M.hostapm_proxy.p.ShareKey,
	--	ShareDevice = _M.hostapm_proxy.p.ShareDevice,
	--	SharedAddress = _M.hostapm_proxy.p.SharedAddress,
	--	ShareAddressRange = _M.hostapm_proxy.p.ShareAddressRange,
	--	WifiChannel = _M.hostapm_proxy.p.WifiChannel,

	local result = {
		Device = gets[1],
		Enable = gets[2],
		Started = gets[3],
		HostAP = gets[4],
		HideSSID = gets[5],
		ShareKey = gets[6],
		ShareDevice = gets[7],
		SharedAddress = gets[8],
		ShareAddressRange = gets[9],
		WifiChannel = gets[10],
		ShareIPRange = gets[11],
}

	--_dbg.Info( "HOSTAP_CTRL", result.Device, result.Enable, result.Started, result.ShareAddressRange )
	return result
end


function _M.StartHostAP(device, deviceType, shareDevice, hostAP, shareKey, ipRange, ipPrefix, wifiChannel, hideSsid)
	--print(device, deviceType, shareDevice, hostAP, shareKey, ipRange, ipPrefix)
	local hostapm_proxy = GetCoAFProxy( "/raw/HOSTAP", "nm_daemon" )
	if not hostapm_proxy then
		return false, "Not initialized"
	end

	local r,code = hostapm_proxy:StartShare(device, deviceType, shareDevice, hostAP, shareKey, ipRange, ipPrefix, wifiChannel or 6, hideSsid )
	hostapm_proxy:Destroy()
	if r and code then
		return true
	else
		--print(code)
		return false, code
	end
end

function _M.StopHostAP()
	local hostapm_proxy = GetCoAFProxy( "/raw/HOSTAP", "nm_daemon" )
	if not hostapm_proxy then
		return false, "Not initialized"
	end

	local r,code = hostapm_proxy:StopShare()
	hostapm_proxy:Destroy()
	if r and code then
		return true
	else
		return false, code
	end
end

function _M.UpdateModems(modems_cfg)
	if modems_cfg then
		local file = io.open( "/data/configs/option-modems.conf", "w" )
		if file then
			file:write( modems_cfg )   
			file:close()           
		end
	end

	local hostapm_proxy = GetCoAFProxy( "/raw/HOSTAP", "nm_daemon" )
	if not hostapm_proxy then
		return false, "Not initialized"
	end

	local r,ret = hostapm_proxy:Control( "update-alter-modems", {} )
	hostapm_proxy:Destroy()
	if r and ret then
		return true
	else
		return false, ret
	end
end

return _M

