--The Sony Visca protocol

local function CalcPanSpeed(spd)
	if not spd or spd < 0 then
		return 0x0a
	end

	spd = spd * 0x18
	if spd < 1 then
		return 1
	elseif spd > 0x18 then
		return 0x18
	else
		return math.floor(spd)
	end
end

local function CalcTiltSpeed(spd)
	if not spd or spd < 0 then
		return 0x0a
	end

	spd = spd * 0x12
	if spd < 1 then
		return 1
	elseif spd > 0x12 then
		return 0x12
	else
		return math.floor(spd)
	end
end


local function RestrictAddr(addr)
	if not addr or addr <= 0 then
		return 1
	elseif addr > 0x0f then
		return 0x0f
	else
		return addr
	end
end

local _M = {

name = "Sony Visca",

AckFilter = nil,	--TODO: Process the ack messages, but now ignore it.
maxAddresses = 8,

supports = {
	["up"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(spd), 0x03, 0x01, 0xff}}
	end,

	["down"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(spd), 0x03, 0x02, 0xff}}
	end,
	
	["left"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(spd), 0x0a, 0x01, 0x03, 0xff}}
	end,

	["right"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(spd), 0x0a, 0x02, 0x03, 0xff}}
	end,

	["left-up"] = function(addr,pan_spd,tilt_spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(pan_spd), 0x0a, 0x01, 0x03, 0xff},
				20, --Wait ms
				{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(tilt_spd), 0x03, 0x01, 0xff}}
	end,

	["right-up"] = function(addr,pan_spd,tilt_spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(pan_spd), 0x0a, 0x02, 0x03, 0xff},
				20, --Wait ms
				{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(tilt_spd), 0x03, 0x01, 0xff}}
	end,

	["left-down"] = function(addr,pan_spd,tilt_spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(pan_spd), 0x0a, 0x01, 0x03, 0xff},
				20, --Wait ms
				{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(tilt_spd), 0x03, 0x02, 0xff}}
	end,

	["right-down"] = function(addr,pan_spd,tilt_spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, CalcPanSpeed(pan_spd), 0x0a, 0x02, 0x03, 0xff},
				20, --Wait ms
				{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x0a, CalcTiltSpeed(tilt_spd), 0x03, 0x02, 0xff}}
	end,

	["home"] = function(addr)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x04, 0xff}}
	end,

	["stop-move"] = function(addr)
		return {{0x80+RestrictAddr(addr), 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x03, 0xff}}
	end,

	["zoom-in"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x20 + math.floor( spd*8 ), 0xff }
			else
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x27, 0xff }
			end
		end

		local buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x02, 0xff } 
		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["zoom-out"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x30 + math.floor( spd*8 ), 0xff }
			else
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x37, 0xff }
			end
		end

		local buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x03, 0xff } 
		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["stop-zoom"] = function(addr)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x07, 0x00, 0xff}}
	end,

	["focus-near"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x30 + math.floor( spd*8 ), 0xff }
			else
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x37, 0xff }
			end
		end

		local buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x03, 0xff } 
		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["focus-far"] = function(addr,spd)
		local spd_buf = nil
		if spd then
			if spd >= 0 and spd < 1 then
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x20 + math.floor( spd*8 ), 0xff }
			else
				spd_buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x27, 0xff }
			end
		end

		local buf = { 0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x02, 0xff } 
		if spd_buf then
			return {spd_buf, buf}
		else
			return {buf}
		end
	end,

	["stop-focus"] = function(addr)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x08, 0x00, 0xff}}
	end,

	["iris-up"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x0b, 0x02, 0xff}}
	end,

	["iris-down"] = function(addr,spd)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x0b, 0x03, 0xff}}
	end,

	["stop-iris"] = function(addr)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x0b, 0x00, 0xff}}
	end,

	["stop-all"] = function(addr)
		addr = RestrictAddr(addr)
		return {
			{0x80+addr, 0x01, 0x06, 0x01, 0x00, 0x00, 0x03, 0x03, 0xff},
			{0x80+addr, 0x01, 0x04, 0x07, 0x00, 0xff},
			{0x80+addr, 0x01, 0x04, 0x08, 0x00, 0xff},
			--{0x80+addr, 0x01, 0x04, 0x0b, 0x00, 0xff}
		}
	end,

	["save-preset"] = function(addr,id)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x3f, 0x01, id%256, 0xff}}
	end,

	["remove-preset"] = function(addr,id)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x3f, 0x00, id%256, 0xff}}
	end,

	["load-preset"] = function(addr,id)
		return {{0x80+RestrictAddr(addr), 0x01, 0x04, 0x3f, 0x02, id%256, 0xff}}
	end
}

}

return _M

